package cn.fw.dalaran.common.domain.enums;

import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 角色信息枚举
 */
@Getter
@AllArgsConstructor
public enum RoleMsgEnum {
    XSGW("XSGW", "销售顾问"),
    XSZG("XSZG", "销售一级管理"),
    XSJL("XSJL", "销售二级管理"),
    XSZJL("XSZJL", "销售三级管理"),
    XSSJGL("XSSJGL", "销售四级管理"),
    SCSJ("SCSJ", "试乘试驾"),
    MKT_HDCZ("MKT-HDCZ", "市场活动操作"),
    RSYJGL("MANAGER-PERSONNEL-1", "人事一级管理"),
    ;

    private final String code;// 角色编码
    private final String desc;// 角色描述

    /**
     * 根据code码获取枚举对象
     *
     * @param code code码
     * @return 对应枚举对象
     */
    public static RoleMsgEnum codeOf(String code) {
        return Arrays.stream(RoleMsgEnum.values())
                .filter(e -> e.getCode().equals(code))
                .findFirst()
                .orElse(null)
                ;
    }

    /**
     * 获取所有销售管理枚举
     *
     * @return 所有销售管理枚举
     */
    public static List<RoleMsgEnum> getSaleManageList() {
        return new ArrayList<>(Arrays.asList(XSZG, XSJL, XSZJL, XSSJGL));
    }
}
