package cn.fw.dalaran.common.myValidator.checkImpl;


import cn.fw.dalaran.common.myValidator.annotation.AssignValues;

import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import java.math.BigDecimal;
import java.util.HashSet;
import java.util.Set;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2022/9/22 23:21
 * @Description 校验器, 校验double类型的
 */
public class AssignBigDecimalValueConstraintValidator implements ConstraintValidator<AssignValues, BigDecimal> {

    private final Set<BigDecimal> values = new HashSet<>();

    /**
     * 初始化方法
     *
     * @param assignValues 校验注解详细信息
     */
    @Override
    public void initialize(AssignValues assignValues) {
        double[] bigDecimals = assignValues.bigDecimals();// 获取校验注解指定的合法值
        for (double value : bigDecimals) {
            this.values.add(BigDecimal.valueOf(value));// 不能使用构造函数 new BigDecimal(value), 会导致经度丢失
        }
    }

    /**
     * 判断是否校验成功
     *
     * @param value   指定值(待校验值 )
     * @param context
     * @return 是否校验成功
     */
    @Override
    public boolean isValid(BigDecimal value, ConstraintValidatorContext context) {
        return this.values.contains(value);
    }

}
