package cn.fw.dalaran.common.page;

import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @param <T> 泛型
 * @author wmy3969
 * @version 1.0
 * @date 2023/4/21 11:28
 * @Description 自定义PageHelper对象
 */
@Data
@NoArgsConstructor
public class PageHelper<T> {
    /**
     * 数据总条数
     */
    private long total = 0;
    /**
     * 原始数据
     */
    private List<T> data = new ArrayList<>();
    /**
     * 当前页
     */
    private long current = 1;
    /**
     * 分页大小
     */
    private long pageSize = 10;
    /**
     * 数据单位
     */
    private String unit = "条";
    /**
     * 额外信息
     */
    private String extInfo;

    public PageHelper(long current) {
        this.current = current;
    }

    public PageHelper(long current, long pageSize) {
        this.current = current;
        this.pageSize = pageSize;
    }

    /**
     * 获取分页查询起始索引
     *
     * @return 分页查询起始索引
     */
    public long getStartIndex() {
        return (this.getCurrentPage() - 1) * this.pageSize;
    }

    /**
     * 获取分页查询终止索引
     *
     * @return 分页查询终止索引
     */
    public long getEndIndex() {
        return this.getCurrentPage() * this.pageSize;
    }

    /**
     * 是否是第1页
     *
     * @return 是否是第1页
     */
    public boolean isFirstPage() {
        return this.getCurrentPage() <= 1;
    }

    /**
     * 是否是最后1页
     *
     * @return 是否是最后1页
     */
    public boolean isLastPage() {
        return this.getCurrentPage() >= this.getPageCount();
    }

    /**
     * 获取下一页页码
     *
     * @return 下一页页码
     */
    public long getNextPage() {
        return this.isLastPage() ? this.getCurrentPage() : this.getCurrentPage() + 1;
    }

    /**
     * 获取上一页页码
     *
     * @return 上一页页码
     */
    public long getPreviousPage() {
        return this.isFirstPage() ? 1 : this.getCurrentPage() - 1;
    }

    /**
     * 获取当前页码
     *
     * @return 当前页码
     */
    public long getCurrentPage() {
        return this.current <= 0 ? 1 : this.current;
    }

    /**
     * 获取总页数
     *
     * @return 总页数
     */
    public long getPageCount() {
        return Objects.equals(this.total % this.pageSize, 0L) ? this.total / this.pageSize :
                this.total / this.pageSize + 1;
    }
}
