package cn.fw.dalaran.common.utils;

import cn.fw.dalaran.common.page.PageHelper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2023/4/21 11:37
 * @Description 分页处理工具
 */
public class PageUtil {

    private PageUtil() {
        throw new IllegalStateException("工具类禁止实例化!");
    }

    /**
     * IPage => PageHelper
     *
     * @param page IPage对象实例
     * @param <T>  泛型
     * @return PageHelper对象实例
     */
    public static <T> PageHelper<T> page2Helper(IPage<T> page) {
        PageHelper<T> result = new PageHelper<>(page.getCurrent(), page.getSize());
        result.setTotal(page.getTotal());
        result.setData(Optional.ofNullable(page.getRecords())
                .orElse(new ArrayList<>()));
        return result;
    }

    /**
     * PageHelper => IPage
     *
     * @param page PageHelper对象实例
     * @param <T>  泛型
     * @return IPage对象实例
     */
    public static <T> IPage<T> helper2Page(PageHelper<?> page) {
        return new Page<>(page.getCurrent(), page.getPageSize());
    }

    /**
     * db => result(vo)转换
     *
     * @param page   IPage对象
     * @param mapper db => result(vo)转换规则
     * @param <E>    db子类对象
     * @param <R>    result(vo)子类对象
     * @return PageHelper result(vo)子类对象
     */
    public static <E, R> PageHelper<R> toPage(IPage<E> page, Function<? super E, ? extends R> mapper) {
        PageHelper<R> result = new PageHelper<>(page.getCurrent(), page.getSize());
        result.setTotal(page.getTotal());
        result.setData(Optional.ofNullable(page.getRecords())
                .orElse(new ArrayList<>())
                .stream()
                .map(mapper)
                .collect(Collectors.toList()));
        return result;
    }

    /**
     * db => result(vo)转换, 附带排序
     *
     * @param page       IPage对象
     * @param mapper     db => result(vo)转换规则
     * @param comparator 排序规则
     * @param <E>        db子类对象
     * @param <R>        result(vo)子类对象
     * @return PageHelper result(vo)子类对象
     */
    public static <E, R> PageHelper<R> toPage(IPage<E> page, Function<? super E, ? extends R> mapper, Comparator<R> comparator) {
        PageHelper<R> result = new PageHelper<>(page.getCurrent(), page.getSize());
        result.setTotal(page.getTotal());
        result.setData(Optional.ofNullable(page.getRecords())
                .orElse(new ArrayList<>())
                .stream()
                .map(mapper)
                .sorted(comparator)
                .collect(Collectors.toList()));
        return result;
    }
}
