package cn.fw.dalaran.dao;

import cn.fw.dalaran.domain.db.Account;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @author kurisu
 * @date 2021-11-17 14:33
 * @description 账号操作
 */
@Repository
public interface AccountDao extends BaseMapper<Account> {
    /**
     * 修改账户
     *
     * @param param 待修改字段值封装
     */
    boolean alertAccount(@Param("param") Account param);

    /**
     * 根据用户id获取账户
     *
     * @param userId 用户id
     * @return 对应账户
     */
    List<Account> getByUserId(Long userId);

    /**
     * 根据账户号获取账户列表(忽略逻辑删除)
     *
     * @param accountNo 账户号
     * @return 对应账户列表
     */
    List<Account> getByAccountNoIgnoreLogicDel(@Param("accountNo") String accountNo);

    /**
     * 根据id获取账户(忽略逻辑删除)
     *
     * @param accountId 账户id
     * @return 对应账户
     */
    Account getByIdIgnoreLogicDel(Long accountId);

    /**
     * 根据ids获取账户列表(忽略逻辑删除)
     *
     * @param accountIds 账户ids
     * @return 对应账户列表
     */
    List<Account> getByIdsIgnoreLogicDel(@Param("accountIds")List<Long> accountIds);

    /**
     * 获取账户列表(忽略逻辑删除)
     *
     * @return 对应账户列表
     */
    List<Account> getAllIgnoreLogicDel();
}
