package cn.fw.dalaran.dao.score;

import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.vo.account.AccountInfoVo;
import cn.fw.dalaran.domain.vo.config.EvlVo;
import cn.fw.dalaran.domain.vo.config.FansLimitVo;
import lombok.Data;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 计算主题得分参数
 *
 * @author: LYM
 * @date: 2023/12/12 15:03
 */
@Data
public class CalThemeScoreParam {
    /**
     * 评分指标
     */
    private Map<Integer, EvlVo> evlMap;

    /**
     * 短视频指标
     */
    private EvlVo videoEvl;

    /**
     * 直播指标
     */
    private EvlVo liveEvl;

    /**
     * 粉丝限制
     */
    private Map<Integer, FansLimitVo> fansLimit = new HashMap<>();

    /**
     * 直播数据
     */
    private List<LivePool> livePoolList;

    /**
     * 短视频数据
     */
    private List<VideoPool> videoPoolList;

    private Map<Long, AccountInfoVo> accountMap = new HashMap<>();

    public boolean getIsIgnoreLive(){
        return PublicUtil.isEmpty(liveEvl) || BigDecimal.ZERO.compareTo(liveEvl.getRate()) >= 0;
    }
    public boolean getIsIgnoreVideo(){
        return PublicUtil.isEmpty(videoEvl) || BigDecimal.ZERO.compareTo(videoEvl.getRate()) >= 0;
    }
}

