package cn.fw.dalaran.domain.db;

import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.enums.DisqualificationEnum;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import cn.fw.dalaran.domain.enums.config.WorksTypeEnum;
import cn.fw.dalaran.domain.vo.DisqualificationRecordVo;
import com.baomidou.mybatisplus.annotation.*;
import lombok.Data;
import org.springframework.util.StringUtils;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2023-03-29 15:08:57
 * @Description
 */
@Data
@TableName("disqualification_record")
public class DisqualificationRecordEntity implements Serializable {
    /**
     * 主键id
     */
    @TableId(type = IdType.AUTO)
    private Long id;
    /**
     * 集团id
     */
    private Long groupId;
    /**
     * 用户id
     */
    private Long userId;
    /**
     * 用户名
     */
    private String userName;
    /**
     * 账号
     */
    private String account;
    /**
     * 作品匹配上的主题id
     */
    private Long themeId;
    /**
     * 平台类型(1:抖音, 2:快手, 3:懂车帝, 4:哔哩哔哩)
     */
    private PlatformEnum platformType;
    /**
     * 作品类型(1:短视频, 2:直播)
     */
    private WorksTypeEnum worksType;
    /**
     * 作品标题
     */
    private String worksTitle;
    /**
     * 作品id
     */
    private String worksId;
    /**
     * 作品创建时间/直播开始时间
     */
    private Date worksStartTime;
    /**
     * 直播结束时间
     */
    private Date worksEndTime;
    /**
     * 主题配置tag
     */
    private String configTags;
    /**
     * 作品包含tags
     */
    private String worksTags;
    /**
     * 作品封面url
     */
    private String worksCover;
    /**
     * 不合格原因
     */
    private DisqualificationEnum reason;
    /**
     * 记录创建时间
     */
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    /**
     * db->vo
     *
     * @param db 数据库实体
     * @return 视图层对象
     */
    public static DisqualificationRecordVo toVo(DisqualificationRecordEntity db) {
        DisqualificationRecordVo vo = PublicUtil.copy(db, DisqualificationRecordVo.class);
        vo.setPlatformType(db.getPlatformType().getValue());
        vo.setWorksType(db.getWorksType().getValue());
        vo.setConfigTags(str2List(db.getConfigTags()));
        vo.setWorksTags(str2List(db.getWorksTags()));
        vo.setReason(db.getReason().getDesc());
        return vo;
    }

    /**
     * 字符串->list
     *
     * @param str 待转换字符串
     * @return list
     */
    public static List<String> str2List(String str) {
        if (!StringUtils.hasText(str))
            return new ArrayList<>();
        try {
            return Arrays.stream(str.split(","))
                    .collect(Collectors.toList());
        } catch (Exception e) {
            return new ArrayList<>();
        }
    }
}
