package cn.fw.dalaran.domain.db;

import cn.fw.common.data.entity.BaseAuditableTimeEntity;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import cn.fw.dalaran.domain.vo.LivePoolVo;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Date;
import java.util.stream.Collectors;

/**
 * @author kurisu
 * @date 2021-11-17 14:02
 * @description 直播池
 */
@EqualsAndHashCode(callSuper = true)
@Data
@ToString(callSuper = true)
public class LivePool extends BaseAuditableTimeEntity<VideoPool, Long> {
    /**
     * 账号id
     */
    private Long accountId;
    /**
     * 房间号
     */
    private String roomNo;
    /**
     * 用户昵称
     */
    private String userNick;
    /**
     * 平台类型
     */
    private PlatformEnum platform;
    /**
     * 直播标题
     */
    private String title;
    /**
     * 直播封面
     */
    private String cover;
    /**
     * 直播回放地址
     */
    private String playbackUrl;
    /**
     * 观看人数
     */
    private Integer uv;
    /**
     * 峰值观看人数
     */
    private Integer uvPeak;
    /**
     * 点赞人数
     */
    private Integer likeNum;
    /**
     * 分享人数
     */
    private Integer shareNum;
    /**
     * 评论人数
     */
    private Integer commentNum;
    /**
     * 新增粉丝数
     */
    private Integer newFanNum;
    /**
     * 收礼数
     */
    private Integer receiveNum;
    /**
     * 收礼金额
     */
    private BigDecimal receiveAmount;
    /**
     * 直播时长(分钟)
     */
    private Integer liveDuration;
    /**
     * 是否为有效直播(-2:被判空挂, -1:付费, 0:无效, 1:有效, 2:最佳但不计综合得分, 11:有效且计综合得分)
     */
    private Integer validLive;
    /**
     * 直播开始时间
     */
    private Date liveStartTime;
    /**
     * 直播结束时间
     */
    private Date liveEndTime;
    /**
     * 主题id
     */
    private Long themeId;
    /**
     * 主题
     */
    private String theme;
    /**
     * 话题
     */
    private String tags;
    /**
     * 品牌id
     */
    private Long brandId;
    /**
     * 品牌名称
     */
    private String brandName;
    /**
     * 车系id
     */
    private Long seriesId;
    /**
     * 车系名称
     */
    private String seriesName;
    /**
     * 作品无效原因
     */
    private String invalidReason;
    /**
     * 作品所属一级管理
     */
    private Long firstManagerId;
    /**
     * 作品所属二级管理
     */
    private Long secondManagerId;
    /**
     * 作品所属三级管理
     */
    private Long thirdManagerId;
    /**
     * 作品所属四级管理
     */
    private Long fourthManagerId;
    /**
     * 作品归属人id
     */
    private Long belongUserId;
    /**
     * 作品归属门店id
     */
    private Long belongShopId;
    /**
     * 单场得分
     */
    private BigDecimal score;
    /**
     * 主题得分
     */
    private BigDecimal themeScore;
    /**
     * 得分计算规则
     */
    private String scoreCalJson;
    /**
     * 0无效 1有效
     */
    private Integer status = 1;

    /**
     * 作品不合格原因收集器
     *
     * @param reasonValue 不合格枚举值
     * @return reasonValue:包含, -1:不包含
     */
    public Integer getInvalidCollector(Integer reasonValue) {
        if (StringUtils.hasText(this.invalidReason) && Arrays.stream(this.invalidReason.split(","))
                .map(Integer::valueOf)
                .collect(Collectors.toList())
                .contains(reasonValue))
            return reasonValue;
        return -1;
    }

    /**
     * db -> vo
     */
    public static LivePoolVo toVo(LivePool db) {
        LivePoolVo vo = PublicUtil.copy(db, LivePoolVo.class);
        vo.setId(db.getId());
        vo.setPlatform(db.getPlatform().getValue());
        return vo;
    }
}
