package cn.fw.dalaran.domain.db.config;

import cn.fw.dalaran.domain.enums.config.ValidConfigTypeEnum;
import cn.fw.dalaran.domain.enums.config.ValidLiveEnum;
import cn.fw.dalaran.domain.enums.config.ValidVideoEnum;
import cn.fw.dalaran.domain.enums.config.WorksTypeEnum;
import cn.fw.dalaran.domain.vo.config.ValidConfigNewVo;
import com.baomidou.mybatisplus.annotation.*;
import lombok.Data;
import org.springframework.beans.BeanUtils;

import java.io.Serializable;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 最低有效性
 * @author wmy3969
 * @version 1.0
 * @date 2022-02-15 12:37:25
 */
@Data
@TableName("valid_config_new")
public class ValidConfigNew implements Serializable {
    /**
     * 主键id
     */
    @TableId(type = IdType.AUTO)
    private Long id;
    /**
     * 所属配置组id
     */
    private Long configGroupId;
    /**
     * 集团id
     */
    @TableField(exist = false)
    private Long groupId;
    /**
     * 指标类型(1:直播, 2:视频, 3:封面, 4话题)
     * {@link ValidConfigTypeEnum}
     */
    private Integer type;
    /**
     * 指标类型
     * {@link cn.fw.dalaran.domain.enums.config.ValidIndexTypeEnum}
     */
    private Integer indexType;
    /**
     * 最小值
     */
    private Integer minValue;
    /**
     * 是否有效
     */
    private Boolean valid;
    /**
     * 创建时间
     */
    @TableField(value = "create_time", fill = FieldFill.INSERT)
    private Date createTime;
    /**
     * 更新时间
     */
    @TableField(value = "update_time", fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    /**
     * dbItem->singleVo
     *
     * @param dbItem
     * @return 将DB传参转换成singleVo
     */
    public static ValidConfigNewVo.singleVo toSingleVo(ValidConfigNew dbItem) {
        final ValidConfigNewVo.singleVo singleVo = new ValidConfigNewVo.singleVo();
        BeanUtils.copyProperties(dbItem, singleVo);
        if (Objects.equals(dbItem.getType(), WorksTypeEnum.VIDEO.getValue())) {
            singleVo.setIndexTypeName(ValidVideoEnum.getNameByVale(dbItem.getIndexType()));
        } else if (Objects.equals(dbItem.getType(), WorksTypeEnum.LIVE.getValue())) {
            singleVo.setIndexTypeName(ValidLiveEnum.getNameByVale(dbItem.getIndexType()));
        }
        return singleVo;
    }

    /**
     * dbList->Vo
     *
     * @param dbList
     * @return 将DB传参转换成VO
     */
    public static ValidConfigNewVo toVO(List<ValidConfigNew> dbList) {
        final ValidConfigNewVo vo = new ValidConfigNewVo();
        vo.setDetails(dbList.stream()
                .map(ValidConfigNew::toSingleVo)
                .collect(Collectors.toList()));
        return vo;
    }

    /**
     * 根据指定条件创建配置
     *
     * @param indexType     指标类型
     * @param configGroupId 所属配置组id
     * @param type          类型(1:视频, 2:直播)
     * @param minValue      指标最小值
     * @return 基础配置
     */
    public static ValidConfigNew createConfig(Integer indexType, Long configGroupId, Integer type, Integer minValue, Boolean valid) {
        final ValidConfigNew db = new ValidConfigNew();
        db.setConfigGroupId(configGroupId);
        db.setType(type);
        db.setIndexType(indexType);
        db.setMinValue(minValue);
        db.setValid(valid);
        return db;
    }
}
