package cn.fw.dalaran.domain.dto.config;

import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.dto.common.ShopDto;
import cn.fw.dalaran.domain.enums.config.*;
import lombok.Data;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.List;

/**
 * 配置组保存参数
 *
 * @author wmy3969
 * @version 1.0
 * @date 2022-5-5 14:50:59
 */
@Data
public class GroupSaveDto {
    /**
     * 生效中配置组id
     */
    private Long id;
    /**
     * 草稿配置组id
     */
    private Long draftId;
    /**
     * 配置组名称
     */
    @NotBlank(message = "配置组名称不能为空")
    private String configName;
    /**
     * 品牌id
     */
    @NotNull(message = "品牌id不能为空")
    private Long brandId;
    /**
     * 品牌名
     */
    private String brandName;
    /**
     * 全部门店(0:否, 1:是)
     */
    @NotNull(message = "未选择门店范围")
    private Integer allShop;
    /**
     * 适用门店
     */
    @Valid
    private List<ShopDto> shopRange;
    /**
     * 指标配置
     */
    @NotEmpty(message = "未完成有效性配置")
    @Valid
    private List<EvlDto> evlItems;
    /**
     * 有效性配置
     */
    @NotEmpty(message = "未完成指标配置")
    @Valid
    private List<ValidDto> validItems;
    /**
     * 是否保存草稿
     */
    @NotNull(message = "是否保存草稿不能为空")
    private Boolean draft;

    /**
     * 有效性配置检查
     */
    public void validCheck() {
        PublicUtil.assertNotEmpty(this.validItems, "有效性配置不能为空");
        for (ValidDto validItem : this.validItems) {
            ValidConfigTypeEnum configTypeEnum = ValidConfigTypeEnum.ofValue(validItem.getType());
            PublicUtil.assertNotEmpty(configTypeEnum, "有效性类型错误,请刷新后重试");
            for (ValidDto.ValidSubConfigDto validSubItem : validItem.getValidSubItems()) {
                Object type = null;
                if (ValidConfigTypeEnum.LIVE.getValue().equals(configTypeEnum.getValue())) {
                    type = ValidLiveEnum.ofValue(validSubItem.getIndexType());
                } else if (ValidConfigTypeEnum.VIDEO.getValue().equals(configTypeEnum.getValue())) {
                    type = ValidVideoEnum.ofValue(validSubItem.getIndexType());
                } else if (ValidConfigTypeEnum.COVER.getValue().equals(configTypeEnum.getValue())) {
                    type = ValidCoverEnum.ofValue(validSubItem.getIndexType());
                    boolean valueCheck = PublicUtil.isEmpty(validSubItem.getMinValue())
                            || (validSubItem.getMinValue() >= 0 && validSubItem.getMinValue() <= 100);
                    PublicUtil.assertTrue(valueCheck, "封面有效性设置错误,请输入0-100数字");
                } else if (ValidConfigTypeEnum.TOPIC.getValue().equals(configTypeEnum.getValue())) {
                    type = ValidTopicEnum.ofValue(validSubItem.getIndexType());
                    boolean valueCheck = PublicUtil.isEmpty(validSubItem.getMinValue())
                            || (validSubItem.getMinValue() >= 0 && validSubItem.getMinValue() <= 100);
                    PublicUtil.assertTrue(valueCheck, "话题有效性设置错误,请输入0-100数字");
                }
                PublicUtil.assertNotEmpty(type, "有效性配置类型代码错误,请刷新后重试");
            }
        }
    }

    /**
     * 指标检查
     */
    public void evlCheck() {
        PublicUtil.assertNotEmpty(this.evlItems, "指标配置不能为空");
        BigDecimal rate = this.evlItems.stream()
                .map(EvlDto::getRate)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        PublicUtil.assertTrue(rate.compareTo(new BigDecimal("100")) == 0, "[短视频]+[直播]指标占比之和不为100%, 请重新配置");
        for (EvlDto evlIndex : this.evlItems) {
            WorksTypeEnum worksTypeEnum = WorksTypeEnum.ofValue(evlIndex.getType());
            PublicUtil.assertNotEmpty(worksTypeEnum, "指标项代码错误,请刷新后重试");
            for (EvlDto.EvlSubIndexDto evlSubIndex : evlIndex.getEvlSubItems()) {
                SubTypeEnum subTypeEnum = SubTypeEnum.ofValue(evlSubIndex.getSubType());
                PublicUtil.assertNotEmpty(subTypeEnum, "指标子项代码错误,请刷新后重试");
            }
        }
    }
}
