package cn.fw.dalaran.domain.dto.report;

import lombok.Data;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;

/**
 * 得分汇总结果
 *
 * @author: lym
 * @date: 2023/5/18
 */
@Data
public class ScoreCalResultDto {
    private LocalDate day;
    /**
     * 短视频数量
     */
    private Integer videoSum = 0;
    /**
     * 直播数量
     */
    private Integer liveSum = 0;
    /**
     * 短视频总得分
     */
    private BigDecimal videoScoreSum = BigDecimal.ZERO;
    /**
     * 直播总得分
     */
    private BigDecimal liveScoreSum = BigDecimal.ZERO;
    /**
     * 直播平均分
     */
    private BigDecimal liveAverage = BigDecimal.ZERO;
    /**
     * 短视频平均分
     */
    private BigDecimal videoAverage = BigDecimal.ZERO;
    /**
     * 综合得分
     */
    private BigDecimal totalScoreRate = BigDecimal.ZERO;


    public BigDecimal getLiveAverage() {
        return this.liveSum == 0 ? BigDecimal.ZERO
                : this.liveScoreSum.divide(BigDecimal.valueOf(this.liveSum), 4, RoundingMode.HALF_UP);
    }

    public BigDecimal getVideoAverage() {
        return this.videoSum == 0 ? BigDecimal.ZERO
                : this.videoScoreSum.divide(BigDecimal.valueOf(this.videoSum), 4, RoundingMode.HALF_UP);
    }

    public BigDecimal getTotalScoreRate() {
        BigDecimal totalRate = this.getLiveAverage().add(this.getVideoAverage());
        return BigDecimal.ONE.compareTo(totalRate) < 0 ? BigDecimal.ONE : totalRate;
    }
}
