package cn.fw.dalaran.domain.dto.report;

import lombok.Data;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;

/**
 * 短视频主题考核统计
 *
 * @author: lym
 * @date: 2023/5/6
 */
@Data
public class VideoTrendDto {
    /**
     * 采集日期
     */
    private LocalDate collectDay;
    /**
     * 总发布量
     */
    private Integer i1 = 0;
    /**
     * 有效发布量
     */
    private Integer i2 = 0;
    /**
     * 播放量
     */
    private Integer i4 = 0;
    /**
     * 完整播放
     */
    private Integer i5 = 0;
    /**
     * 有效发布率
     */
    private BigDecimal validRate = BigDecimal.ZERO;
    /**
     * 完整播放率
     */
    private BigDecimal fullPlayRate = BigDecimal.ZERO;

    /**
     * 获取有效率
     *
     * @return 有效发布率
     */
    public BigDecimal getValidRate() {
        return this.i1 > 0 ? BigDecimal.valueOf(this.i2).divide(BigDecimal.valueOf(this.i1), 4, RoundingMode.HALF_UP) : BigDecimal.ZERO;
    }

    /**
     * 获取完播率
     *
     * @return 完播率
     */
    public BigDecimal getFullPlayRate() {
        return this.i4 > 0 ? BigDecimal.valueOf(this.i5).divide(BigDecimal.valueOf(this.i4), 4, RoundingMode.HALF_UP) : BigDecimal.ZERO;
    }
}
