package cn.fw.dalaran.domain.dto.report.score;

import com.alibaba.fastjson.annotation.JSONField;
import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.Data;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Set;

/**
 * 得分率数据
 *
 * @author: juzi
 * @date: 2023/9/15 14:41
 */
@Data
public class ScoringRateDto {
    /**
     * 用户id
     */
    private Long userId;
    /**
     * 管理门店id列表
     */
    @JsonIgnore
    @JSONField(serialize = false)
    private Set<Long> shopIds;
    /**
     * 综合得分率
     */
    private BigDecimal i1 = BigDecimal.ZERO;
    /**
     * 直播综合得分率
     */
    private BigDecimal i2 = BigDecimal.ZERO;
    /**
     * 短视频综合得分率
     */
    private BigDecimal i3 = BigDecimal.ZERO;
    /**
     * 直播计分占比
     */
    private BigDecimal i4 = BigDecimal.ZERO;
    /**
     * 直播总计得分
     */
    private BigDecimal i5 = BigDecimal.ZERO;
    /**
     * 短视频计分占比
     */
    private BigDecimal i6 = BigDecimal.ZERO;
    /**
     * 短视频总计得分
     */
    private BigDecimal i7 = BigDecimal.ZERO;

    public static BigDecimal getI5Total(List<ScoringRateDto> staffScoringRateList, Integer num) {
        BigDecimal total = staffScoringRateList.stream()
                .map(ScoringRateDto::getI5)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        return total.compareTo(BigDecimal.ZERO) > 0 ?
                total.divide(BigDecimal.valueOf(num), 4, RoundingMode.HALF_UP) : BigDecimal.ZERO;
    }

    public static BigDecimal getI7Total(List<ScoringRateDto> staffScoringRateList, Integer num) {
        BigDecimal total = staffScoringRateList.stream()
                .map(ScoringRateDto::getI7)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        return total.compareTo(BigDecimal.ZERO) > 0 ?
                total.divide(BigDecimal.valueOf(num), 4, RoundingMode.HALF_UP) : BigDecimal.ZERO;
    }

    public static BigDecimal getI2Total(List<ScoringRateDto> staffScoringRateList, Integer num) {
        BigDecimal total = staffScoringRateList.stream()
                .map(ScoringRateDto::getI2)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        return total.compareTo(BigDecimal.ZERO) > 0 ?
                total.divide(BigDecimal.valueOf(num), 4, RoundingMode.HALF_UP) : BigDecimal.ZERO;
    }

    public static BigDecimal getI3Total(List<ScoringRateDto> staffScoringRateList, Integer num) {
        BigDecimal total = staffScoringRateList.stream()
                .map(ScoringRateDto::getI3)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        return total.compareTo(BigDecimal.ZERO) > 0 ?
                total.divide(BigDecimal.valueOf(num), 4, RoundingMode.HALF_UP) : BigDecimal.ZERO;
    }

    public static BigDecimal getI1Total(List<ScoringRateDto> staffScoringRateList, Integer num) {
        BigDecimal total = staffScoringRateList.stream()
                .map(ScoringRateDto::getI1)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        return total.compareTo(BigDecimal.ZERO) > 0 ?
                total.divide(BigDecimal.valueOf(num), 4, RoundingMode.HALF_UP) : BigDecimal.ZERO;
    }
}

