package cn.fw.dalaran.domain.dto.report.score;

import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.VideoPool;
import lombok.Data;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 人员综合得分参数
 *
 * @author: lym
 * @date: 2023/5/15
 */
@Data
public class StaffScoreSumDto {
    /**
     * 主题id
     */
    private Long themeId;
    /**
     * 归属得分人
     */
    private Long belongUserId;
    /**
     * 归属门店
     */
    private Long belongShopId;
    /**
     * 数据日期
     */
    private LocalDate collectDay;
    /**
     * 有效得分率
     */
    private BigDecimal score = BigDecimal.ZERO;
    /**
     * 是否计入综合得分
     */
    public Boolean calScoreRate = true;

    /**
     * 按门店拆分员工数据
     *
     * @param list
     * @return
     */
    public static Map<Long, List<StaffScoreSumDto>> groupByShop(List<StaffScoreSumDto> list) {
        return list.stream().collect(Collectors.groupingBy(StaffScoreSumDto::getBelongShopId));
    }

    /**
     * 按门店累计直播作品总得分
     *
     * @param liveList
     * @return
     */
    public static Map<Long, List<StaffScoreSumDto>> sumLiveScoreByShop(List<LivePool> liveList) {
        Map<Long, List<StaffScoreSumDto>> result = new HashMap<>();
        Map<Long, List<LivePool>> shopLive = liveList.stream()
                .collect(Collectors.groupingBy(LivePool::getBelongShopId));
        shopLive.forEach((k, v) -> result.put(k, new LinkedList<>(sumLiveScoreByUser(v).values())));
        return result;
    }

    /**
     * 按门店累计短视频作品总得分
     *
     * @param videoList
     * @return
     */
    public static Map<Long, List<StaffScoreSumDto>> sumVideoScoreByShop(List<VideoPool> videoList) {
        Map<Long, List<StaffScoreSumDto>> result = new HashMap<>();
        Map<Long, List<VideoPool>> shopVideo = videoList.stream()
                .collect(Collectors.groupingBy(VideoPool::getBelongShopId));
        shopVideo.forEach((k, v) -> result.put(k, new LinkedList<>(sumVideoScoreByUser(v).values())));
        return result;
    }


    /**
     * 按人员累计直播作品总得分
     *
     * @param liveList
     * @return
     */
    public static Map<Long, StaffScoreSumDto> sumLiveScoreByUser(List<LivePool> liveList) {
        Map<Long, List<LivePool>> staffLive = liveList.stream()
                .collect(Collectors.groupingBy(LivePool::getBelongUserId));
        Map<Long, StaffScoreSumDto> result = new HashMap<>();
        for (Map.Entry<Long, List<LivePool>> entry : staffLive.entrySet()) {
            result.put(entry.getKey(), getStaffLiveScore(entry.getValue()));
        }
        return result;
    }

    /**
     * 按人员累计短视频作品总得分
     *
     * @param videoList
     * @return
     */
    public static Map<Long, StaffScoreSumDto> sumVideoScoreByUser(List<VideoPool> videoList) {
        Map<Long, List<VideoPool>> staffLive = videoList.stream()
                .collect(Collectors.groupingBy(VideoPool::getBelongUserId));
        Map<Long, StaffScoreSumDto> result = new HashMap<>();
        for (Map.Entry<Long, List<VideoPool>> entry : staffLive.entrySet()) {
            result.put(entry.getKey(), getStaffVideoScore(entry.getValue()));
        }
        return result;
    }

    /**
     * 按人员累计直播作品总得分
     */
    public static StaffScoreSumDto getStaffLiveScore(List<LivePool> liveList) {
        LivePool livePool = liveList.get(0);
        StaffScoreSumDto staffScoreSumDto = new StaffScoreSumDto();
        staffScoreSumDto.setThemeId(livePool.getThemeId());
        staffScoreSumDto.setBelongShopId(livePool.getBelongShopId());
        staffScoreSumDto.setBelongUserId(livePool.getBelongUserId());
        staffScoreSumDto.setScore(liveList.stream().map(LivePool::getScore).reduce(BigDecimal::add).orElse(BigDecimal.ZERO));
        staffScoreSumDto.setCalScoreRate(liveList.stream().noneMatch(r -> r.getValidLive() != 11));
        return staffScoreSumDto;
    }

    /**
     * 按人员累计短视频总得分
     */
    public static StaffScoreSumDto getStaffVideoScore(List<VideoPool> videoList) {
        VideoPool videoPool = videoList.get(0);
        StaffScoreSumDto staffScoreSumDto = new StaffScoreSumDto();
        staffScoreSumDto.setThemeId(videoPool.getThemeId());
        staffScoreSumDto.setBelongShopId(videoPool.getBelongShopId());
        staffScoreSumDto.setBelongUserId(videoPool.getBelongUserId());
        staffScoreSumDto.setScore(videoList.stream().map(VideoPool::getScore).reduce(BigDecimal::add).orElse(BigDecimal.ZERO));
        staffScoreSumDto.setCalScoreRate(videoList.stream().noneMatch(r -> r.getValidVideo() != 11));
        return staffScoreSumDto;
    }
}
