package cn.fw.dalaran.domain.enums;

import com.baomidou.mybatisplus.annotation.EnumValue;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.List;

/**
 * @author kurisu
 * @date 2020-08-11 17:37
 * @description 平台   1：快手 2：抖音
 */
@Getter
@AllArgsConstructor
public enum PlatformEnum {

    KS(1, "快手", true),// 快手
    DY(2, "抖音", true),// 抖音
    DCD(3, "懂车帝", false),// 懂车帝
    BILIBILI(4, "哔哩哔哩", false),// 哔哩哔哩
    WX(5, "微信视频号", false),// 微信视频号
    ;

    /**
     * 值
     */
    @JsonValue
    @EnumValue
    private final Integer value;
    /**
     * 名称
     */
    private final String name;

    private final Boolean limitFans;

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static PlatformEnum ofValue(final Integer value) {
        for (final PlatformEnum typeEnum : PlatformEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final PlatformEnum typeEnum : PlatformEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return null;
    }

    /**
     * 获取目前在使用的平台枚举
     *
     * @return 目前在使用的平台枚举
     */
    public static List<PlatformEnum> getInUsePlatform() {
        return Arrays.asList(KS, DY, DCD, BILIBILI);
    }
}
