package cn.fw.dalaran.domain.enums.config;

import com.baomidou.mybatisplus.annotation.EnumValue;
import com.fasterxml.jackson.annotation.JsonCreator;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 评价指标枚举
 *
 * @author kurisu
 * @date 2020-08-11 17:37
 */
@Getter
@AllArgsConstructor
public enum SubTypeEnum {
    /**
     * 观看人数/播放量
     */
    UV(1, "观看人数", null),

    /**
     * 完整播放量
     */
    FULL_UV(6, "完整播放量", WorksTypeEnum.VIDEO),

    /**
     * 在线峰值
     */
    UV_PEAK(7, "在线峰值", WorksTypeEnum.LIVE),
    /**
     * 直播时长
     */
    LIVE_DURATION(8, "直播时长", WorksTypeEnum.LIVE),

    /**
     * 点赞
     */
    LIKE(2, "点赞", null),
    /**
     * 分享
     */
    SHARE(3, "分享", null),
    /**
     * 评论
     */
    COMMENT(4, "评论", null),
    /**
     * 增粉
     */
    NEW_FANS(5, "增粉", null),
    ;

    /**
     * 值
     */
    @EnumValue
    private final Integer value;
    /**
     * 名称
     */
    private final String name;
    /**
     * 所属业务类型 null则通用
     */
    private final WorksTypeEnum bizType;

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static SubTypeEnum ofValue(final Integer value) {
        for (final SubTypeEnum typeEnum : SubTypeEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final SubTypeEnum typeEnum : SubTypeEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }

    public static List<SubTypeEnum> getByBizType(final WorksTypeEnum bizType) {
        List<SubTypeEnum> list = new ArrayList<>();
        for (final SubTypeEnum typeEnum : SubTypeEnum.values()) {
            if (Objects.isNull(typeEnum.bizType) || bizType.equals(typeEnum.bizType)) {
                list.add(typeEnum);
            }
        }
        return list;
    }
}
