package cn.fw.dalaran.domain.enums.config;

import cn.fw.dalaran.domain.vo.config.ValidVo;
import com.baomidou.mybatisplus.annotation.EnumValue;
import com.fasterxml.jackson.annotation.JsonCreator;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.stream.Collectors;

/**
 * 封面有效性设置
 *
 * @author: lym
 * @date: 2023/4/28
 */
@Getter
@AllArgsConstructor
public enum ValidCoverEnum {
    /**
     * 封面相似度
     */
    COVER_SIMILARITY(1, "封面相似度", "%"),
    ;

    /**
     * 值
     */
    @EnumValue
    private final Integer value;
    /**
     * 名称
     */
    private final String name;
    /**
     * 单位
     */
    private final String unit;

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static ValidCoverEnum ofValue(final Integer value) {
        for (final ValidCoverEnum typeEnum : ValidCoverEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final ValidCoverEnum typeEnum : ValidCoverEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }

    public static ValidVo getConfigItems() {
        ValidVo videoVo = new ValidVo();
        videoVo.setType(ValidConfigTypeEnum.COVER.getValue());
        videoVo.setTypeName(ValidConfigTypeEnum.COVER.getName());
        videoVo.setValidSubItems(Arrays.stream(ValidCoverEnum.values()).map(r -> {
            ValidVo.ValidSubConfigVo validSubConfigVo = new ValidVo.ValidSubConfigVo();
            validSubConfigVo.setIndexType(r.getValue());
            validSubConfigVo.setIndexTypeName(r.getName());
            validSubConfigVo.setUnit(r.getUnit());
            return validSubConfigVo;
        }).collect(Collectors.toList()));
        return videoVo;
    }
}
