package cn.fw.dalaran.domain.enums.config;

import cn.fw.dalaran.common.utils.PublicUtil;
import com.baomidou.mybatisplus.annotation.EnumValue;
import com.fasterxml.jackson.annotation.JsonCreator;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 封面有效性设置
 *
 * @author: lym
 * @date: 2023/4/28
 */
@Getter
@AllArgsConstructor
public enum ValidIndexTypeEnum {
    /**
     * 播放量
     */
    VIDEO_PLAY_CNT(ValidConfigTypeEnum.VIDEO, 1, "播放量", ""),
    /**
     * 完整播放量
     */
    VIDEO_FULL_PLAY_CNT(ValidConfigTypeEnum.VIDEO, 2, "完整播放量", ""),
    /**
     * 点赞数
     */
    VIDEO_LIKE_CNT(ValidConfigTypeEnum.VIDEO, 3, "点赞数", ""),
    /**
     * 评论数
     */
    VIDEO_COMMENT_CNT(ValidConfigTypeEnum.VIDEO, 4, "评论数", ""),
    /**
     * 分享数
     */
    VIDEO_SHARE_CNT(ValidConfigTypeEnum.VIDEO, 5, "分享数", ""),
    /**
     * 增粉数
     */
    VIDEO_NEW_FANS_CNT(ValidConfigTypeEnum.VIDEO, 6, "增粉数", ""),
    /**
     * 观看人数
     */
    LIVE_WATCH_USER(ValidConfigTypeEnum.LIVE, 1, "观看人数", ""),
    /**
     * 观看人数巅峰
     */
    LIVE_WATCH_USER_PEAK(ValidConfigTypeEnum.LIVE, 2, "观看人数巅峰", ""),
    /**
     * 点赞数
     */
    LIVE_LIKE_CNT(ValidConfigTypeEnum.LIVE, 3, "点赞数", ""),
    /**
     * 评论数
     */
    LIVE_COMMENT_CNT(ValidConfigTypeEnum.LIVE, 4, "评论数", ""),
    /**
     * 分享数
     */
    LIVE_SHARE_CNT(ValidConfigTypeEnum.LIVE, 5, "分享数", ""),
    /**
     * 增粉数
     */
    LIVE_NEW_FANS_CNT(ValidConfigTypeEnum.LIVE, 6, "增粉数", ""),
    /**
     * 直播时长
     */
    LIVE_LIVE_DURATION(ValidConfigTypeEnum.LIVE, 7, "直播时长", ""),
    /**
     * 封面相似度
     */
    COVER_SIMILARITY(ValidConfigTypeEnum.COVER, 1, "封面相似度", "%"),
    /**
     * 话题相似度
     */
    TOPIC_SIMILARITY(ValidConfigTypeEnum.TOPIC, 1, "话题相似度", "%"),
    ;

    /**
     * 有效性类型
     */
    private final ValidConfigTypeEnum validType;
    /**
     * 值
     */
    @EnumValue
    private final Integer value;
    /**
     * 名称
     */
    private final String name;
    /**
     * 单位
     */
    private final String unit;

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static ValidIndexTypeEnum ofValue(final Integer value) {
        for (final ValidIndexTypeEnum typeEnum : ValidIndexTypeEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(ValidConfigTypeEnum validType, Integer value) {
        List<ValidIndexTypeEnum> list = Arrays.stream(ValidIndexTypeEnum.values())
                .filter(r -> r.getValidType().getValue().equals(validType.getValue()) && r.getValue().equals(value))
                .collect(Collectors.toList());
        return PublicUtil.isEmpty(list) ? "" : list.get(0).name;
    }
}
