package cn.fw.dalaran.domain.enums.config;

import cn.fw.dalaran.domain.vo.config.ValidVo;
import com.baomidou.mybatisplus.annotation.EnumValue;
import com.fasterxml.jackson.annotation.JsonCreator;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.stream.Collectors;

/**
 * 直播有效性配置
 *
 * @author wmy3969
 * @version 1.0
 * @date 2022/2/15 15:56
 */
@Getter
@AllArgsConstructor
public enum ValidLiveEnum {
    /**
     * 付费推广判定倍率
     */
//    PAY_LIVE_RATE(-1, "付费推广判定倍率"),
    /**
     * 单次活动有效直播场次
     */
//    LIVE_CNT(0, "单次活动有效直播场次"),
    /**
     * 观看人数
     */
    WATCH_USER(1, "观看人数"),
    /**
     * 观看人数巅峰
     */
    WATCH_USER_PEAK(2, "观看人数巅峰"),
    /**
     * 点赞数
     */
    LIKE_CNT(3, "点赞数"),
    /**
     * 评论数
     */
    COMMENT_CNT(4, "评论数"),
    /**
     * 分享数
     */
    SHARE_CNT(5, "分享数"),
    /**
     * 增粉数
     */
    NEW_FANS_CNT(6, "增粉数"),
    /**
     * 直播时长
     */
    LIVE_DURATION(7, "直播时长(分钟)"),
    ;

    /**
     * 值
     */
    @EnumValue
    private final Integer value;
    /**
     * 名称
     */
    private final String name;

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static ValidLiveEnum ofValue(final Integer value) {
        for (final ValidLiveEnum typeEnum : ValidLiveEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final ValidLiveEnum typeEnum : ValidLiveEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }

    public static ValidVo getConfigItems() {
        ValidVo videoVo = new ValidVo();
        videoVo.setType(ValidConfigTypeEnum.LIVE.getValue());
        videoVo.setTypeName(ValidConfigTypeEnum.LIVE.getName());
        videoVo.setValidSubItems(Arrays.stream(ValidLiveEnum.values()).map(r -> {
            ValidVo.ValidSubConfigVo validSubConfigVo = new ValidVo.ValidSubConfigVo();
            validSubConfigVo.setIndexType(r.getValue());
            validSubConfigVo.setIndexTypeName(r.getName());
            return validSubConfigVo;
        }).collect(Collectors.toList()));
        return videoVo;
    }
}
