package cn.fw.dalaran.domain.enums.config;

import cn.fw.dalaran.domain.vo.config.ValidVo;
import com.baomidou.mybatisplus.annotation.EnumValue;
import com.fasterxml.jackson.annotation.JsonCreator;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.stream.Collectors;

/**
 * 视频有效性配置
 *
 * @author wmy3969
 * @version 1.0
 * @date 2022/2/15 10:56
 */
@Getter
@AllArgsConstructor
public enum ValidVideoEnum {
    /**
     * 单次活动有效视频数
     */
//    VIDEO_CNT(0, "单次活动有效视频数"),
    /**
     * 播放量
     */
    PLAY_CNT(1, "播放量"),
    /**
     * 完整播放量
     */
    FULL_PLAY_CNT(2, "完整播放量"),
    /**
     * 点赞数
     */
    LIKE_CNT(3, "点赞数"),
    /**
     * 评论数
     */
    COMMENT_CNT(4, "评论数"),
    /**
     * 分享数
     */
    SHARE_CNT(5, "分享数"),
    /**
     * 增粉数
     */
    NEW_FANS_CNT(6, "增粉数"),
    /**
     * 短视频时长
     */
    VIDEO_DURATION(7, "短视频时长(秒)"),
    ;

    /**
     * 值
     */
    @EnumValue
    private final Integer value;
    /**
     * 名称
     */
    private final String name;

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static ValidVideoEnum ofValue(Integer value) {
        for (ValidVideoEnum typeEnum : ValidVideoEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(Integer value) {
        for (ValidVideoEnum typeEnum : ValidVideoEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }

    public static ValidVo getConfigItems() {
        ValidVo videoVo = new ValidVo();
        videoVo.setType(ValidConfigTypeEnum.VIDEO.getValue());
        videoVo.setTypeName(ValidConfigTypeEnum.VIDEO.getName());
        videoVo.setValidSubItems(Arrays.stream(ValidVideoEnum.values()).map(r -> {
            ValidVo.ValidSubConfigVo validSubConfigVo = new ValidVo.ValidSubConfigVo();
            validSubConfigVo.setIndexType(r.getValue());
            validSubConfigVo.setIndexTypeName(r.getName());
            return validSubConfigVo;
        }).collect(Collectors.toList()));
        return videoVo;
    }
}
