package cn.fw.dalaran.domain.enums.report;

import cn.fw.dalaran.domain.enums.config.WorksTypeEnum;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2023/5/11 9:40
 * @Description 作品不合格原因枚举
 */
@Getter
@AllArgsConstructor
public enum InvalidReasonEnum {
    /*短视频*/
    PLAY_CNT(1, "播放量指标不足"),
    FULL_PLAY_CNT(2, "完整播放量指标不足"),
    VIDEO_DURATION(14, "短视频时长指标不足"),

    /*公共*/
    LIKE_CNT(3, "点赞数指标不足"),
    COMMENT_CNT(4, "评论数指标不足"),
    SHARE_CNT(5, "分享数指标不足"),
    NEW_FANS_CNT(6, "增粉数指标不足"),
    COVER(7, "封面不匹配"),
    TAG(8, "话题不匹配"),
    MATCH_MULTIPLE_THEME(15, "作品匹配多主题"),

    /*直播*/
    WATCH_USER(9, "观看人数指标不足"),
    WATCH_USER_PEAK(10, "观看人数巅峰指标不足"),
    LIVE_DURATION(11, "直播时长指标不足"),
    LOSE_PLAYBACK(12, "无直播回放"),
    PAY(13, "判定为付费"),
    ;

    private final Integer value;
    private final String desc;

    /**
     * 根据value值获取枚举对象
     *
     * @param value 枚举值
     * @return 对应枚举对象
     */
    public static InvalidReasonEnum getEnumByValue(Integer value) {
        for (InvalidReasonEnum anEnum : InvalidReasonEnum.values()) {
            if (Objects.equals(value, anEnum.getValue()))
                return anEnum;
        }
        return null;
    }

    /**
     * 按作品类型获取考核指标集合
     *
     * @param type 作品类型
     * @return 对应考核指标集合
     */
    public static List<InvalidReasonEnum> getCheckIndexByWorkType(WorksTypeEnum type) {
        List<InvalidReasonEnum> result = new ArrayList<>(Arrays.asList(LIKE_CNT, COMMENT_CNT, SHARE_CNT,
                NEW_FANS_CNT, COVER, TAG, MATCH_MULTIPLE_THEME));// 加入公共指标
        if (Objects.equals(type, WorksTypeEnum.LIVE)) {
            result.add(WATCH_USER);
            result.add(WATCH_USER_PEAK);
            result.add(LIVE_DURATION);
            result.add(LOSE_PLAYBACK);
            result.add(PAY);
        } else if (Objects.equals(type, WorksTypeEnum.VIDEO)) {
            result.add(PLAY_CNT);
            result.add(FULL_PLAY_CNT);
            result.add(VIDEO_DURATION);
        }
        return result;
    }
}
