package cn.fw.dalaran.domain.param.report.base;

import cn.fw.common.web.annotation.LoginContextField;
import cn.fw.dalaran.common.utils.DateUtil;
import lombok.Data;
import org.hibernate.validator.constraints.ScriptAssert;

import javax.validation.constraints.NotEmpty;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2023/5/9 9:57
 * @Description 报表查询基础参数
 */
@Data
@ScriptAssert(lang = "javascript", script = "_this.checkParam()", message = "不指定主题id时, 开始时间不能为空")
public class ReportBaseQueryParam {
    /**
     * 集团id
     */
    @LoginContextField(LoginContextField.Name.GROUP_ID)
    private Long groupId;
    /**
     * 主题id
     */
    private Long themeId;
    /**
     * 门店id列表
     */
    @NotEmpty(message = "门店列表不能为空")
    private List<Long> shopIds;
    /**
     * 用户id列表
     */
    private List<Long> userIds = new ArrayList<>();
    /**
     * 开始时间
     */
    private Date beginTime;
    /**
     * 截止时间
     */
    private Date endTime;
    /**
     * 有效标识(不传:查询有效+无效, true:有效, false:无效)
     */
    private Boolean valid;

    /**
     * 是否为主题维度获取报表
     *
     * @return 是否为主题维度获取报表
     */
    public Boolean getThemeDimension() {
        return Objects.nonNull(this.themeId);
    }

    /**
     * 参数校验: 如果未指定主题id, 则必须传开始时间
     *
     * @return 校验是否通过
     */
    public boolean checkParam() {
        if (Objects.isNull(this.themeId))
            return Objects.nonNull(beginTime);
        return true;
    }

    /**
     * 重写get方法
     *
     * @return endTime
     */
    public Date getEndTime() {
        if (Objects.isNull(this.endTime))
            return null;
        return DateUtil.dayEnd(this.endTime);
    }
}
