package cn.fw.dalaran.domain.param.report.score;

import cn.fw.common.web.annotation.LoginContextField;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import lombok.Data;
import org.hibernate.validator.constraints.ScriptAssert;

import javax.validation.constraints.NotEmpty;
import java.time.LocalDate;
import java.util.*;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2023/5/9 9:57
 * @Description 报表查询基础参数
 */
@Data
@ScriptAssert(lang = "javascript", script = "_this.checkParam()", message = "不指定主题id时, 开始时间不能为空")
public class ScoreBaseQueryParam {
    /**
     * 集团id
     */
    @LoginContextField(LoginContextField.Name.GROUP_ID)
    private Long groupId;
    /**
     * 主题id
     */
    private Long themeId;
    /**
     * 门店id列表
     */
    @NotEmpty(message = "门店列表不能为空")
    private List<Long> shopIds;
    /**
     * 月度主题集合
     */
    private List<Long> themeIds = new LinkedList<>();
    /**
     * 用户id列表
     */
    private List<Long> userIds;
    /**
     * 开始时间
     */
    private Date beginTime;
    /**
     * 截止时间
     */
    private Date endTime;
    /**
     * 有效标识(不传:查询有效+无效, true:有效, false:无效)
     */
    private Boolean valid;
    private List<ActivityTheme> themeList = new LinkedList<>();

    /**
     * 获取开始时间
     *
     * @return 开始时间
     */
    public Date getBeginTime() {
        return Objects.nonNull(this.themeId) ? null : DateUtil.monthBegin(this.beginTime);
    }

    /**
     * 获取截止时间
     *
     * @return 截止时间
     */
    public Date getEndTime() {
        return Objects.nonNull(this.themeId) ? null : DateUtil.monthEnd(this.beginTime);
    }


    /**
     * 是否为主题维度获取报表
     *
     * @return 是否为主题维度获取报表
     */
    public Boolean getThemeDimension() {
        return Objects.nonNull(this.themeId);
    }

    /**
     * 参数校验: 如果未指定主题id, 则必须传开始时间
     *
     * @return 校验是否通过
     */
    public boolean checkParam() {
        if (Objects.isNull(this.themeId))
            return Objects.nonNull(beginTime);
        return true;
    }

    /**
     * 设置月度门店范围
     */
    public void setMonthThemeShopIds(Map<ActivityTheme, List<Long>> themeRange) {
        Set<Long> shopSet = new HashSet<>();
        if (PublicUtil.isNotEmpty(themeRange)) {
            themeRange.values().forEach(shopSet::addAll);
        }
        this.setShopIds(new ArrayList<>(shopSet));
    }
}
