package cn.fw.dalaran.domain.param.report.video;

import cn.fw.dalaran.common.myValidator.annotation.AssignValues;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.enums.report.ReportListTypeEnum;
import cn.fw.dalaran.domain.param.report.base.ReportBaseQueryParam;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;

import javax.validation.constraints.NotNull;

/**
 * 短视频报表分析列表查询参数
 * <p>
 * create at 2019-12-26
 *
 * @author zhangjj
 */
@Data
@ToString(callSuper = true)
@EqualsAndHashCode(callSuper = true)
public class VideoListParam extends VideoBaseQueryParam {

    /**
     * 一级列表分析类型(1:门店, 2:销售顾问, 3:平台, 4:不合格原因, 5:销售一级管理, 6:销售二级管理, 7:销售三级管理)
     */
    @NotNull(message = "列表类型不能为空")
    @AssignValues(ints = {1, 2, 3, 4, 5, 6, 7,8}, message = "你提交的维度有误!")
    private Integer rowType;
    /**
     * 一级分析结果id(下钻二级分析需要当前行id)
     */
    private String rowId;
    /**
     * 二级列表分析纬度（下钻二级分析需要当前分析纬度）
     * 1:门店, 2:销售顾问, 3:平台, 4:不合格原因, 5:销售一级管理, 6:销售二级管理, 7:销售三级管理
     */
    private Integer pointType;

    public boolean isCompletion() {
        return true;
//        return PublicUtil.isEmpty(this.pointType);
    }

    public boolean isManager() {
        return ReportListTypeEnum.MANAGER_1.getValue().equals(this.rowType) ||
                ReportListTypeEnum.MANAGER_2.getValue().equals(this.rowType) ||
                ReportListTypeEnum.MANAGER_3.getValue().equals(this.rowType);
    }
}
