package cn.fw.dalaran.domain.vo;

import cn.fw.dalaran.domain.dto.report.EvlFormulaDto;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import cn.fw.dalaran.domain.enums.report.InvalidReasonEnum;
import cn.fw.dalaran.domain.vo.activity.TopicVo;
import lombok.Data;
import lombok.ToString;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author kurisu
 * @date 2021-12-20 16:03
 * @description 直播池
 */
@Data
@ToString
public class LivePoolVo {
    /**
     * 主键id
     */
    private Long id;
    /**
     * 房间号
     */
    private String roomNo;
    /**
     * 用户昵称
     */
    private String userNick;
    /**
     * 平台类型
     * {@link PlatformEnum}
     */
    private Integer platform;
    /**
     * 直播标题
     */
    private String title;
    /**
     * 直播封面
     */
    private String cover;
    /**
     * 直播回放地址
     */
    private String playbackUrl;
    /**
     * 观看人数
     */
    private Integer uv;
    /**
     * 峰值观看人数
     */
    private Integer uvPeak;
    /**
     * 点赞人数
     */
    private Integer likeNum;
    /**
     * 分享人数
     */
    private Integer shareNum;
    /**
     * 评论人数
     */
    private Integer commentNum;
    /**
     * 新增粉丝数
     */
    private Integer newFanNum;
    /**
     * 收礼数
     */
    private Integer receiveNum;
    /**
     * 收礼金额
     */
    private BigDecimal receiveAmount;
    /**
     * 直播时长(分钟)
     */
    private Integer liveDuration;
    /**
     * 是否有效直播(-2:被判空挂, -1:付费, 0:无效, 1:有效, 11:有效且最佳)
     */
    private Integer valid;
    /**
     * 直播开始时间
     */
    private Date liveStartTime;
    /**
     * 直播结束时间
     */
    private Date liveEndTime;
    /**
     * 所属活动主题
     */
    private String theme;
    /**
     * 话题(作品包含)
     */
    private List<String> tags;
    /**
     * 话题(主题设置)
     */
    private List<TopicVo> setTags;
    /**
     * 品牌id
     */
    private Long brandId;
    /**
     * 品牌名称
     */
    private String brandName;
    /**
     * 车系id
     */
    private Long seriesId;
    /**
     * 车系名称
     */
    private String seriesName;
    /**
     * 所属人员
     */
    private String userName;
    /**
     * 所属门店
     */
    private String shopName;
    /**
     * 账号
     */
    private String account;
    /**
     * 得分率
     */
    private BigDecimal score;
    /**
     * 计分详情
     */
    private EvlFormulaDto scoreDetails;
    /**
     * 无效原因
     */
    private String invalidReason;

    /**
     * 获取有效性描述
     */
    public String getValidStatusDesc() {
        if (Objects.equals(this.valid, -2))
            return "空挂";
        if (Objects.equals(this.valid, -1))
            return "付费";
        if (Objects.equals(this.valid, 0))
            return "无效";
        if (Objects.equals(this.valid, 1))
            return "有效";
        if (Objects.equals(this.valid, 2))
            return "最佳不计分";
        if (Objects.equals(this.valid, 11))
            return "最佳计分";
        return "未知";
    }

    /**
     * 是否为最佳直播
     */
    public boolean getBestLive() {
        return Objects.nonNull(this.valid) && this.valid > 10;
    }

    /**
     * 重写get方法
     *
     * @return 是否为有效直播
     */
    /*public boolean getValid() {
        return Objects.nonNull(this.valid) && this.valid > 0;
    }*/
    public String getPlatformDesc() {
        PlatformEnum platformEnum = PlatformEnum.ofValue(platform);
        if (Objects.isNull(platformEnum)) {
            return "";
        }
        return platformEnum.getName();
    }

    /**
     * 获取无效原因
     *
     * @return 无效原因
     */
    public List<String> getInvalidReason() {
        if (!StringUtils.hasText(this.invalidReason))
            return null;
        try {
            return Arrays.stream(this.invalidReason.split(","))
                    .map(Integer::valueOf)
                    .map(InvalidReasonEnum::getEnumByValue)
                    .filter(Objects::nonNull)
                    .map(InvalidReasonEnum::getDesc)
                    .collect(Collectors.toList());
        } catch (Exception e) {
            return null;
        }
    }
}
