package cn.fw.dalaran.domain.vo.config;

import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.config.*;
import cn.fw.dalaran.domain.enums.config.*;
import cn.fw.dalaran.domain.vo.common.ShopVo;
import com.alibaba.fastjson.JSON;
import lombok.Data;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 配置组参数
 *
 * @author wmy3969
 * @version 1.0
 * @date 2022-5-5 14:50:59
 */
@Data
public class ConfigGroupDetailVo {
    /**
     * 草稿/配置组id
     */
    private Long id;
    /**
     * 配置组名称
     */
    private String configName;
    /**
     * 品牌id
     */
    private Long brandId;
    /**
     * 品牌名
     */
    private String brandName;
    /**
     * 全部门店(0:否, 1:是)
     */
    private Integer allShop;
    /**
     * 适用门店
     */
    private List<ShopVo> shopRange;
    /**
     * 指标配置
     */
    private List<EvlVo> evlItems;
    /**
     * 有效性配置
     */
    private List<ValidVo> validItems;

    /**
     * 获取草稿详情
     *
     * @param draft
     * @return
     */
    public static ConfigGroupDetailVo dbToDraftVo(ConfigGroupDraft draft) {
        ConfigGroupDetailVo configGroupVo = getInitConfig();
        configGroupVo.setId(draft.getId());
        configGroupVo.setConfigName(draft.getConfigName());
        configGroupVo.setAllShop(draft.getAllShop());
        configGroupVo.setShopRange(PublicUtil.isNotEmpty(draft.getShopJson())
                ? JSON.parseArray(draft.getShopJson(), ShopVo.class) : null);
        toEvlDraftVo(draft, configGroupVo.evlItems);
        toValidDraftVo(draft, configGroupVo.getValidItems());
        return configGroupVo;
    }


    /**
     * 指标评分草稿转换vo
     *
     * @param draft
     * @param evlItems
     */
    private static void toEvlDraftVo(ConfigGroupDraft draft, List<EvlVo> evlItems) {
        if (PublicUtil.isEmpty(draft.getEvlJson())) {
            return;
        }
        List<EvlVo> evlDraftList = JSON.parseArray(draft.getEvlJson(), EvlVo.class);
        Map<Integer, EvlVo> draftMap = evlDraftList.stream().collect(Collectors.toMap(EvlVo::getType, r -> r));
        for (EvlVo evlVo : evlItems) {
            EvlVo evlDraft = draftMap.get(evlVo.getType());
            if (PublicUtil.isEmpty(evlDraft)) {
                continue;
            }
            evlVo.setRate(evlDraft.getRate());
            //子项
            Map<Integer, EvlVo.EvlSubIndexVo> subMap = evlDraft.getEvlSubItems().stream().collect(Collectors.toMap(EvlVo.EvlSubIndexVo::getSubType, r -> r));
            for (EvlVo.EvlSubIndexVo evlSubVo : evlVo.getEvlSubItems()) {
                EvlVo.EvlSubIndexVo evlSubDraft = subMap.get(evlSubVo.getSubType());
                if (PublicUtil.isNotEmpty(evlSubDraft)) {
                    evlSubVo.setRate(evlSubDraft.getRate());
                    evlSubVo.setStandard(evlSubDraft.getStandard());
                }
            }
        }
    }

    /**
     * 有效性配置草稿转换vo
     *
     * @param draft
     * @param validItems
     */
    private static void toValidDraftVo(ConfigGroupDraft draft, List<ValidVo> validItems) {
        if (PublicUtil.isEmpty(draft.getValidJson())) {
            return;
        }
        List<ValidVo> validDraftList = JSON.parseArray(draft.getValidJson(), ValidVo.class);
        Map<Integer, ValidVo> draftMap = validDraftList.stream().collect(Collectors.toMap(ValidVo::getType, r -> r));
        for (ValidVo validVo : validItems) {
            ValidVo validDraft = draftMap.get(validVo.getType());
            if (PublicUtil.isEmpty(validDraft)) {
                continue;
            }
            Map<Integer, ValidVo.ValidSubConfigVo> indexTypeMap = validDraft.getValidSubItems().stream()
                    .collect(Collectors.toMap(ValidVo.ValidSubConfigVo::getIndexType, r -> r));
            for (ValidVo.ValidSubConfigVo validSubItem : validVo.getValidSubItems()) {
                ValidVo.ValidSubConfigVo validSubDraftVo = indexTypeMap.get(validSubItem.getIndexType());
                if (PublicUtil.isNotEmpty(validSubDraftVo)) {
                    validSubItem.setMinValue(validSubDraftVo.getMinValue());
                    validSubItem.setValid(validSubDraftVo.getValid());
                }
            }
        }
    }

    /**
     * 获取配置组详情vo
     *
     * @param configGroup
     * @param validList
     * @return
     */
    public static ConfigGroupDetailVo dbToVo(ConfigGroup configGroup, List<ValidConfigNew> validList,
                                             List<EvlIndex> evlIndexList, List<EvlSubIndex> evlSubIndexList) {
        ConfigGroupDetailVo vo = new ConfigGroupDetailVo();
        vo.setConfigName(configGroup.getConfigName());
        vo.setBrandId(configGroup.getBrandId());
        vo.setBrandName(configGroup.getBrandName());
        vo.setId(configGroup.getId());
        vo.setAllShop(configGroup.getAllShop());
        vo.setShopRange(PublicUtil.isNotEmpty(configGroup.getShopJson())
                ? JSON.parseArray(configGroup.getShopJson(), ShopVo.class) : null);
        vo.setValidItems(toValidVo(validList));
        vo.setEvlItems(toEvlVo(evlIndexList, evlSubIndexList));
        return vo;
    }

    /**
     * 有效性配置 db转vo
     */
    public static List<ValidVo> toValidVo(List<ValidConfigNew> configNewList) {
        Map<Integer, List<ValidConfigNew>> validMap = configNewList.stream().collect(Collectors.groupingBy(ValidConfigNew::getType));
        List<ValidVo> validVoList = validMap.values().stream().map(subValidList -> {
            ValidConfigNew configNew = subValidList.get(0);
            ValidVo validVo = new ValidVo();
            ValidConfigTypeEnum configTypeEnum = ValidConfigTypeEnum.ofValue(configNew.getType());
            validVo.setType(configNew.getType());
            validVo.setTypeName(Objects.nonNull(configTypeEnum) ? configTypeEnum.getName() : null);
            //有效性子项
            validVo.setValidSubItems(subValidList.stream().map(dbItem -> {
                ValidVo.ValidSubConfigVo validSubVo = new ValidVo.ValidSubConfigVo();
                validSubVo.setIndexType(dbItem.getIndexType());
                if (ValidConfigTypeEnum.LIVE.getValue().equals(dbItem.getType())) {
                    validSubVo.setIndexTypeName(ValidLiveEnum.getNameByVale(dbItem.getIndexType()));
                } else if (ValidConfigTypeEnum.VIDEO.getValue().equals(dbItem.getType())) {
                    validSubVo.setIndexTypeName(ValidVideoEnum.getNameByVale(dbItem.getIndexType()));
                } else if (ValidConfigTypeEnum.TOPIC.getValue().equals(dbItem.getType())) {
                    validSubVo.setIndexTypeName(ValidTopicEnum.getNameByVale(dbItem.getIndexType()));
                } else if (ValidConfigTypeEnum.COVER.getValue().equals(dbItem.getType())) {
                    validSubVo.setIndexTypeName(ValidCoverEnum.getNameByVale(dbItem.getIndexType()));
                }
                validSubVo.setMinValue(dbItem.getMinValue());
                validSubVo.setValid(dbItem.getValid());
                return validSubVo;
            }).collect(Collectors.toList()));
            return validVo;
        }).collect(Collectors.toList());
        return validVoList.stream().sorted(Comparator.comparing(ValidVo::getType).reversed()).collect(Collectors.toList());
    }

    /**
     * 指标评分配置 db转vo
     */
    public static List<EvlVo> toEvlVo(List<EvlIndex> evlIndexList, List<EvlSubIndex> evlSubIndexList) {
        Map<Long, List<EvlSubIndex>> subMap = evlSubIndexList.stream().collect(Collectors.groupingBy(EvlSubIndex::getEvlId));
        return evlIndexList.stream().map(r -> {
            EvlVo evlVo = new EvlVo();
            evlVo.setType(r.getType().getValue());
            evlVo.setTypeName(r.getType().getName());
            evlVo.setRate(r.getRate());
            List<EvlVo.EvlSubIndexVo> subIndexVoList = subMap.getOrDefault(r.getId(), new ArrayList<>()).stream().map(evlSubIndex -> {
                EvlVo.EvlSubIndexVo evlSubIndexVo = new EvlVo.EvlSubIndexVo();
                evlSubIndexVo.setSubType(evlSubIndex.getSubType().getValue());
                evlSubIndexVo.setSubTypeName(evlSubIndex.getSubType().getName());
                evlSubIndexVo.setRate(evlSubIndex.getRate());
                evlSubIndexVo.setStandard(evlSubIndex.getStandard());
                return evlSubIndexVo;
            }).collect(Collectors.toList());
            evlVo.setEvlSubItems(subIndexVoList);
            return evlVo;
        }).collect(Collectors.toList());
    }

    /**
     * 新增初始配置信息
     *
     * @return 配置信息
     */
    public static ConfigGroupDetailVo getInitConfig() {
        ConfigGroupDetailVo detailVo = new ConfigGroupDetailVo();
        detailVo.setValidItems(ValidVo.getInitConfig());
        detailVo.setEvlItems(EvlVo.getInitConfig());
        return detailVo;
    }
}
