package cn.fw.dalaran.domain.vo.config;

import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.config.ConfigGroup;
import cn.fw.dalaran.domain.db.config.ConfigGroupDraft;
import cn.fw.dalaran.domain.vo.common.ShopVo;
import com.alibaba.fastjson.JSON;
import lombok.Data;

import java.util.List;

/**
 * 草稿/配置组参数
 *
 * @author wmy3969
 * @version 1.0
 * @date 2022-5-5 14:50:59
 */
@Data
public class ConfigGroupVo {
    /**
     * 草稿/配置组id
     */
    private Long id;
    /**
     * 配置组名称
     */
    private String configName;
    /**
     * 品牌id
     */
    private Long brandId;
    /**
     * 品牌名
     */
    private String brandName;
    /**
     * 创建该配置的用户id
     */
    private Long userId;
    /**
     * 创建该配置的用户名
     */
    private String userName;
    /**
     * 全部门店(0:否, 1:是)
     */
    private Integer allShop;
    /**
     * 适用门店
     */
    private List<ShopVo> shopRange;
    /**
     * 配置是否启用
     */
    private Boolean enable;

    /**
     * draftDb转换vo
     */
    public static ConfigGroupVo dbToVo(ConfigGroupDraft draftDb) {
        ConfigGroupVo vo = PublicUtil.copy(draftDb, ConfigGroupVo.class);
        String shopRange = draftDb.getShopJson();
        vo.setShopRange(PublicUtil.isNotEmpty(shopRange) ? JSON.parseArray(shopRange, ShopVo.class) : null);
        vo.setId(draftDb.getId());
        return vo;
    }

    /**
     * db转换vo
     */
    public static ConfigGroupVo dbToVo(ConfigGroup db) {
        ConfigGroupVo vo = PublicUtil.copy(db, ConfigGroupVo.class);
        String shopRange = db.getShopJson();
        vo.setShopRange(PublicUtil.isNotEmpty(shopRange) ? JSON.parseArray(shopRange, ShopVo.class) : null);
        return vo;
    }
}
