package cn.fw.dalaran.domain.vo.config;

import cn.fw.dalaran.domain.db.config.EvlIndex;
import cn.fw.dalaran.domain.db.config.EvlSubIndex;
import cn.fw.dalaran.domain.enums.config.SubTypeEnum;
import cn.fw.dalaran.domain.enums.config.WorksTypeEnum;
import lombok.Data;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 评价指标配置
 *
 * @author: lym
 * @date: 2023/4/24
 */
@Data
public class EvlVo {
    /**
     * 指标类型(1直播 2短视频)
     */
    private Integer type;
    /**
     * 指标类型名称
     */
    private String typeName;
    /**
     * 占比 （%）
     */
    private BigDecimal rate;
    /**
     * 子指标配置
     */
    private List<EvlSubIndexVo> evlSubItems;

    @Data
    public static class EvlSubIndexVo {
        /**
         * 主键id
         */
        private Long id;
        /**
         * 父指标id
         */
        private Long evlId;
        /**
         * 子指标类型
         */
        private Integer subType;
        /**
         * 子指标类型名称
         */
        private String subTypeName;
        /**
         * 分值占比 （%）
         */
        private BigDecimal rate;
        /**
         * 考核标准
         */
        private Integer standard;
    }

    public static EvlSubIndexVo toSubIndexVo(Long evlId, EvlSubIndex evlSubIndex) {
        EvlSubIndexVo evlSubIndexVo = new EvlSubIndexVo();
        evlSubIndexVo.setId(evlSubIndex.getId());
        evlSubIndexVo.setEvlId(evlId);
        evlSubIndexVo.setSubType(evlSubIndex.getSubType().getValue());
        evlSubIndexVo.setSubTypeName(evlSubIndex.getSubType().getName());
        evlSubIndexVo.setRate(evlSubIndex.getRate());
        evlSubIndexVo.setStandard(evlSubIndex.getStandard());
        return evlSubIndexVo;
    }

    /**
     * 吃刷
     * @return
     */
    public static List<EvlVo> getInitConfig() {
        return Arrays.stream(WorksTypeEnum.values()).map(indexType -> {
            EvlVo evlIndexVo = new EvlVo();
            evlIndexVo.setType(indexType.getValue());
            evlIndexVo.setTypeName(indexType.getName());
            evlIndexVo.setRate(BigDecimal.ZERO);
            //子指标
            List<SubTypeEnum> evlSubIndexTypeList = SubTypeEnum.getByBizType(indexType);
            List<EvlVo.EvlSubIndexVo> subIndexVoList = evlSubIndexTypeList.stream().map(subIndexType -> {
                EvlVo.EvlSubIndexVo evlSubIndexVo = new EvlVo.EvlSubIndexVo();
                evlSubIndexVo.setSubType(subIndexType.getValue());
                evlSubIndexVo.setSubTypeName(subIndexType.getName());
                evlSubIndexVo.setRate(BigDecimal.ZERO);
                return evlSubIndexVo;
            }).sorted(Comparator.comparing(EvlVo.EvlSubIndexVo::getSubType)).collect(Collectors.toList());
            evlIndexVo.setEvlSubItems(subIndexVoList);
            return evlIndexVo;
        }).collect(Collectors.toList());
    }
}
