package cn.fw.dalaran.domain.vo.report;

import lombok.Data;

import java.util.List;
import java.util.Objects;

/**
 * 综合得分计算规则
 *
 * @author: lym
 * @date: 2023/7/24
 */
@Data
public class UserCalRuleVo {
    /**
     * 用户id
     */
    private Long userId;

    /**
     * 是否计算综合得分
     */
    private Boolean calcCompositeScore;

    /**
     * 最低有效短视频场次
     */
    private Integer needVideoValidTimes = 0;
    /**
     * 有效短视频场次
     */
    private Integer currentVideoValidTimes = 0;
    /**
     * 最低有效短视频场次是否达标
     */
    private Boolean videoTimesQualified;

    /**
     * 最低有效直播场次
     */
    private Integer needLiveValidTimes = 0;
    /**
     * 有效直播场次
     */
    private Integer currentLiveValidTimes = 0;
    /**
     * 最低有效直播场次是否达标
     */
    private Boolean liveTimesQualified;

    /**
     * 账号是否达标
     */
    private List<AccountFansLimit> fansLimit;

    @Data
    public static class AccountFansLimit {
        /**
         * 账号
         */
        private String accountNo;
        /**
         * 平台类型
         */
        private Integer platformType;
        /**
         * 平台名称
         */
        private String platformName;
        /**
         * 最低粉丝数
         */
        private Integer minNum;
        /**
         * 账号粉丝数
         */
        private Long fansNum;
        /**
         * 粉丝数是否达标
         */
        private Boolean fansQualified;
    }


    /**
     * 是否计算综合得分
     */
    public boolean getCalcCompositeScore() {
        if (Objects.isNull(this.currentLiveValidTimes) || Objects.isNull(this.currentVideoValidTimes))
            return false;
        return this.currentLiveValidTimes >= this.needLiveValidTimes &&
                this.currentVideoValidTimes >= this.needVideoValidTimes;
    }
}
