package cn.fw.dalaran.domain.vo.report.live;

import cn.fw.dalaran.domain.vo.report.score.TargetResultDto;
import cn.fw.dalaran.domain.vo.report.score.WorksCompleteDetails;
import lombok.Data;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Objects;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2023/5/8 17:38
 * @Description 基础vo
 */
@Data
public class BaseVo {
    /**
     * 直播场次
     */
    private Integer i1 = 0;
    /**
     * 观看人数
     */
    private Integer i2 = 0;
    /**
     * 平均每场观看人数
     */
    private Integer i2Avg = 0;
    /**
     * 点赞量
     */
    private Integer i3 = 0;
    /**
     * 平均每场点赞量
     */
    private Integer i3Avg = 0;
    /**
     * 评论数
     */
    private Integer i4 = 0;
    /**
     * 平均每场评论数
     */
    private Integer i4Avg = 0;
    /**
     * 分享次数
     */
    private Integer i5 = 0;
    /**
     * 平均每场分享次数
     */
    private Integer i5Avg = 0;
    /**
     * 新增粉丝数
     */
    private Integer i6 = 0;
    /**
     * 平均每场新增粉丝数
     */
    private Integer i6Avg = 0;
    /**
     * 直播时长
     */
    private Integer i7 = 0;
    /**
     * 平均每场直播时长
     */
    private Integer i7Avg = 0;
    /**
     * 有效直播场数
     */
    private Integer i8 = 0;
    /**
     * 观看人数巅峰
     */
    private Integer i9 = 0;
    /**
     * 平均每场观看人数巅峰
     */
    private Integer i9Avg = 0;

    /**
     * 无效场次
     */
    private Integer invalidCnt = 0;
    /**
     * 有效直播率
     */
    private BigDecimal validRate = BigDecimal.ZERO;
    /**
     * 目标及计分情况结果
     */
    private TargetResultDto targetResult;
    /**
     * 综合得分
     */
    private BigDecimal score;

    /**
     * 获取有效率
     *
     * @return 有效发布率
     */
    public BigDecimal getValidRate() {
        if (Objects.isNull(this.i1) || Objects.isNull(i8) || Objects.equals(this.i1, 0))
            return BigDecimal.ZERO;
        return BigDecimal.valueOf(this.i8).divide(BigDecimal.valueOf(this.i1), 4, RoundingMode.HALF_UP);
    }

    /**
     * 获取无效场次
     *
     * @return 无效场次
     */
    public Integer getInvalidCnt() {
        if (Objects.isNull(this.i1) || Objects.isNull(i8))
            return 0;
        return Math.max(this.i1 - this.i8, 0);
    }

    /* 重写get方法, 获取平均值, 结果向下取整 */
    public Integer getI2Avg() {
        if (Objects.equals(this.i8, 0))
            return 0;
        return this.i2 / this.i8;
    }

    public Integer getI3Avg() {
        if (Objects.equals(this.i8, 0))
            return 0;
        return this.i3 / this.i8;
    }

    public Integer getI4Avg() {
        if (Objects.equals(this.i8, 0))
            return 0;
        return this.i4 / this.i8;
    }

    public Integer getI5Avg() {
        if (Objects.equals(this.i8, 0))
            return 0;
        return this.i5 / this.i8;
    }

    public Integer getI6Avg() {
        if (Objects.equals(this.i8, 0))
            return 0;
        return this.i6 / this.i8;
    }

    public Integer getI7Avg() {
        if (Objects.equals(this.i8, 0))
            return 0;
        return this.i7 / this.i8;
    }

    public Integer getI9Avg() {
        if (Objects.equals(this.i8, 0))
            return 0;
        return this.i9 / this.i8;
    }
}
