package cn.fw.dalaran.domain.vo.report.score;

import cn.fw.dalaran.domain.dto.report.score.ScoreResultDto;
import lombok.Data;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Objects;

/**
 * @author: LYM
 * @date: 2023/12/14 18:46
 */
@Data
public class TargetResultDto {
    /**
     * 需要最佳短视频条数
     */
    private Integer needBestVideoCnt = 0;
    /**
     * 需要最佳直播场次
     */
    private Integer needBestLiveCnt = 0;
    /**
     * 当前最佳短视频条数
     */
    private Integer currentBestVideoCnt = 0;
    /**
     * 需要最佳直播场次
     */
    private Integer currentBestLiveCnt = 0;
    /**
     * 最佳短视频占比
     */
    private BigDecimal bestVideoRate = BigDecimal.ZERO;
    /**
     * 最佳直播率占比
     */
    private BigDecimal bestLiveRate = BigDecimal.ZERO;


    public static TargetResultDto initTargetResult(ScoreResultDto resultDto) {
        TargetResultDto targetResultDto = new TargetResultDto();
        targetResultDto.needBestLiveCnt = resultDto.getNeedLiveNum();
        targetResultDto.currentBestLiveCnt = resultDto.getLiveQualifiedNum();
        targetResultDto.needBestVideoCnt = resultDto.getNeedVideoNum();
        targetResultDto.currentBestVideoCnt = resultDto.getVideoQualifiedNum();
        return targetResultDto;
    }

    public BigDecimal getBestVideoRate() {
        if (Objects.equals(this.needBestVideoCnt, 0) || Objects.equals(this.currentBestVideoCnt, 0))
            return BigDecimal.ZERO;
        if (Objects.isNull(this.needBestVideoCnt) || Objects.isNull(this.currentBestVideoCnt))
            return BigDecimal.ZERO;
        return BigDecimal.valueOf(this.currentBestVideoCnt)
                .divide(BigDecimal.valueOf(this.needBestVideoCnt), 4, RoundingMode.HALF_UP);
    }

    public BigDecimal getBestLiveRate() {
        if (Objects.equals(this.needBestLiveCnt, 0) || Objects.equals(this.currentBestLiveCnt, 0))
            return BigDecimal.ZERO;
        if (Objects.isNull(this.needBestLiveCnt) || Objects.isNull(this.currentBestLiveCnt))
            return BigDecimal.ZERO;
        return BigDecimal.valueOf(this.currentBestLiveCnt)
                .divide(BigDecimal.valueOf(this.needBestLiveCnt), 4, RoundingMode.HALF_UP);
    }
}

