package cn.fw.dalaran.domain.vo.report.video;

import cn.fw.dalaran.domain.dto.report.EvlFormulaDto;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import cn.fw.dalaran.domain.enums.report.InvalidReasonEnum;
import cn.fw.dalaran.domain.vo.activity.TopicVo;
import lombok.Data;
import lombok.ToString;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 短视频池
 *
 * @author kurisu
 * @date 2021-12-20 16:03
 */
@Data
@ToString
public class VideoReportPoolVo {
    /**
     * 主键id
     */
    private Long id;
    /**
     * 视频id
     */
    private String videoId;
    /**
     * 视频标题
     */
    private String title;
    /**
     * 是否有效视频
     */
    private Integer valid;
    /**
     * 得分
     */
    private BigDecimal score;
    /**
     * 所属人员
     */
    private String userName;
    /**
     * 所属门店
     */
    private String shopName;
    /**
     * 平台类型
     */
    private Integer platform;
    /**
     * 预览图
     */
    private String preview;
    /**
     * 播放地址
     */
    private String playUrl;
    /**
     * 时长（秒）
     */
    private Integer duration;
    /**
     * 发布时间
     */
    private Date publishTime;
    /**
     * 观看人数
     */
    private Integer uv;
    /**
     * 完整观看人数
     */
    private Integer fullUv;
    /**
     * 视频完播率
     */
    private BigDecimal fullPlayRate;
    /**
     * 点赞数
     */
    private Integer likeNum;
    /**
     * 分享数
     */
    private Integer shareNum;
    /**
     * 评论数
     */
    private Integer commentNum;
    /**
     * 新增粉丝数
     */
    private Integer newFanNum;
    /**
     * 话题(作品包含)
     */
    private List<String> tags;
    /**
     * 话题(主题设置)
     */
    private List<TopicVo> setTags;
    /**
     * 无效原因
     */
    private String invalidReason;
    /**
     * 账号
     */
    private String account;
    /**
     * 用户昵称
     */
    private String userNick;
    /**
     * 所属活动主题id
     */
    private Long themeId;
    /**
     * 所属活动主题
     */
    private String theme;
    /**
     * 得分详情
     */
    private EvlFormulaDto scoreDetails;

    /**
     * 获取平台名称
     *
     * @return 平台名
     */
    public String getPlatformDesc() {
        PlatformEnum platformEnum = PlatformEnum.ofValue(platform);
        if (Objects.isNull(platformEnum)) {
            return "";
        }
        return platformEnum.getName();
    }

    /**
     * 获取完播率
     *
     * @return 完播率
     */
    public BigDecimal getFullPlayRate() {
        if (Objects.isNull(this.uv) || Objects.isNull(fullUv) || Objects.equals(BigDecimal.valueOf(this.uv).compareTo(BigDecimal.ZERO), 0))
            return BigDecimal.ZERO;
        return BigDecimal.valueOf(this.fullUv).divide(BigDecimal.valueOf(this.uv), 4, RoundingMode.HALF_UP);
    }

    /**
     * 获取无效原因
     *
     * @return 无效原因
     */
    public List<String> getInvalidReason() {
        if (!StringUtils.hasText(this.invalidReason))
            return null;
        try {
            return Arrays.stream(this.invalidReason.split(","))
                    .map(Integer::valueOf)
                    .map(InvalidReasonEnum::getEnumByValue)
                    .filter(Objects::nonNull)
                    .map(InvalidReasonEnum::getDesc)
                    .collect(Collectors.toList());
        } catch (Exception e) {
            return null;
        }
    }
}
