package cn.fw.dalaran.domain.vo.report.video;

import cn.fw.dalaran.domain.vo.report.score.TargetResultDto;
import cn.fw.dalaran.domain.vo.report.score.WorksCompleteDetails;
import lombok.Data;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Objects;

/**
 * 短视频主题考核统计
 *
 * @author: lym
 * @date: 2023/5/6
 */
@Data
public class VideoSummaryVo {
    /**
     * 总发布量
     */
    private Integer i1 = 0;
    /**
     * 有效发布量
     */
    private Integer i2 = 0;
    /**
     * 无效发布量
     */
    private Integer i3 = 0;
    /**
     * 播放量
     */
    private Integer i4 = 0;
    /**
     * 平均每场播放数
     */
    private Integer i4Avg = 0;
    /**
     * 完整播放
     */
    private Integer i5 = 0;
    /**
     * 平均每场完播数
     */
    private Integer i5Avg = 0;
    /**
     * 点赞
     */
    private Integer i6 = 0;
    /**
     * 平均每场点赞数
     */
    private Integer i6Avg = 0;
    /**
     * 分享
     */
    private Integer i7 = 0;
    /**
     * 平均每场分享数
     */
    private Integer i7Avg = 0;
    /**
     * 评论
     */
    private Integer i8 = 0;
    /**
     * 平均每场评论数
     */
    private Integer i8Avg = 0;
    /**
     * 增粉
     */
    private Integer i9 = 0;
    /**
     * 平均每场增分数
     */
    private Integer i9Avg = 0;
    /**
     * 有效发布率
     */
    private BigDecimal validRate = BigDecimal.ZERO;
    /**
     * 完整播放率
     */
    private BigDecimal fullPlayRate = BigDecimal.ZERO;
    /**
     * 目标及计分情况结果
     */
    private TargetResultDto targetResult;
    /**
     * 目标场次
     */
    private Integer targetCnt;
    /**
     * 达成场次
     */
    private Integer completeCnt;

    /**
     * 获取完播率
     *
     * @return 完播率
     */
    public BigDecimal getFullPlayRate() {
        return this.i4 > 0 ? BigDecimal.valueOf(this.i5).divide(BigDecimal.valueOf(this.i4), 4, RoundingMode.HALF_UP) : BigDecimal.ZERO;
    }

    /**
     * 获取有效率
     *
     * @return 有效发布率
     */
    public BigDecimal getValidRate() {
        return this.i1 > 0 ? BigDecimal.valueOf(this.i2).divide(BigDecimal.valueOf(this.i1), 4, RoundingMode.HALF_UP) : BigDecimal.ZERO;
    }

    /**
     * 获取场次达标率
     *
     * @return 场次达标率
     */
    public BigDecimal getCompleteRate() {
        if (Objects.isNull(this.targetCnt) || Objects.isNull(completeCnt) || Objects.equals(this.targetCnt, 0))
            return BigDecimal.ZERO;
        return BigDecimal.valueOf(this.completeCnt).divide(BigDecimal.valueOf(this.targetCnt), 4, RoundingMode.HALF_UP);
    }

    /* 重写get方法, 获取平均值, 结果向下取整 */
    public Integer getI4Avg() {
        if (Objects.equals(this.i2, 0))
            return 0;
        return this.i4 / this.i2;
    }

    public Integer getI5Avg() {
        if (Objects.equals(this.i2, 0))
            return 0;
        return this.i5 / this.i2;
    }

    public Integer getI6Avg() {
        if (Objects.equals(this.i2, 0))
            return 0;
        return this.i6 / this.i2;
    }

    public Integer getI7Avg() {
        if (Objects.equals(this.i2, 0))
            return 0;
        return this.i7 / this.i2;
    }

    public Integer getI8Avg() {
        if (Objects.equals(this.i2, 0))
            return 0;
        return this.i8 / this.i2;
    }

    public Integer getI9Avg() {
        if (Objects.equals(this.i2, 0))
            return 0;
        return this.i9 / this.i2;
    }
}
