package cn.fw.dalaran.rpc.dfs;

import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.data.base.domain.common.Message;
import cn.fw.dfs.sdk.api.FileService;
import cn.fw.dfs.sdk.api.result.DfsFile;
import cn.fw.dfs.sdk.api.result.FileInfo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2023/11/9 10:54
 * @Description
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class FileRpc {
    private final FileService fileService;

    /**
     * 文件上传确认
     *
     * @param fid        文件id
     * @param systemName 系统名称
     */
    public void confirm(String fid, String systemName) {
        fileService.confirm(fid, systemName);
    }

    /**
     * 文件上传
     *
     * @param data        文件数据
     * @param fileName    文件名
     * @param contentType 数据类型
     * @param systemName  系统名称
     * @param aliasName   别名
     * @return 文件id
     */
    public String upload(byte[] data, String fileName, String contentType, String systemName, String aliasName) {
        Message<String> result = fileService.upload(data, fileName, contentType, systemName, aliasName);
        PublicUtil.assertTrue(result.isSuccess(), result.getResult());
        return result.getData();
    }

    /**
     * 文件下载
     *
     * @param fid 文件id
     * @return 文件字节码信息
     */
    public DfsFile download(String fid) {
        Message<DfsFile> result = fileService.download(fid);
        PublicUtil.assertTrue(result.isSuccess(), result.getResult());
        return result.getData();
    }

    /**
     * 文件删除
     *
     * @param fid 文件id
     */
    public void delete(String fid) {
        log.info("调用文件系统RPC执行文件删除, 文件id: {}", fid);
        fileService.delete(fid);
    }

    /**
     * 查询文件信息
     *
     * @param fid 文件id
     * @return 文件信息
     */
    public FileInfo queryInfo(String fid) {
        Message<FileInfo> result = fileService.queryInfo(fid);
        PublicUtil.assertTrue(result.isSuccess(), result.getResult());
        return result.getData();
    }

    /**
     * 批量查询文件信息
     *
     * @param fidList 文件id列表
     * @return 文件信息
     */
    public List<FileInfo> queryBatchInfo(List<String> fidList) {
        Message<List<FileInfo>> result = fileService.queryBatchInfo(fidList);
        PublicUtil.assertTrue(result.isSuccess(), result.getResult());
        return result.getData();
    }
}
