package cn.fw.dalaran.rpc.erp;

import cn.fw.common.exception.BusinessException;
import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.data.base.domain.common.Message;
import cn.fw.erp.sdk.api.UserRoleApi;
import cn.fw.erp.sdk.api.result.UserRoleDataRange;
import cn.fw.erp.sdk.api.result.UserRoleInfo;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author zhanglz
 * @description erp 远程调用
 * @date 2019/4/23.
 */

@Slf4j
@Service
@RequiredArgsConstructor
public class ErpRpc {

    private final UserRoleApi userRoleApi;

    /**
     * 批量获取当前角色授权范围下所属人员
     *
     * @return
     */
    public List<UserRoleInfo> getUserRoleInfoList(RoleMsgEnum roleEnum, List<Long> rangeValues) {
        try {
            log.info("批量获取当前角色授权范围下所属人员erpRpc.getUserRoleInfoList 传入参数roleCode：{}, rangeValues:{}", roleEnum.getCode(), rangeValues);
            Message<List<UserRoleInfo>> message = userRoleApi.queryUserRoleInfoList(roleEnum.getCode(), rangeValues);
            log.info("批量获取当前角色授权范围下所属人员erpRpc.getUserRoleInfoList 返回数据：所属人员数量 {}", message.getData().size());
            PublicUtil.assertTrue(message.isSuccess(), message.getResult());
            return message.getData();
        } catch (Exception e) {
            throw new BusinessException("获取当前角色授权范围下所属人员失败");
        }
    }

    /**
     * 获取当前角色授权范围下所属人员
     *
     * @return
     */
    public List<UserRoleInfo> queryUserRoleInfos(String roleCode, Long rangeValue) {
        try {
            log.info("获取角色授权范围下所属人员erpRpc.queryUserRoleInfos 传入参数roleCode：{}, rangeValues:{}", roleCode, rangeValue);
            Message<List<UserRoleInfo>> message = userRoleApi.queryUserRoleInfos(roleCode, rangeValue);
            log.info("获取角色授权范围下所属人员erpRpc.queryUserRoleInfos 返回数据：所属人员数量 {}", message.getData().size());
            PublicUtil.assertTrue(message.isSuccess(), message.getResult());
            return message.getData();
        } catch (Exception e) {
            throw new BusinessException("获取角色授权范围下所属人员失败");
        }
    }

    /**
     * 获取人员指定角色授权门店范围
     *
     * @return
     */
    public List<UserRoleDataRange> queryUserRoleDataRange(Long userId, String roleCode) {
        try {
            log.info("获取角色授权范围下所属人员erpRpc.queryUserRoleInfos userId:{}roleCode：{}", userId, roleCode);
            Message<List<UserRoleDataRange>> message = userRoleApi.queryUserRoleDataRange(userId, roleCode);
            log.info("获取角色授权范围下所属人员erpRpc.queryUserRoleInfos 返回数据：所属门店数量 {}", JSON.toJSONString(message.getData()));
            PublicUtil.assertTrue(message.isSuccess(), message.getResult());
            return message.getData();
        } catch (Exception e) {
            throw new BusinessException("获取角色授权范围下所属人员失败");
        }
    }
}
