package cn.fw.dalaran.server;

import cn.fw.dalaran.common.ConfigProperties;
import cn.fw.dalaran.common.utils.AutoExpireLocalCache;
import cn.fw.rocketmq.annotation.EnableRocket;
import cn.fw.security.auth.client.EnableAuthClient;
import cn.hutool.core.thread.ThreadFactoryBuilder;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.openfeign.EnableFeignClients;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.EnableAspectJAutoProxy;
import org.springframework.data.redis.repository.configuration.EnableRedisRepositories;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * 启动类
 *
 * @author kurisu
 */

@EnableAsync
@EnableRocket
@SpringBootApplication
@EnableCaching
@EnableAuthClient
@EnableScheduling
@EnableDiscoveryClient
@EnableTransactionManagement
@Configuration
@EnableRedisRepositories
@MapperScan("cn.fw.**.dao")
@ComponentScan({"cn.fw.dalaran.*"})
@EnableFeignClients({"cn.fw.**.sdk"})
@EnableAspectJAutoProxy(exposeProxy = true)
@EnableConfigurationProperties({ConfigProperties.class})
public class Application {
    public static void main(final String[] args) {
        SpringApplication.run(Application.class, args);
    }

    /**
     * 初始化一个自动过期的本地缓存
     */
    @Bean
    public AutoExpireLocalCache initAutoExpireLocalCache() {
        return new AutoExpireLocalCache(1000);
    }

    @Bean(value = "dalaranThreadPool")
    public ThreadPoolExecutor getThreadPool() {
        return new ThreadPoolExecutor(
                6,
                6,
                30,
                TimeUnit.SECONDS,
                new LinkedBlockingQueue<>(1000),
                new ThreadFactoryBuilder().setNamePrefix("dalaran-thread-pool-").build(),
                new ThreadPoolExecutor.DiscardPolicy());
    }
}
