package cn.fw.dalaran.server.config;

import cn.fw.dalaran.common.config.ThreadTemplateVal;
import lombok.RequiredArgsConstructor;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import java.util.concurrent.Executor;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * 线程池配置
 *
 * @author: lym
 * @date: 2023/4/28
 */
@Configuration
@RequiredArgsConstructor
public class ThreadPoolConfig {
    private final ThreadTemplateVal threadTemplateVal;

    @Bean("asyncTaskExecutor")
    public Executor taskExecutor() {
        ThreadPoolTaskExecutor executor = new VisibleThreadPoolTaskExecutor();
        executor.setCorePoolSize(threadTemplateVal.getCorePoolSize());
        executor.setMaxPoolSize(threadTemplateVal.getMaxPoolSize());
        executor.setQueueCapacity(threadTemplateVal.getQueueCapacity());
        executor.setKeepAliveSeconds(threadTemplateVal.getKeepAliveSeconds());
        executor.setThreadNamePrefix(threadTemplateVal.getPrefixName());
        //设置线程池关闭的时候 等待所有的任务完成后再继续销毁其他的bean
        executor.setWaitForTasksToCompleteOnShutdown(true);
        //策略
        executor.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
        return executor;
    }
}
