package cn.fw.dalaran.server.controller.app;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.dto.DelPoolParam;
import cn.fw.dalaran.service.biz.PoolBizService;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 业务池控制器
 *
 * @author kurisu
 * @date 2021-11-17 15:02
 * @description 短视频/直播池控制器
 */
@Slf4j
@Validated
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.APP)
@RequestMapping("/pool")
public class PoolController {
    private final PoolBizService poolBizService;

    /**
     * 删除短视频
     *
     * @param param 参数
     * @return 删除结果
     * @ignore
     */
    @PostMapping("/video/del")
    @ControllerMethod("删除视频")
    @IgnoreAuth
    public Message<Boolean> delVideo(@RequestBody DelPoolParam param) {
        return success(poolBizService.delVideo(param));
    }

    /**
     * 删除直播
     *
     * @param param 参数
     * @return 删除结果
     * @ignore
     */
    @PostMapping("/live/del")
    @ControllerMethod("删除直播")
    @IgnoreAuth
    public Message<Boolean> delLive(@RequestBody DelPoolParam param) {
        return success(poolBizService.delLive(param));
    }

    /**
     * 更新视频
     *
     * @param param 参数
     * @return 更新结果
     * @ignore
     */
    @PostMapping("/video/update")
    @ControllerMethod("更新视频")
    @IgnoreAuth
    public Message<Boolean> updateVideo(@RequestBody VideoPool param) {
        return success(poolBizService.updateVideo(param));
    }

    /**
     * 更新直播
     *
     * @param param 参数
     * @return 更新结果
     * @ignore
     */
    @PostMapping("/live/update")
    @ControllerMethod("更新直播")
    @IgnoreAuth
    public Message<Boolean> updateLive(@RequestBody LivePool param) {
        return success(poolBizService.updateLive(param));
    }
}
