package cn.fw.dalaran.server.controller.app.remind;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.dalaran.domain.vo.AccountConfirmDetailVo;
import cn.fw.dalaran.service.biz.account.AccountConfirmBizService;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotNull;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 账号配置确认待办接口
 *
 * @author: lym
 * @date: 2023/4/19
 */
@Slf4j
@Validated
@IgnoreAuth
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@RequestMapping("/account/config/open")
public class AccountAnalysisRemindController {
    private final AccountConfirmBizService confirmBizService;

    /**
     * 获取账号配置确认待办详情
     *
     * @param accountId 账号id
     */
    @ControllerMethod("获取账号配置确认详情")
    @GetMapping("/detail")
    public Message<AccountConfirmDetailVo> getConfirmDetail(@NotNull(message = "账号id不能为空") Long accountId) {
        return success(confirmBizService.getDetail(accountId));
    }

    /**
     * 主动确认已开通数据分析功能
     *
     * @param accountId 账号id
     */
    @ControllerMethod("账号数据功能开通确认")
    @GetMapping("/confirm")
    public Message<Void> confirm(@NotNull(message = "账号id不能为空") Long accountId) {
        confirmBizService.confirm(accountId);
        return success();
    }
}
