package cn.fw.dalaran.server.controller.app.report;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.dalaran.common.domain.vo.NormTrendVO;
import cn.fw.dalaran.domain.param.report.score.ScoreBaseQueryParam;
import cn.fw.dalaran.domain.param.report.score.ScoreListParam;
import cn.fw.dalaran.domain.param.report.score.ScoreTrendParam;
import cn.fw.dalaran.domain.vo.report.UserCalRuleVo;
import cn.fw.dalaran.domain.vo.report.score.ScoreSummaryVo;
import cn.fw.dalaran.domain.vo.report.score.ThemeScoreListVo;
import cn.fw.dalaran.service.biz.report.score.ScoreMonthBizService;
import cn.fw.dalaran.service.biz.report.score.ScoreThemeBizService;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 考核得分报表控制器
 *
 * @author: lym
 * @date: 2023/5/12
 */
@Slf4j
@Validated
@IgnoreAuth
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.APP)
@RequestMapping("/report/score/theme")
public class ScoreReportController {
    private final ScoreThemeBizService scoreThemeBizService;
    private final ScoreMonthBizService scoreMonthBizService;

    /**
     * 获取得分统计
     *
     * @param param 查询参数
     * @return 主题得分统计
     */
    @GetMapping("/summary")
    @ControllerMethod("获取得分统计")
    public Message<ScoreSummaryVo> getScoreSummary(@Valid ScoreBaseQueryParam param) {
        return success(param.getThemeDimension()
                ? scoreThemeBizService.getScoreSummary(param)
                : scoreMonthBizService.getScoreSummary(param));
    }

    /**
     * 获取得分趋势
     *
     * @param param 查询参数
     * @return 主题趋势wu
     */
    @GetMapping("/trend")
    @ControllerMethod("获取主题趋势")
    public Message<NormTrendVO<BigDecimal>> getTrend(@Valid ScoreTrendParam param) {
        return success(param.getThemeDimension()
                ? scoreThemeBizService.getTrend(param)
                : scoreMonthBizService.getTrend(param));
    }

    /**
     * 获取得分率分析列表
     *
     * @param param 查询参数
     * @return 主题得分分析列表
     */
    @GetMapping("/analysis")
    @ControllerMethod("获取得分分析列表")
    public Message<List<ThemeScoreListVo>> getScoreAnalysis(@Valid ScoreListParam param) {
        return success(param.getThemeDimension()
                ? scoreThemeBizService.getScoreAnalysis(param)
                : scoreMonthBizService.getScoreAnalysis(param));
    }

    /**
     * 获取计分统计规则
     *
     * @param themeId
     * @param staffId
     * @return
     */
    @GetMapping("/user/cal/rule")
    @ControllerMethod("获取计分统计规则")
    public Message<UserCalRuleVo> getScoreUserCalRule(@NotNull(message = "未指定主题") Long themeId,
                                                      @NotNull(message = "未指定人员") Long staffId) {
        return success(scoreThemeBizService.getScoreUserCalRule(themeId, staffId));
    }
}
