package cn.fw.dalaran.server.controller.app.report;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.dalaran.common.domain.vo.NormTrendVO;
import cn.fw.dalaran.common.page.PageHelper;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.param.report.video.*;
import cn.fw.dalaran.domain.vo.report.video.VideoAnalysisVo;
import cn.fw.dalaran.domain.vo.report.video.VideoRateAnalysisListVo;
import cn.fw.dalaran.domain.vo.report.video.VideoReportPoolVo;
import cn.fw.dalaran.domain.vo.report.video.VideoSummaryVo;
import cn.fw.dalaran.service.biz.report.video.VideoRateAnalysisService;
import cn.fw.dalaran.service.biz.report.video.VideoReportBizService;
import cn.fw.dalaran.service.biz.report.video.VideoReportPoolBizService;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.math.BigDecimal;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 短视频报表
 *
 * @author: lym
 * @date: 2023/5/6
 */
@Slf4j
@Validated
@IgnoreAuth
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.APP)
@RequestMapping("/report/video")
public class VideoReportController {
    private final VideoReportBizService videoReportBizService;
    private final VideoRateAnalysisService videoRateAnalysisService;
    private final VideoReportPoolBizService videoReportPoolBizService;

    /**
     * 获取短视频汇总统计
     *
     * @param param 查询参数
     * @return 短视频汇总统计
     */
    @GetMapping("/summary")
    @ControllerMethod("获取短视频考核汇总统计")
    public Message<VideoSummaryVo> getVideoSummary(VideoBaseQueryParam param) {
        return success(videoReportBizService.getVideoSummary(param));
    }

    /**
     * 获取短视频趋势
     *
     * @param param 查询参数
     * @return 短视频趋势
     */
    @GetMapping("/trend")
    @ControllerMethod("获取短视频趋势")
    public Message<NormTrendVO<BigDecimal>> getVideoTrend(VideoTrendParam param) {
        return success(videoReportBizService.getVideoTrend(param));
    }

    /**
     * 获取短视频分析列表
     *
     * @param param 查询参数
     * @return 短视频分析列表
     */
    @GetMapping("/analysis")
    @ControllerMethod("获取短视频分析列表")
    public Message<List<VideoAnalysisVo>> getVideoAnalysis(VideoListParam param) {
        return success(videoReportBizService.getVideoAnalysis(param));
    }

    /**
     * 数据占比分析
     *
     * @param param 分析维度参数
     * @return 占比分析结果
     */
    @GetMapping("/rate/analysis")
    public Message<List<VideoRateAnalysisListVo>> getRateAnalysis(@CurrentUser LoginAuthBean user,
                                                                  @Valid VideoRateParam param) {
        return success(videoRateAnalysisService.getRateAnalysis(user, param));
    }

    /**
     * 获取短视频池
     *
     * @param param 查询参数
     * @return 短视频池
     * @ignoreParams pageHelper
     */
    @GetMapping("/pool")
    @ControllerMethod("获取短视频池")
    public Message<PageHelper<VideoReportPoolVo>> getThemeVideoPool(PageHelper<VideoPool> pageHelper,
                                                                    @Valid VideoPoolParam param) {
        return success(videoReportPoolBizService.getVideoReportPool(pageHelper, param));
    }
}
