package cn.fw.dalaran.server.controller.web;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.dalaran.common.page.PageHelper;
import cn.fw.dalaran.domain.param.DisqualificationRecordParam;
import cn.fw.dalaran.domain.vo.DisqualificationRecordVo;
import cn.fw.dalaran.service.biz.DisqualificationRecordBizService;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 自媒体作品不合格原因查询控制器
 *
 * @author wmy3969
 * @version 1.0
 * @date 2023/3/30 9:25
 * @Description 自媒体作品不合格原因查询控制器
 */
@Slf4j
@Validated
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@RequestMapping("/works/disqualification/record")
@IgnoreAuth
public class DisqualificationRecordController {

    private final DisqualificationRecordBizService disqualificationRecordBizService;

    /**
     * 根据查询条件分页获取满足条件的记录列表
     *
     * @param page  分页参数
     * @param param 查询条件
     * @return 满足条件的记录列表
     */
    @GetMapping("/getPage")
    @ControllerMethod("根据查询条件获取满足条件的记录列表")
    public Message<PageHelper<DisqualificationRecordVo>> getPage(PageHelper<Void> page,
                                                                 @Valid DisqualificationRecordParam param) {
        return success(disqualificationRecordBizService.getPage(page, param));
    }

}
