package cn.fw.dalaran.server.controller.web;

import cn.fw.attendance.sdk.api.AttendanceApi;
import cn.fw.attendance.sdk.api.StaffStatusApi;
import cn.fw.attendance.sdk.api.dto.UsersScheduleDetailsDto;
import cn.fw.attendance.sdk.api.result.UserStatusVo;
import cn.fw.attendance.sdk.api.result.UsersScheduleDetailsVo;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.dto.LivePoolDto;
import cn.fw.dalaran.domain.dto.VideoPoolDto;
import cn.fw.dalaran.domain.dto.activity.TopicDto;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import cn.fw.dalaran.domain.enums.report.InvalidReasonEnum;
import cn.fw.dalaran.service.biz.CommonBizService;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.dalaran.service.data.LivePoolService;
import cn.fw.dalaran.service.data.VideoPoolService;
import cn.fw.data.base.domain.common.Message;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.framework.AopContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2022/10/12 0:23
 * @Description
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class TestBizService {

    private final AccountService accountService;
    private final LivePoolService livePoolService;
    private final VideoPoolService videoPoolService;
    private final CommonBizService commonBizService;
    private final ActivityThemeService activityThemeService;

    /*
     * A(不+事务注解)中调用B(+事务注解):
     *      A无事务, B无事务
     *
     * A(+事务注解)中调用B(+事务注解):
     *      A有事务, B无事务
     *
     * 解决方案:
     *      1: bean中自己注入自己, 让A中调用B方法使用注入的自己去调用B方法(testBizService.methodB(), 而不是this.methodB())
     *      2: 配置类上声明@EnableAspectJAutoProxy(exposeProxy = true), 然后业务方法((TestBizService) AopContext.currentProxy()).methodB()
     * AopContext.currentProxy()其实就是获取的代理对象, bean中注入的自己也是代理对象
     */

    private TestBizService testBizService;
    private final StaffStatusApi staffStatusApi;
    private final AttendanceApi attendanceApi;

    @Autowired
    public void inject(TestBizService testBizService) {
        this.testBizService = testBizService;
    }

    @Transactional(rollbackFor = Exception.class)
    public void methodA() {
        Message<UserStatusVo> details = staffStatusApi.getUserStatusDetails(Arrays.asList(760L, 1545L, 1486L, 1342L), 1611975222000L, 1675047222937L);
        UsersScheduleDetailsDto dto = new UsersScheduleDetailsDto();
        dto.setGroupId(2L);
        dto.setUserIds(Arrays.asList(760L, 1545L, 1486L, 1342L));
        dto.setStartTime(new Date(1646818372000L));
        dto.setEndTime(new Date(1678354372000L));
        Message<List<UsersScheduleDetailsVo>> scheduleDetails = attendanceApi.getUsersScheduleDetails(dto);
        Account account = new Account();
        account.setAccount("methodA");
        account.setYn(Boolean.FALSE);
        account.setGroupId(2L);
        account.setType(PlatformEnum.DY);
        account.setUserId(1545L);
        account.setShopId(11L);
        accountService.save(account);
        try {
            //this.methodB();
            //testBizService.methodB();
            //testBizService1.methodB();
            ((TestBizService) AopContext.currentProxy()).methodB();
        } catch (Exception e) {
            e.printStackTrace();
        }
        //int i = 1 / 0;
    }

    @Transactional(rollbackFor = Exception.class/*, propagation = Propagation.REQUIRES_NEW*/)
    public void methodB() {
        Account account = new Account();
        account.setAccount("methodB");
        account.setYn(Boolean.FALSE);
        account.setGroupId(2L);
        account.setType(PlatformEnum.DY);
        account.setUserId(1545L);
        account.setShopId(11L);
        accountService.save(account);
        int i = 1 / 0;
    }

    /**
     * 重新根据配置按月刷直播短视频有效性数据
     *
     * @param month 指定月份
     */
    public void retryCalcValid(Date month) {
        if (Objects.isNull(month))
            month = new Date();
        Date monthBegin = DateUtil.monthBegin(month);
        Date monthEnd = DateUtil.monthEnd(month);
        Map<Long, Account> accountMap = this.getAccountMap();
        Map<Long, ActivityTheme> activityMap = this.getActivityMap();
        List<LivePool> newLiveResult = livePoolService.lambdaQuery()
                .between(LivePool::getLiveStartTime, monthBegin, monthEnd)
                .list()
                .stream()
                .map(item -> {
                    Account account = accountMap.get(item.getAccountId());
                    ActivityTheme activity = activityMap.get(item.getThemeId());
                    if (Objects.isNull(account) || Objects.isNull(activity)) return null;
                    Long configGroupId = activity.getConfigGroupId();
                    List<Integer> list = commonBizService.newValidLive(account, configGroupId, this.livePool2Dto(item));
                    List<Integer> oldRetain = this.getOldRetain(item.getInvalidReason());
                    commonBizService.invalidReason2String(oldRetain);
                    String newResult = this.jointInvalidReason(commonBizService.invalidReason2String(oldRetain),
                            commonBizService.invalidReason2String(list));
                    if (StringUtils.hasText(newResult))
                        item.setValidLive(0);
                    else
                        item.setValidLive(1);
                    item.setInvalidReason(newResult);
                    return item;
                })
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
        List<VideoPool> newVideoResult = videoPoolService.lambdaQuery()
                .between(VideoPool::getPublishTime, monthBegin, monthEnd)
                .list()
                .stream()
                .map(item -> {
                    Account account = accountMap.get(item.getAccountId());
                    ActivityTheme activity = activityMap.get(item.getThemeId());
                    if (Objects.isNull(account) || Objects.isNull(activity)) return null;
                    Long configGroupId = activity.getConfigGroupId();
                    List<Integer> list = commonBizService.newValidVideo(account, configGroupId, this.videoPool2Dto(item));
                    List<Integer> oldRetain = this.getOldRetain(item.getInvalidReason());
                    commonBizService.invalidReason2String(oldRetain);
                    String newResult = this.jointInvalidReason(commonBizService.invalidReason2String(oldRetain),
                            commonBizService.invalidReason2String(list));
                    if (StringUtils.hasText(newResult))
                        item.setValidVideo(0);
                    else
                        item.setValidVideo(1);
                    item.setInvalidReason(newResult);
                    return item;
                })
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
        if (!newLiveResult.isEmpty())
            livePoolService.updateBatchById(newLiveResult, Integer.MAX_VALUE);
        if (!newVideoResult.isEmpty())
            videoPoolService.updateBatchById(newVideoResult, Integer.MAX_VALUE);
    }

    private Map<Long, Account> getAccountMap() {
        return accountService.getAllIgnoreLogicDel()
                .stream()
                .collect(Collectors.toMap(Account::getId, Function.identity()));
    }

    private Map<Long, ActivityTheme> getActivityMap() {
        return activityThemeService.lambdaQuery()
                .list()
                .stream()
                .collect(Collectors.toMap(ActivityTheme::getId, Function.identity()));
    }

    private List<Integer> getOldRetain(String oldInvalidReason) {
        List<Integer> oldRetain = new ArrayList<>();
        if (Objects.nonNull(oldInvalidReason)) {
            Arrays.stream(oldInvalidReason.split(","))
                    .forEach(item -> {
                        Integer type = Integer.valueOf(item.trim());
                        if (Objects.equals(type, InvalidReasonEnum.COVER.getValue()))
                            oldRetain.add(InvalidReasonEnum.COVER.getValue());
                        else if (Objects.equals(type, InvalidReasonEnum.TAG.getValue()))
                            oldRetain.add(InvalidReasonEnum.TAG.getValue());
                        else if (Objects.equals(type, InvalidReasonEnum.LOSE_PLAYBACK.getValue()))
                            oldRetain.add(InvalidReasonEnum.LOSE_PLAYBACK.getValue());
                    })
            ;
        }
        return oldRetain;
    }

    private LivePoolDto.LiveDTO livePool2Dto(LivePool db) {
        LivePoolDto.LiveDTO dto = PublicUtil.copy(db, LivePoolDto.LiveDTO.class);
        return dto;
    }

    private VideoPoolDto.VideoDTO videoPool2Dto(VideoPool db) {
        VideoPoolDto.VideoDTO dto = PublicUtil.copy(db, VideoPoolDto.VideoDTO.class);
        return dto;
    }

    private String jointInvalidReason(String str1, String str2) {
        String result = "";
        if (StringUtils.hasText(str1))
            result = str1;
        if (StringUtils.hasText(str2)) {
            if (StringUtils.hasText(str1))
                result += ",";
            result += str2;
        }
        return result;
    }

    /**
     * 重刷话题存储格式
     */
    public void retryCalcTopic() {
        activityThemeService.updateBatchById(activityThemeService.list()
                .stream()
                .peek(item -> {
                    String topic = item.getTopic();
                    if (topic.contains("#")) {
                        item.setTopic(JSON.toJSONString(Arrays.stream(topic.split("#"))
                                .skip(1)
                                .map(item1 -> {
                                    if (item1.endsWith(","))
                                        item1 = item1.substring(0, item1.length() - 1);
                                    TopicDto dto = new TopicDto();
                                    dto.setTopic(item1);
                                    return dto;
                                })
                                .collect(Collectors.toList())
                        ));
                    }
                })
                .collect(Collectors.toList())
        );
    }
}
