package cn.fw.dalaran.server.controller.web;

import cn.fw.backlog.sdk.api.result.MeddleItem;
import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.common.exception.InterveneTypeEnum;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.enums.account.AccountEnableAnalysisEnum;
import cn.fw.dalaran.domain.enums.account.AccountTypeEnum;
import cn.fw.dalaran.rpc.backlog.TodoInterveneRpc;
import cn.fw.dalaran.rpc.dto.StaffBaseInfoRpc;
import cn.fw.dalaran.rpc.ehr.EhrRpc;
import cn.fw.dalaran.service.biz.CommonBizService;
import cn.fw.dalaran.service.biz.LocalCacheService;
import cn.fw.dalaran.service.biz.account.AccountChangeBizService;
import cn.fw.dalaran.service.biz.account.AccountConfirmBizService;
import cn.fw.dalaran.service.biz.config.ConfigCommonBizService;
import cn.fw.dalaran.service.biz.config.ConfigGroupBizService;
import cn.fw.dalaran.service.biz.report.calculate.CalThemeScoreBizService;
import cn.fw.dalaran.service.biz.report.calculate.CalThemeTargetBizService;
import cn.fw.dalaran.service.biz.report.calculate.ThemeSnapshotTaskBizService;
import cn.fw.dalaran.service.biz.task.PushSalaryBizService;
import cn.fw.dalaran.service.biz.task.ReportTaskBizService;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.dalaran.service.data.LivePoolService;
import cn.fw.dalaran.service.data.VideoPoolService;
import cn.fw.data.base.domain.common.Message;
import cn.fw.ehr.sdk.api.mq.StaffPostChangeEvent;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.annotation.IgnoreUserToken;
import cn.fw.security.auth.client.enums.AuthType;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2022/10/12 0:23
 * @Description
 * @ignore
 */
@Slf4j
@Validated
@IgnoreUserToken
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@RequestMapping("/web/test")
@IgnoreAuth
public class TestController {
    private final EhrRpc ehrRpc;
    private final AccountService accountService;
    private final TestBizService testBizService;
    private final LivePoolService livePoolService;
    private final VideoPoolService videoPoolService;
    private final TodoInterveneRpc todoInterveneRpc;
    private final CommonBizService commonBizService;
    private final LocalCacheService localCacheService;
    private final PushSalaryBizService pushSalaryBizService;
    private final ActivityThemeService activityThemeService;
    private final ReportTaskBizService reportTaskBizService;
    private final ConfigGroupBizService configGroupBizService;
    private final ConfigCommonBizService configCommonBizService;
    private final CalThemeScoreBizService calThemeScoreBizService;
    private final AccountChangeBizService accountChangeBizService;
    private final AccountConfirmBizService accountConfirmBizService;
    private final CalThemeTargetBizService calThemeTargetBizService;
    private final ThemeSnapshotTaskBizService themeSnapshotTaskBizService;


    /**
     * @ignore
     */
    @GetMapping("/methodA")
    public void methodA() {
        testBizService.methodA();
    }

    /**
     * @ignore
     */
    @GetMapping("/getBelongManagerId")
    public void getBelongManagerId(Long groupId, Long shopId, String level) {
        localCacheService.getBelongManagerId(groupId, shopId, RoleMsgEnum.codeOf(level));
    }

    /**
     * 推送账号配置确认干预待办
     *
     * @ignore
     */
    @GetMapping("/retry/push/intervene/todo")
    public Message<Void> pushInterveneTodo(Long groupId, Long accountId) {
        List<Account> needConfirmList = new LinkedList<>();
        if (PublicUtil.isNotEmpty(groupId)) {
            needConfirmList = accountService.list(Wrappers.<Account>lambdaQuery()
                    .eq(Account::getGroupId, groupId)
                    .in(Account::getEnableAnalysis, AccountEnableAnalysisEnum.ADD, AccountEnableAnalysisEnum.THEME_ZERO));
        } else if (PublicUtil.isNotEmpty(accountId)) {
            Account account = accountService.getById(accountId);
            needConfirmList.add(account);
            groupId = account.getGroupId();
        } else {
            return Message.failure();
        }
        List<MeddleItem> thresholdList = todoInterveneRpc.getThresholdList(groupId, InterveneTypeEnum.ACCOUNT_CONFIRM.getValue());
        if (PublicUtil.isEmpty(thresholdList)) {
            return Message.failure();
        }
        final List<MeddleItem> sortList = thresholdList.stream()
                .sorted(Comparator.comparing(MeddleItem::getThreshold).reversed()).collect(Collectors.toList());
        needConfirmList.forEach(account -> accountConfirmBizService.interveneCheck(account, sortList));
        return Message.success();
    }

    /**
     * 重试计算得分
     *
     * @ignore
     */
    @GetMapping("/retry/cal/score")
    public Message<Void> retryCalScore(Long themeId) {
        if (PublicUtil.isNotEmpty(themeId)) {
            ActivityTheme theme = activityThemeService.getById(themeId);
            calThemeScoreBizService.calVideoOrLiveScore(theme);
        } else {
            reportTaskBizService.handleThemeScoreCal();
        }
        return Message.success();
    }

    /**
     * 配置组变更
     *
     * @ignore
     */
    @GetMapping("/retry/config/group/change")
    public Message<Void> retrySetConfigRange(Long configId) {
        configGroupBizService.syncGroupConfigToTheme(configId);
        return Message.success();
    }

    /**
     * 重试生成快照
     *
     * @ignore
     */
    @GetMapping("/retry/create/theme/snapshot")
    public Message<Void> retryCreateThemeSnapshot(Long themeId) {
        ActivityTheme theme = activityThemeService.getById(themeId);
        themeSnapshotTaskBizService.createThemeSnapshot(theme);
        return Message.success();
    }

    /**
     * 重试同步前一日视频
     *
     * @ignore
     */
    @GetMapping("/retry/sync/pre/video")
    public Message<Void> syncPreDayVideo() {
        reportTaskBizService.syncPreDayVideo();
        return Message.success();
    }

    /**
     * 重试刷新主题有效性配置备份
     *
     * @ignore
     */
    @GetMapping("/retry/set/activity/valid/config")
    public Message<Void> activityValidConfigSet() {
        List<ActivityTheme> list = activityThemeService.list();
        for (ActivityTheme theme : list) {
            theme.setValidConfigJson(configCommonBizService.getValidConfigJson(theme.getConfigGroupId()));
        }
        activityThemeService.updateBatchById(list);
        return Message.success();
    }

    /**
     * 重新按最新配置刷新数据有效性
     */
    @GetMapping("/retryCalcValid")
    public Message<Void> retryCalcValid(Date month) {
        testBizService.retryCalcValid(month);
        return Message.success();
    }

    /**
     * 重刷话题存储格式
     */
    @GetMapping("/retryCalcTopic")
    public Message<Void> retryCalcTopic() {
        testBizService.retryCalcTopic();
        return Message.success();
    }

    /**
     * 重新上报绩效
     */
    @GetMapping("/retry/push/salary")
    public Message<Void> retryPushSalary(Long groupId, Long time) {
        pushSalaryBizService.pushGroupSalary(groupId, new Date(time));
        return Message.success();
    }

    /**
     * 刷新账号归属
     */
    @GetMapping("/retry/set/account/shop")
    @Transactional(rollbackFor = Exception.class)
    public Message<Void> retrySetAccountShop(Long groupId) {
        List<Account> accountList = accountService.list(Wrappers.<Account>lambdaQuery()
                .eq(Account::getGroupId, groupId)
                .eq(Account::getAccountType, AccountTypeEnum.STAFF.getValue())
                .eq(Account::getYn, true));
        Map<Long, List<Account>> staffAccount = accountList.stream().collect(Collectors.groupingBy(Account::getUserId));
        List<StaffBaseInfoRpc> staffInfoBatch = ehrRpc.getStaffInfoBatch(new ArrayList<>(staffAccount.keySet()));
        List<ActivityTheme> activityThemes = activityThemeService.getGoingActivityTheme();
        for (StaffBaseInfoRpc staff : staffInfoBatch) {
            List<Account> accounts = staffAccount.get(staff.getId());
            accounts.forEach(account -> setShop(staff, account, activityThemes));
        }
        return Message.success();
    }

    /**
     * 按账号处理
     *
     * @param staff
     * @param account
     * @param themes
     */
    @Transactional(rollbackFor = Exception.class)
    public void setShop(StaffBaseInfoRpc staff, Account account, List<ActivityTheme> themes) {
        if (staff.getShopId().equals(account.getShopId())) {
            return;
        }
        account.setShopId(staff.getShopId());
        account.setShopName(staff.getShopName());
        accountService.updateById(account);
        //刷新作品归属门店
        List<Long> themeIds = themes.stream().map(ActivityTheme::getId).collect(Collectors.toList());
        livePoolService.update(Wrappers.<LivePool>lambdaUpdate()
                .in(LivePool::getThemeId, themeIds)
                .eq(LivePool::getAccountId, account.getId())
                .set(LivePool::getBelongShopId, account.getShopId())
        );
        videoPoolService.update(Wrappers.<VideoPool>lambdaUpdate()
                .in(VideoPool::getThemeId, themeIds)
                .eq(VideoPool::getAccountId, account.getId())
                .set(VideoPool::getBelongShopId, account.getShopId())
        );
    }

}
