package cn.fw.dalaran.server.controller.web.config;

import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.dalaran.common.page.PageHelper;
import cn.fw.dalaran.domain.dto.config.GroupQueryDto;
import cn.fw.dalaran.domain.dto.config.GroupSaveDto;
import cn.fw.dalaran.domain.vo.config.ConfigGroupDetailVo;
import cn.fw.dalaran.domain.vo.config.ConfigGroupVo;
import cn.fw.dalaran.service.biz.config.ConfigGroupBizService;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 配置组设置接口
 *
 * @author wmy3969
 * @date 2022-4-28 14:41:47
 */
@Slf4j
@Validated
@IgnoreAuth
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@RequestMapping("/config/group")
public class ConfigGroupController {
    private final ConfigGroupBizService configGroupBizService;

    /**
     * 查询配置组列表
     *
     * @param queryDto 查询参数
     * @return 符合条件的配置列表
     * @ignoreParams pageHelper user
     */
    @GetMapping("/list")
    public Message<PageHelper<ConfigGroupVo>> getConfigGroupList(@CurrentUser LoginAuthBean user,
                                                                 PageHelper<ConfigGroupVo> pageHelper,
                                                                 GroupQueryDto queryDto) {
        return success(configGroupBizService.getConfigGroupList(user, pageHelper, queryDto));
    }

    /**
     * 查询配置组详情（新增无需传递id）
     *
     * @param draft 查询参数
     * @param id    查询参数
     * @return 详情
     * @ignoreParams user
     */
    @GetMapping("/detail")
    public Message<ConfigGroupDetailVo> getConfigGroupList(
            @RequestParam(value = "draft") @NotNull(message = "未指明确是否草稿") Boolean draft,
            @RequestParam(value = "id", required = false) Long id) {
        return success(configGroupBizService.getConfigGroupDetail(draft, id));
    }

    /**
     * 保存配置组
     *
     * @param param 新的配置参数
     * @return 操作结果
     */
    @PostMapping("/save")
    public Message<Void> saveOrUpdate(@CurrentUser LoginAuthBean user, @RequestBody @Valid GroupSaveDto param) {
        configGroupBizService.save(user, param);
        return success();
    }

    /**
     * 删除配置组
     *
     * @param configGroupId 配置组id
     * @return 操作结果
     */
    @GetMapping("/del")
    public Message<Void> del(Long draftId, Long configGroupId) {
        configGroupBizService.del(draftId, configGroupId);
        return success();
    }
}
