package cn.fw.dalaran.server.converter;

import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.type.TypeFactory;
import com.fasterxml.jackson.databind.util.Converter;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2023/6/6 10:30
 * @Description 字符串转为List<Long>
 */
public class String2ListConverter implements Converter<String, List<Long>> {
    /**
     * Main conversion method.
     *
     * @param value 源数据
     */
    @Override
    public List<Long> convert(String value) {
        if (!StringUtils.hasText(value))
            return new ArrayList<>();
        try {
            return Arrays.stream(value.split(","))
                    .map(item -> Long.valueOf(item.trim()))
                    .collect(Collectors.toList());
        } catch (Exception e) {
            return new ArrayList<>();
        }
    }

    /**
     * Method that can be used to find out actual input (source) type; this
     * usually can be determined from type parameters, but may need
     * to be implemented differently from programmatically defined
     * converters (which cannot change static type parameter bindings).
     *
     * @param typeFactory
     * @since 2.2
     */
    @Override
    public JavaType getInputType(TypeFactory typeFactory) {
        return null;
    }

    /**
     * Method that can be used to find out actual output (target) type; this
     * usually can be determined from type parameters, but may need
     * to be implemented differently from programmatically defined
     * converters (which cannot change static type parameter bindings).
     *
     * @param typeFactory
     * @since 2.2
     */
    @Override
    public JavaType getOutputType(TypeFactory typeFactory) {
        return null;
    }
}
