package cn.fw.dalaran.server.rocketMQ;

import cn.fw.dalaran.common.constants.Constants;
import cn.fw.dalaran.service.biz.account.AccountChangeBizService;
import cn.fw.dalaran.service.biz.account.AccountTaskBizService;
import cn.fw.ehr.sdk.api.mq.StaffLeaveEvent;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.stereotype.Component;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2022/03/14 14:04
 * @Description 员工离职消息队列消息监听, 删除员工自媒体账号
 */
@Slf4j
@Component
@RequiredArgsConstructor
@RocketMQMessageListener(
        topic = StaffLeaveEvent.TOPIC,
        consumerGroup = Constants.APPLICATION_NAME + "-" + StaffLeaveEvent.TOPIC)
public class StaffLeaveEventConsumer implements RocketMQListener<StaffLeaveEvent> {
    private final AccountTaskBizService accountTaskBizService;

    /**
     * 员工离职, 删除员工自媒体账号
     *
     * @param staffLeaveEvent 封装了员工离职信息
     */
    @Override
    public void onMessage(StaffLeaveEvent staffLeaveEvent) {
        log.info("收到员工离职消息 message:{}", JSON.toJSONString(staffLeaveEvent));
        accountTaskBizService.handleStaffLeave(staffLeaveEvent.getId());
    }
}
