package cn.fw.dalaran.server.schedule;

import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.rpc.attendance.AttendanceRpc;
import cn.fw.dalaran.rpc.attendance.dto.StaffWorkStatusDto;
import cn.fw.dalaran.rpc.erp.UserRoleRpcService;
import cn.fw.dalaran.rpc.oop.OopRpc;
import cn.fw.dalaran.rpc.oop.dto.GroupDTO;
import cn.fw.dalaran.service.biz.task.AccountStaffCheckBizService;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.erp.sdk.api.result.UserRoleDataRange;
import cn.fw.erp.sdk.api.result.UserRoleInfo;
import cn.fw.oop.sdk.result.ShopInfoVo;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 账号相关定时任务
 *
 * @author wmy3969
 * @version 1.0
 * @date 2022/4/13 15:00
 */
@Slf4j
@Component
@RequiredArgsConstructor
@SuppressWarnings("Duplicates")
public class AccountTask {
    private final OopRpc oopRpc;

    private final AttendanceRpc attendanceRpc;
    private final AccountService accountService;
    private final UserRoleRpcService userRoleRpcService;
    private final AccountStaffCheckBizService accountStaffCheckBizService;

    /**
     * 每天12点检查人员昨天工作状态
     */
    @Scheduled(cron = "0 00 12 * * *")
    public void checkStaffWorkStatus() {
        List<Account> list = accountService.lambdaQuery()
                .eq(Account::getAccountStatus, 0)
                .list();
        if (CollectionUtils.isEmpty(list))
            return;
        List<StaffWorkStatusDto> staffWorkStatus = attendanceRpc.getStaffWorkStatus(list.stream()
                        .map(Account::getUserId)
                        .collect(Collectors.toList()),
                System.currentTimeMillis() - 36 * 3600 * 1000L + 10 * 60 * 1000);
        if (CollectionUtils.isEmpty(staffWorkStatus))
            return;
        accountService.lambdaUpdate()
                .in(Account::getUserId, staffWorkStatus.stream()
                        .filter(item -> Objects.equals(item.getWorkStatus(), 1))
                        .map(StaffWorkStatusDto::getUserId)
                        .distinct()
                        .collect(Collectors.toList())
                )
                .set(Account::getAccountStatus, 1)
                .update();
    }


    /**
     * 每30分检查一次账号人员角色
     */
//    @Scheduled(cron = "0 0/30 * * * ? ")
    public void checkStaffRoleRange() {
        List<GroupDTO> allGroups = oopRpc.getAllGroups();
        for (GroupDTO groupDTO : allGroups) {
            List<ShopInfoVo> allShopsByGroup = oopRpc.getAllShopsByGroup(groupDTO.getId());
            List<Long> shopIds = allShopsByGroup.stream().map(ShopInfoVo::getId).collect(Collectors.toList());
            List<Account> accountList = accountService.list(Wrappers.<Account>lambdaQuery().eq(Account::getGroupId, groupDTO.getId()));
            List<UserRoleInfo> users = userRoleRpcService.getUsers(shopIds, RoleMsgEnum.XSGW.getCode());
            Map<Long, List<UserRoleDataRange>> staffMap = users.stream().collect(Collectors.toMap(UserRoleInfo::getUserId, UserRoleInfo::getDataRangeList));
            for (Account account : accountList) {
                accountStaffCheckBizService.checkStaffRoleRange(account, staffMap);
            }
        }
    }
}
