package cn.fw.dalaran.server.schedule;

import cn.fw.dalaran.common.constants.CacheName;
import cn.fw.dalaran.service.biz.task.ReportTaskBizService;
import cn.fw.starter.redis.redis.RedisUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

/**
 * 报表相关定时任务
 *
 * @author: lym
 * @date: 2023/5/12
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class ReportTask {
    private final RedisUtil redisUtil;
    private final ReportTaskBizService reportTaskBizService;

    /**
     * 定时生成活动快照(每1小时执行一次)
     */
    @Scheduled(fixedRate = 60 * 60 * 1000, initialDelay = 60 * 1000)
    void createThemeSnapshot() {
        String redisKey = CacheName.CREATE_THEME_SNAPSHOT_TASK;
        try {
            if (redisUtil.getLock(redisKey)) {
                log.info("定时生成活动快照开始");
                reportTaskBizService.createThemeSnapshot();
            }
        } catch (Exception e) {
            log.warn("定时生成活动快照异常", e);
        } finally {
            redisUtil.releaseLock(redisKey);
            log.info("定时生成活动快照结束");
        }
    }

    /**
     * 定时计算主题得分（30分钟计算一次）
     */
    @Scheduled(fixedRate = 30 * 60 * 1000, initialDelay = 60 * 1000)
    void calScore() {
        String redisKey = CacheName.CAL_THEME_SCORE_TASK;
        try {
            log.info("定时计算未结束主题得分开始");
            if (redisUtil.getLock(redisKey)) {
                reportTaskBizService.handleThemeScoreCal();
            }
            log.info("定时计算未结束主题得分结束");
        } catch (Exception e) {
            log.warn("定时计算未结束主题得分异常", e);
        } finally {
            redisUtil.releaseLock(redisKey);
            log.info("定时计算未结束主题得分结束");
        }
    }

    /**
     * 每天0点0分1秒同步前一天视频（停用）
     */
//    @Scheduled(cron = "1 0 0 * * ?")
    void syncPreDayVideo() {
        String redisKey = "dalaran:sync:pre:day:video";
        try {
            if (redisUtil.getLock(redisKey)) {
                log.info("同步前一天短视频、直播数据开始");
                reportTaskBizService.syncPreDayVideo();
            }
        } catch (Exception e) {
            log.warn("同步前一天短视频、直播数据异常", e);
        } finally {
            redisUtil.releaseLock(redisKey);
            log.info("同步前一天短视频、直播数据结束");
        }
    }
}
