package cn.fw.dalaran.server.schedule;

import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.rpc.oop.OopRpc;
import cn.fw.dalaran.rpc.oop.dto.GroupDTO;
import cn.fw.dalaran.service.biz.task.PushSalaryBizService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.temporal.TemporalAdjusters;
import java.util.List;

/**
 * 推送社保确认待办
 *
 * @author: lym
 * @date: 2023/7/29
 */
@Slf4j
@Component
@RequiredArgsConstructor
@SuppressWarnings("Duplicates")
public class SalaryTask {
    private final OopRpc oopRpc;
    private final PushSalaryBizService pushSalaryBizService;

    /**
     * 绩效定时推送任务
     */
    @Scheduled(cron = "0 3 0 * * ?")
    public void pushSalary() {
        LocalDate now = LocalDate.now();
        List<GroupDTO> allGroups = oopRpc.getAllGroups();

        //补偿上月数据
        LocalDate lastMonthEndDay = now.minusMonths(1).with(TemporalAdjusters.lastDayOfMonth());
        LocalDate maxPushDate = lastMonthEndDay.plusDays(6);
        if (maxPushDate.isAfter(now)) {
            for (GroupDTO group : allGroups) {
                try {
                    pushSalaryBizService.pushGroupSalary(group.getId(), DateUtil.dayEnd(DateUtil.localDate2Date(lastMonthEndDay)));
                } catch (Exception e) {
                    log.error("推送集团：{}失败", group.getId(), e);
                }
            }
        }

        //推送本月数据
        LocalDate currentMonth = now.minusDays(1);
        if (now.getYear() == currentMonth.getYear() && now.getMonth() == currentMonth.getMonth()) {
            for (GroupDTO group : allGroups) {
                try {
                    pushSalaryBizService.pushGroupSalary(group.getId(), DateUtil.dayEnd(DateUtil.localDate2Date(currentMonth)));
                } catch (Exception e) {
                    log.error("推送集团：{}失败", group.getId(), e);
                }
            }
        }
    }
}
