package cn.fw.dalaran.service.biz.common;


import cn.fw.dalaran.common.domain.dto.ToDoDto;
import cn.fw.dalaran.common.domain.enums.ToDoTypeEnum;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.common.TodoHistory;
import cn.fw.dalaran.rpc.backlog.TodoRpc;
import cn.fw.dalaran.service.data.common.TodoHistoryService;
import com.alibaba.fastjson.JSONObject;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.Map;

/**
 *
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ToDoBizService {
    private final TodoRpc todoRpc;
    private final TodoHistoryService todoHistoryService;

    /**
     * 推送待办
     *
     * @param todoHistory 待办记录
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public void pushToDo(TodoHistory todoHistory) {
        try {
            todoHistory.setPlanTime(PublicUtil.isEmpty(todoHistory.getPlanTime()) ? new Date() : todoHistory.getPlanTime());
            ToDoDto toDoDto = new ToDoDto();
            toDoDto.setTodoCode(todoHistory.getTodoCode());
            toDoDto.setDataId(todoHistory.getDataId().toString());
            toDoDto.setPlanTime(todoHistory.getPlanTime());
            toDoDto.setExpireTime(todoHistory.getExpireTime());
            toDoDto.setRoleCode(todoHistory.getRoleCode());
            toDoDto.setShopId(todoHistory.getShopId());
            toDoDto.setUserId(todoHistory.getUserId());
            if (PublicUtil.isNotEmpty(todoHistory.getDynamicInfo())) {
                toDoDto.setDynamicMap((Map<String, String>) JSONObject.parseObject(todoHistory.getDynamicInfo(), Map.class));
            }
            try {
                if (PublicUtil.isNotEmpty(todoHistory.getExtra())) {
                    toDoDto.setExtraData((Map<String, Object>) JSONObject.parseObject(todoHistory.getExtra(), Map.class));
                }
            } catch (Exception e) {
                log.error("转换扩展参数异常，id:{}", todoHistory.getId());
            }
            todoRpc.toDoAdd(toDoDto);
            todoHistory.setSend(Boolean.TRUE);
            todoHistoryService.updateById(todoHistory);
        } catch (Exception e) {
            log.error("推送待办异常，待办id:{}", todoHistory.getId(), e);
        }
    }

    /**
     * 完成待办
     *
     * @param todoHistory 待办记录
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public void finishToDo(TodoHistory todoHistory) {
        try {
            todoHistory.setCommitTime(PublicUtil.isEmpty(todoHistory.getCommitTime()) ? new Date() : todoHistory.getCommitTime());
            ToDoDto toDoDto = new ToDoDto();
            toDoDto.setTodoCode(todoHistory.getTodoCode());
            toDoDto.setDataId(todoHistory.getDataId().toString());
            toDoDto.setUserId(todoHistory.getUserId());
            toDoDto.setCommitTime(todoHistory.getCommitTime());
            todoRpc.toDoComplete(toDoDto);
            todoHistory.setTodoDone(Boolean.TRUE);
            todoHistoryService.updateById(todoHistory);
        } catch (Exception e) {
            log.error("完成待办异常，待办id:{}", todoHistory.getId(), e);
        }
    }

    /**
     * 取消待办
     *
     * @param todoHistory 待办记录
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public void cancelToDo(TodoHistory todoHistory) {
        try {
            todoHistory.setCommitTime(PublicUtil.isEmpty(todoHistory.getCommitTime()) ? new Date() : todoHistory.getCommitTime());
            ToDoDto toDoDto = new ToDoDto();
            toDoDto.setTodoCode(todoHistory.getTodoCode());
            toDoDto.setDataId(todoHistory.getDataId().toString());
            toDoDto.setUserId(todoHistory.getUserId());
            toDoDto.setCommitTime(todoHistory.getCommitTime());
            todoRpc.toDoCancel(toDoDto);
            todoHistory.setTodoDone(Boolean.TRUE);
            todoHistoryService.updateById(todoHistory);
        } catch (Exception e) {
            log.error("取消待办异常，待办id:{}", todoHistory.getId(), e);
        }
    }

}
