package cn.fw.dalaran.service.biz.event;

import cn.fw.dalaran.common.domain.AccountEventDetailsMsg;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.service.biz.account.AccountChangeBizService;
import cn.fw.dalaran.service.data.AccountService;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2023/6/13 10:19
 * @Description 事件消费biz业务
 */
@Slf4j
@Component
@RequiredArgsConstructor
@SuppressWarnings("Duplicates")
public class SpringEventConsumerBizService {

    private final AccountService accountService;
    private final AccountChangeBizService accountChangeBizService;

    /**
     * 自媒体账号新增事件
     *
     * @param event 事件消息
     */
    @Async(value = "dalaranThreadPool")
    @EventListener(AccountEventDetailsMsg.class)
    public void consumerEventMsg(AccountEventDetailsMsg event) {
        log.info(String.format("事件监听器收到消息: %s", JSON.toJSONString(event)));
        Account account = accountService.getByIdIgnoreLogicDel(event.getAccountId());
        /*LockSupport.parkNanos(TimeUnit.SECONDS.toNanos(5));// 休眠5秒, 防止变更数据未落库, 查询到了变更前的旧账号数据
        while (Objects.isNull(account)) {// 因为是异步, 可能导致数据插入未落库, 这边就先执行了, 找不到新加账号
            account = accountService.getByIdIgnoreLogicDel(event.getAccountId());
            LockSupport.parkNanos(TimeUnit.MILLISECONDS.toNanos(50));
        }*/
        accountChangeBizService.accountDataRestore(account, null, null);
    }
}
