package cn.fw.dalaran.service.biz.report.calculate;

import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.db.snapshot.ConsultantSnapshot;
import cn.fw.dalaran.domain.db.snapshot.ManagerSnapshot;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author: LYM
 * @date: 2023/12/16 10:01
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class CalThemeTargetBizService {

    /**
     * 计算发布人目标
     *
     * @param theme
     * @param liveList
     * @param videoList
     * @return
     */
    public void calStaffTarget(ActivityTheme theme, List<ConsultantSnapshot> staffList,
                                List<LivePool> liveList, List<VideoPool> videoList) {
        Map<Long, List<LivePool>> staffLiveMap = liveList.stream().filter(r -> r.getValidLive() > 1)
                .collect(Collectors.groupingBy(LivePool::getBelongUserId));
        Map<Long, List<VideoPool>> staffVideoMap = videoList.stream().filter(r -> r.getValidVideo() > 1)
                .collect(Collectors.groupingBy(VideoPool::getBelongUserId));
        BigDecimal liveRate = theme.getLiveRate();
        BigDecimal videoRate = theme.getVideoRate();
        for (ConsultantSnapshot snapshot : staffList) {
            List<LivePool> livePools = staffLiveMap.getOrDefault(snapshot.getStaffId(), new ArrayList<>());
            snapshot.setNeedLiveNum(theme.getLiveValidTimes());
            snapshot.setLiveQualifiedNum(PublicUtil.isEmpty(livePools)
                    ? 0 : livePools.size() > theme.getLiveValidTimes()
                    ? theme.getLiveValidTimes() : livePools.size());
            List<VideoPool> videoPools = staffVideoMap.getOrDefault(snapshot.getStaffId(), new ArrayList<>());
            snapshot.setNeedVideoNum(theme.getVideoValidTimes());
            snapshot.setVideoQualifiedNum(PublicUtil.isEmpty(videoPools)
                    ? 0 : videoPools.size() > theme.getVideoValidTimes()
                    ? theme.getVideoValidTimes() : videoPools.size());
            snapshot.setLiveRate(liveRate);
            snapshot.setVideoRate(videoRate);
        }
    }

    /**
     * 计算门店目标
     *
     * @param theme
     * @param staffList
     * @return
     */
    public void calShopTarget(ActivityTheme theme, List<ManagerSnapshot> shopList, List<ConsultantSnapshot> staffList) {
        Map<Long, List<ConsultantSnapshot>> shopStaffMap = staffList.stream()
                .collect(Collectors.groupingBy(ConsultantSnapshot::getShopId));
        BigDecimal liveRate = theme.getLiveRate();
        BigDecimal videoRate = theme.getVideoRate();
        for (ManagerSnapshot managerSnapshot : shopList) {
            List<ConsultantSnapshot> shopStaffs = shopStaffMap.getOrDefault(managerSnapshot.getShopId(), new LinkedList<>());
            managerSnapshot.setConsultantNum(shopStaffs.size());
            managerSnapshot.setNeedLiveNum(shopStaffs.stream().mapToInt(ConsultantSnapshot::getNeedLiveNum).sum());
            managerSnapshot.setNeedVideoNum(shopStaffs.stream().mapToInt(ConsultantSnapshot::getNeedVideoNum).sum());

            managerSnapshot.setLiveQualifiedNum(shopStaffs.stream().mapToInt(ConsultantSnapshot::getLiveQualifiedNum).sum());
            managerSnapshot.setVideoQualifiedNum(shopStaffs.stream().mapToInt(ConsultantSnapshot::getVideoQualifiedNum).sum());
            managerSnapshot.setLiveRate(liveRate);
            managerSnapshot.setVideoRate(videoRate);
        }
    }
}

