package cn.fw.dalaran.service.biz.report.live;

import cn.fw.common.exception.BusinessException;
import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.common.page.PageHelper;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PageUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.dto.ManagerInfo;
import cn.fw.dalaran.domain.dto.report.EvlFormulaDto;
import cn.fw.dalaran.domain.dto.report.score.ScoreResultDto;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import cn.fw.dalaran.domain.enums.config.WorksTypeEnum;
import cn.fw.dalaran.domain.enums.report.AnalysisTypeEnum;
import cn.fw.dalaran.domain.enums.report.InvalidReasonEnum;
import cn.fw.dalaran.domain.enums.report.ReportListTypeEnum;
import cn.fw.dalaran.domain.param.report.base.ReportBaseQueryParam;
import cn.fw.dalaran.domain.param.report.live.*;
import cn.fw.dalaran.domain.vo.ActivityThemeVo;
import cn.fw.dalaran.domain.vo.LivePoolVo;
import cn.fw.dalaran.domain.vo.report.RateAnalysisListVo;
import cn.fw.dalaran.domain.vo.report.UserFansCountVo;
import cn.fw.dalaran.domain.vo.report.live.BaseVo;
import cn.fw.dalaran.domain.vo.report.live.ListVo;
import cn.fw.dalaran.domain.vo.report.live.SummaryVo;
import cn.fw.dalaran.domain.vo.report.live.TrendVo;
import cn.fw.dalaran.domain.vo.report.score.TargetResultDto;
import cn.fw.dalaran.rpc.dto.StaffBaseInfoRpc;
import cn.fw.dalaran.rpc.ehr.EhrRpc;
import cn.fw.dalaran.service.biz.LocalCacheService;
import cn.fw.dalaran.service.biz.report.CalSupport;
import cn.fw.dalaran.service.biz.report.ReportCommonBizService;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.dalaran.service.data.LivePoolService;
import cn.fw.ehr.sdk.api.result.StaffBaseInfo;
import cn.fw.oop.sdk.result.ShopInfoVo;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collector;
import java.util.stream.Collectors;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2023/5/8 17:22
 * @Description 直播报表实现
 */
@Slf4j
@Service
@RequiredArgsConstructor
@SuppressWarnings("Duplicates")
public class LiveReportBizService implements CalSupport {
    private final EhrRpc ehrRpc;

    private static final String DEFAULT_ROW_NAME = "-";

    private final AccountService accountService;// 账号
    private final LivePoolService livePoolService;// 直播池
    private final LocalCacheService localCacheService;// 本地缓存
    private final ActivityThemeService activityThemeService;// 活动主题
    private final ReportCommonBizService reportCommonBizService;
    private final LiveListReportBizService liveListReportBizService;

    /**
     * 获取直播汇总报表
     *
     * @param param 查询参数
     * @return 直播汇总报表
     */
    public SummaryVo getSummary(ReportBaseQueryParam param) {
        List<LivePool> dataList =
                livePoolService.list(this.getBaseWrapper(PublicUtil.copy(param, PoolQueryParam.class), true));
        SummaryVo summary = PublicUtil.copy(this.list2BaseVo(dataList), SummaryVo.class);
        List<Long> themeIds = this.getThemeIds(param);
        summary.setTargetResult(reportCommonBizService.getTargetTotal(param.getGroupId(), themeIds, param.getShopIds()));
        return summary;
    }

    private List<Long> getThemeIds(ReportBaseQueryParam param) {
        List<Long> themeIds;
        if (Objects.nonNull(param.getThemeId())) {
            themeIds = Collections.singletonList(param.getThemeId());
        } else {
            Map<ActivityTheme, List<Long>> themeRange = reportCommonBizService.getThemeShopRange(param.getGroupId(),
                    param.getBeginTime(), param.getShopIds());
            themeIds = themeRange.keySet().stream().map(ActivityTheme::getId).collect(Collectors.toList());
        }
        return themeIds;
    }

    /**
     * 获取直播趋势报表
     *
     * @param param 查询参数
     * @return 直播趋势报表
     */
    public TrendVo<BigDecimal> getTrend(TrendQueryParam param) {
        TrendVo<BigDecimal> trend = new TrendVo<>();
        PoolQueryParam queryParam = PublicUtil.copy(param, PoolQueryParam.class);
        queryParam.setCurrent(null);
        queryParam.setPageSize(null);
        if (param.getThemeDimension()) {
            ActivityThemeVo theme = this.getThemeVo(param.getThemeId());
            queryParam.setBeginTime(theme.getStartTime());
            queryParam.setEndTime(theme.getEndTime());
        }
        List<TrendVo.Single<BigDecimal>> vos = new ArrayList<>();
        List<LivePool> list = livePoolService.list(this.getBaseWrapper(queryParam, true));
        Date temp = DateUtil.dayBegin(queryParam.getBeginTime());
        Date dayBegin = DateUtil.dayBegin(new Date());
        while (temp.before(queryParam.getEndTime()) && temp.before(dayBegin)) {
            TrendVo.Single<BigDecimal> single = new TrendVo.Single<>();
            single.setDate(DateUtil.format(temp, DateUtil.FORMAT_2));
            Date finalTemp = temp;
            BaseVo baseVo = this.list2BaseVo(list.stream()
                    .filter(item -> DateUtil.timePointInRange(item.getLiveStartTime(), queryParam.getBeginTime(),
                            DateUtil.dayEnd(finalTemp)))
                    .collect(Collectors.toList())
            );
            single.setValue(baseVo.getValidRate());
            temp = DateUtil.plusDays(temp, 1);
            vos.add(single);
        }
        trend.setVos(vos);
        return trend;
    }

    /**
     * 获取列表数据
     *
     * @param param           查询参数
     * @param secondDimension 二次拆分维度
     * @return 列表数据
     */
    public List<ListVo> getListHaveSecondAnalysis(ListQueryParam param, Integer secondDimension) {
        ReportListTypeEnum dimension = null;
        if (Objects.equals(secondDimension, ReportListTypeEnum.SHOP.getValue()))
            dimension = ReportListTypeEnum.SHOP;
        else if (Objects.equals(secondDimension, ReportListTypeEnum.USER.getValue()))
            dimension = ReportListTypeEnum.USER;
        else if (Objects.equals(secondDimension, ReportListTypeEnum.PLATFORM.getValue()))
            dimension = ReportListTypeEnum.PLATFORM;
        else if (Objects.equals(secondDimension, ReportListTypeEnum.INVALID_REASON.getValue()))
            dimension = ReportListTypeEnum.INVALID_REASON;
//        ListAndScorePojo pojo = new ListAndScorePojo();
//        ReportListTypeEnum finalDimension = dimension;
//        ListQueryParam copyParam = PublicUtil.copy(param, ListQueryParam.class);
//        CompletableFuture.allOf(
//                CompletableFuture.runAsync(() -> this.setListData(param, finalDimension, pojo)),
//                CompletableFuture.runAsync(() -> this.setScoreData(copyParam, pojo))
//        ).join();
//        return pojo.getListResult(param.getThemeDimension());
        Map<Long, ScoreResultDto> scoreMap = liveListReportBizService.getScoreMap(param, null, null);
        List<ListVo> list = this.getList(param, dimension);
        list.forEach(vo -> {
            ScoreResultDto result = scoreMap.getOrDefault(vo.getRowId(), new ScoreResultDto());
            vo.setScore(result.getLiveScore());
            vo.setTargetResult(TargetResultDto.initTargetResult(result));
        });
        return list;
    }

//    /**
//     * 设置列表数据
//     *
//     * @param param     列表查询参数
//     * @param dimension 二次拆分维度
//     * @param pojo      临时数据对象
//     */
//    public void setListData(ListQueryParam param, ReportListTypeEnum dimension, ListAndScorePojo pojo) {
//        pojo.setListVos(this.getList(param, dimension));
//    }

//    /**
//     * 设置列表数据
//     *
//     * @param param 列表查询参数
//     * @param pojo  列表数据与得分数据
//     */
//    public void setScoreData(ListQueryParam param, ListAndScorePojo pojo) {
//        ScoreListParam scoreListParam = PublicUtil.copy(param, ScoreListParam.class);
//        List<ThemeScoreListVo> themeScoreListVos = param.getThemeDimension() ?
//                scoreThemeBizService.getScoreAnalysis(scoreListParam) :
//                scoreMonthBizService.getScoreAnalysis(scoreListParam);
//        pojo.setScoreVos(themeScoreListVos);
//        pojo.setScoreVoMap(themeScoreListVos.stream()
//                .collect(Collectors.toMap(item -> Long.valueOf(item.getId()), Function.identity()))
//        );
//    }

    /**
     * 获取直播列表报表
     *
     * @param param             查询参数
     * @param analysisDimension 二次分析维度
     * @return 直播列表报表
     */
    public List<ListVo> getList(ListQueryParam param, ReportListTypeEnum analysisDimension) {
        Collector<LivePool, ?, Map<Long, List<LivePool>>> mapCollector = null;
        Integer rowType = param.getRowType();
        Long groupId = param.getGroupId();
        List<Long> shopIds = param.getShopIds();
        List<Long> themeIds = getThemeIds(param);
        boolean shop = Objects.equals(rowType, ReportListTypeEnum.SHOP.getValue());
        boolean user = Objects.equals(rowType, ReportListTypeEnum.USER.getValue());
        boolean platform = Objects.equals(rowType, ReportListTypeEnum.PLATFORM.getValue());
        boolean invalidReason = Objects.equals(rowType, ReportListTypeEnum.INVALID_REASON.getValue());
        Boolean themeDimension = param.getThemeDimension();
        Map<Long, LocalCacheService.KV> allDataMap = new HashMap<>();
        if (shop) {
            mapCollector = Collectors.groupingBy(LivePool::getBelongShopId);
            allDataMap = localCacheService.getGroupShops(groupId)
                    .stream()
                    .filter(item -> shopIds.contains(item.getId()))
                    .map(item -> new LocalCacheService.KV(item.getId(), item.getShopName()))
                    .distinct()
                    .collect(Collectors.toMap(LocalCacheService.KV::getId, Function.identity()));// 将门店分组
            if (themeDimension) {// 如果是主题维度, 移除掉主题不相关的门店
                List<Long> themeShopIds = this.getThemeShopIds(param.getThemeId());
                Iterator<Map.Entry<Long, LocalCacheService.KV>> iterator = allDataMap.entrySet().iterator();
                while (iterator.hasNext()) {
                    Map.Entry<Long, LocalCacheService.KV> next = iterator.next();
                    Long k = next.getKey();
                    if (!themeShopIds.contains(k))
                        iterator.remove();
                }
            }
        } else if (user) {
            mapCollector = Collectors.groupingBy(LivePool::getBelongUserId);
            if (themeDimension)// 主题维度, 求交集
                shopIds.retainAll(this.getThemeShopIds(param.getThemeId()));
            List<Long> shouldShowUserIds = new ArrayList<>(localCacheService.getShopsConsultantMap(groupId, shopIds).keySet());
            //param.setUserIds(shouldShowUserIds);// 仅展示销售顾问发布数据
            allDataMap = localCacheService.getGroupConsultant(groupId)
                    .stream()
                    .filter(item -> shouldShowUserIds.contains(item.getId()))
                    .map(item -> new LocalCacheService.KV(item.getId(), item.getUserName()))
                    .distinct()
                    .collect(Collectors.toMap(LocalCacheService.KV::getId, Function.identity()));// 将销售顾问分组
        } else if (platform) {
            mapCollector = Collectors.groupingBy(item -> Long.valueOf(item.getPlatform().getValue()));
            allDataMap = PlatformEnum.getInUsePlatform().stream()
                    .map(item -> new LocalCacheService.KV(Long.valueOf(item.getValue()), item.getName()))
                    .distinct()
                    .collect(Collectors.toMap(LocalCacheService.KV::getId, Function.identity()));// 将平台分组
        } else if (invalidReason) {
            param.setValid(false);
            allDataMap = InvalidReasonEnum.getCheckIndexByWorkType(WorksTypeEnum.LIVE)
                    .stream()
                    .map(item -> new LocalCacheService.KV(Long.valueOf(item.getValue()), item.getDesc()))
                    .distinct()
                    .collect(Collectors.toMap(LocalCacheService.KV::getId, Function.identity()));// 将不合格原因分组
        } else {
            return this.getManagerList(param, themeIds);
        }
        ArrayList<ListVo> list = new ArrayList<>();
        QueryWrapper<LivePool> wrapper = this.getBaseWrapper(PublicUtil.copy(param, PoolQueryParam.class), true);
        List<LivePool> queryResult = livePoolService.list(wrapper);// 查询数据库结果集
        Map<Long, List<LivePool>> groupMap;// K:类型id, V:类型对应结果集
        if (invalidReason)
            groupMap = this.groupByInvalidReason(queryResult);
        else
            groupMap = queryResult.stream().collect(mapCollector);

        for (Map.Entry<Long, List<LivePool>> entry : groupMap.entrySet()) {
            Long k = entry.getKey();
            List<LivePool> v = entry.getValue();
            ListVo vo = PublicUtil.copy(this.list2BaseVo(v), ListVo.class);
            vo.setRowId(k);
            if (shop) {
                vo.setRowName(localCacheService.getShopInfo(groupId, k).getShopName());
            } else if (user) {
                StaffBaseInfo staffBaseInfo = localCacheService.getStaffBaseInfo(k);
                vo.setRowName(Objects.nonNull(staffBaseInfo) ? staffBaseInfo.getName() : null);
            } else if (platform) {
                vo.setRowName(Objects.requireNonNull(PlatformEnum.ofValue(k.intValue())).getName());
            } else {// 不合格原因维度
                vo.setRowName(Objects.requireNonNull(InvalidReasonEnum.getEnumByValue(k.intValue())).getDesc());
            }
            vo.setDataDistributionMsg(this.getDataDistribution(v, analysisDimension));
            list.add(vo);
        }
        list.addAll(this.generateNoDataRow(list, allDataMap));
        Set<Long> userIds = list.stream().map(ListVo::getRowId).collect(Collectors.toSet());
        Map<Long, UserFansCountVo> userFansCountMap = Objects.isNull(param.getThemeId()) ?
                reportCommonBizService.getUserFansCount(userIds) : reportCommonBizService.getThemeFansCount(param.getThemeId());
        for (ListVo vo : list) {
            vo.setUserFans(userFansCountMap.get(vo.getRowId()));
        }
        return this.sorted(list);
    }

    /**
     * 列表数据分析(门店/销顾/平台...分析)
     *
     * @param param 分析维度参数
     * @return 分析结果
     */
    public List<ListVo> getAnalysis(AnalysisParams param) {
        Long rowId = param.getRowId();// 列表项数据id
        Integer rowType = param.getRowType();// 站在什么维度的列表下
        Integer analysisType = param.getAnalysisType();// 进行什么类型分析
        boolean shop = Objects.equals(analysisType, AnalysisTypeEnum.SHOP.getValue());
        boolean user = Objects.equals(analysisType, AnalysisTypeEnum.USER.getValue());
        boolean platform = Objects.equals(analysisType, AnalysisTypeEnum.PLATFORM.getValue());
//        ListAndScorePojo pojo = new ListAndScorePojo();
        if (platform)
            return this.getPlatformAnalysis(PublicUtil.copy(param, ListQueryParam.class), rowType, rowId);
        if (shop || user) {
            ListQueryParam copyParams = PublicUtil.copy(param, ListQueryParam.class);
            if (shop)
                copyParams.setRowType(ReportListTypeEnum.SHOP.getValue());
            if (user)
                copyParams.setRowType(ReportListTypeEnum.USER.getValue());
//            CompletableFuture.allOf(
//                    CompletableFuture.runAsync(() -> this.setAnalysisListData(param, pojo)),
//                    CompletableFuture.runAsync(() -> this.setScoreData(copyParams, pojo))
//            ).join();
//            return pojo.getListResult(param.getThemeDimension());
            List<ListVo> list = this.setAnalysisListData(param);
            Map<Long, ScoreResultDto> scoreMap = liveListReportBizService.getScoreMap(param, param.getRowId(), param.getAnalysisType());
            list.forEach(vo -> {
                ScoreResultDto result = scoreMap.getOrDefault(vo.getRowId(), new ScoreResultDto());
                vo.setScore(result.getLiveScore());
                vo.setTargetResult(TargetResultDto.initTargetResult(result));
            });
            return list;
        }
        return new ArrayList<>();
    }

    /**
     * 获取下钻分析维度列表数据(仅门店分析/人员分析)
     *
     * @param param 分析参数
     *              //     * @param pojo  列表数据与得分数据
     */
    private List<ListVo> setAnalysisListData(AnalysisParams param) {
        Long rowId = param.getRowId();// 列表项数据id
        Integer rowType = param.getRowType();// 站在什么维度的列表下
        Integer analysisType = param.getAnalysisType();// 进行什么类型分析
        boolean shop = Objects.equals(analysisType, AnalysisTypeEnum.SHOP.getValue());
        boolean user = Objects.equals(analysisType, AnalysisTypeEnum.USER.getValue());
        ListQueryParam queryParams = PublicUtil.copy(param, ListQueryParam.class);
        if (shop)
            return this.getShopAnalysis(queryParams, rowType, rowId);
//            pojo.setListVos(this.getShopAnalysis(queryParams, rowType, rowId));
        if (user)
            return this.getUserAnalysis(queryParams, rowType, rowId);
//            pojo.setListVos(this.getUserAnalysis(queryParams, rowType, rowId));
        return new ArrayList<>();
    }

    /**
     * 列表数据占比分析
     *
     * @param param 分析维度参数
     * @return 占比分析结果
     */
    public List<RateAnalysisListVo> getRateAnalysis(RatioAnalysisParams param) {
        Long groupId = param.getGroupId();
        Integer rowType = param.getRowType();// 站在什么维度的列表下
        Integer indexType = param.getIndexType();// 指标类型
        Integer platformType = param.getPlatformType();// 平台类型
        if (Objects.equals(rowType, ReportListTypeEnum.MANAGER_1.getValue()) ||
                Objects.equals(rowType, ReportListTypeEnum.MANAGER_2.getValue()) ||
                Objects.equals(rowType, ReportListTypeEnum.MANAGER_3.getValue()))
            throw new BusinessException("管理层维度下无占比分析!");
        if (Objects.equals(platformType, -1))
            param.setPlatformType(null);// 查询所有平台数据
        boolean shop = Objects.equals(rowType, ReportListTypeEnum.SHOP.getValue());
        boolean user = Objects.equals(rowType, ReportListTypeEnum.USER.getValue());
        boolean platform = Objects.equals(rowType, ReportListTypeEnum.PLATFORM.getValue());
        boolean invalidReason = Objects.equals(rowType, ReportListTypeEnum.INVALID_REASON.getValue());
        List<ListVo> list = new ArrayList<>();
        param.setRowType(ReportListTypeEnum.PLATFORM.getValue());
        /* 根据所站列表维度决定二次拆分维度 */
        if (shop)
            list = this.getList(param, ReportListTypeEnum.SHOP);// 按门店 二次拆分
        else if (user)
            list = this.getList(param, ReportListTypeEnum.USER);// 按销顾 二次拆分
        else if (platform)
            list = this.getList(param, ReportListTypeEnum.PLATFORM);// 按平台 二次拆分
        else if (invalidReason)
            list = this.getList(param, ReportListTypeEnum.INVALID_REASON);// 按无效原因 二次拆分
        /* 根据平台参数获取对应平台数据 */
        if (Objects.equals(platformType, PlatformEnum.KS.getValue()))
            list = list.stream()
                    .filter(item -> Objects.equals(item.getRowId(), Long.valueOf(PlatformEnum.KS.getValue())))
                    .collect(Collectors.toList());
        else if (Objects.equals(platformType, PlatformEnum.DY.getValue()))
            list = list.stream()
                    .filter(item -> Objects.equals(item.getRowId(), Long.valueOf(PlatformEnum.DY.getValue())))
                    .collect(Collectors.toList());
        else if (Objects.equals(platformType, PlatformEnum.DCD.getValue()))
            list = list.stream()
                    .filter(item -> Objects.equals(item.getRowId(), Long.valueOf(PlatformEnum.DCD.getValue())))
                    .collect(Collectors.toList());
        else if (Objects.equals(platformType, PlatformEnum.BILIBILI.getValue()))
            list = list.stream()
                    .filter(item -> Objects.equals(item.getRowId(), Long.valueOf(PlatformEnum.BILIBILI.getValue())))
                    .collect(Collectors.toList());
        else
            list = this.integrateAllSameDimensionData(list);
        if (list.isEmpty() || Objects.isNull(list.get(0)) || Objects.isNull(list.get(0).getDataDistributionMsg()))
            return new ArrayList<>();
        BaseVo summary = this.listVo2BaseVo(new ArrayList<>(list.get(0).getDataDistributionMsg().values()));// 所有指标汇总数据
        List<RateAnalysisListVo> vos = new ArrayList<>();
        list.get(0).getDataDistributionMsg().forEach((k, v) -> {
            RateAnalysisListVo vo = new RateAnalysisListVo();
            vo.setRowId(k);
            if (shop) {
                ShopInfoVo shopInfo = localCacheService.getGroupShopMap(groupId).get(k);
                vo.setRowName(Objects.nonNull(shopInfo) ? shopInfo.getShopName() : DEFAULT_ROW_NAME);
            } else if (user) {
                StaffBaseInfo staffInfo = localCacheService.getStaffBaseInfo(k);
                vo.setRowName(Objects.nonNull(staffInfo) ? staffInfo.getName() : DEFAULT_ROW_NAME);
            } else if (platform) {
                String platformName = PlatformEnum.getNameByVale(k.intValue());
                vo.setRowName(Objects.nonNull(platformName) ? platformName : DEFAULT_ROW_NAME);
            } else if (invalidReason) {
                InvalidReasonEnum reasonEnum = InvalidReasonEnum.getEnumByValue(k.intValue());
                vo.setRowName(Objects.nonNull(reasonEnum) ? reasonEnum.getDesc() : DEFAULT_ROW_NAME);
            }
            this.handleRate(indexType, vo, v, summary);// 处理占比信息
            vos.add(vo);
        });
        return vos.stream()
                .sorted(Comparator.comparing(RateAnalysisListVo::getRate).reversed())
                .collect(Collectors.toList());
    }

    /**
     * 获取直播池
     *
     * @param param 池查询条件参数
     * @return 直播池
     */
    public PageHelper<LivePoolVo> getPool(PoolQueryParam param) {
        QueryWrapper<LivePool> wrapper = this.getBaseWrapper(param);
        PageHelper<LivePoolVo> pageHelper = new PageHelper<>(Long.valueOf(param.getCurrent()),
                Long.valueOf(param.getPageSize()));
        IPage<LivePool> page = livePoolService.page(PageUtil.helper2Page(pageHelper), wrapper);
        List<LivePool> records = page.getRecords();
        if (records.isEmpty())
            return new PageHelper<>();
        Map<Long, Account> accountMap = accountService.getByIdsIgnoreLogicDel(records.stream()
                        .map(LivePool::getAccountId)
                        .collect(Collectors.toList())
                ).stream()
                .collect(Collectors.toMap(Account::getId, Function.identity()));// K:accountId, V:对应账号实体
        Map<Long, ActivityTheme> activityThemeMap = activityThemeService.lambdaQuery()
                .in(ActivityTheme::getId, records.stream()
                        .map(LivePool::getThemeId)
                        .collect(Collectors.toList())
                )
                .list()
                .stream()
                .collect(Collectors.toMap(ActivityTheme::getId, Function.identity()));// K:活动主题id, V:对应活动主题
        List<Long> userIds = page.getRecords().stream().map(LivePool::getBelongUserId).collect(Collectors.toList());
        List<StaffBaseInfoRpc> staffInfoBatch = ehrRpc.getStaffInfoBatch(userIds);
        Map<Long, String> userMap = staffInfoBatch.stream()
                .collect(Collectors.toMap(StaffBaseInfoRpc::getId, StaffBaseInfoRpc::getName));
        return PageUtil.toPage(page, item -> {
                    LivePoolVo vo = LivePool.toVo(item);
                    vo.setValid(item.getValidLive());
                    vo.setTags(reportCommonBizService.handleWorksTopic(item.getTags()));
                    Account account = accountMap.get(item.getAccountId());
                    if (Objects.nonNull(account)) {
                        vo.setAccount(account.getAccount());
//                        vo.setUserName(account.getUserName());
                        vo.setShopName(account.getShopName());
                    }
                    vo.setUserName(userMap.get(item.getBelongUserId()));
                    ActivityTheme activityTheme = activityThemeMap.get(item.getThemeId());
                    if (Objects.nonNull(activityTheme))
                        vo.setSetTags(reportCommonBizService.handleSetTopic(activityTheme));
                    try {
                        vo.setScoreDetails(JSON.parseObject(item.getScoreCalJson(), EvlFormulaDto.class));
                    } catch (Exception e) {
                        log.error("反序列化得分详情出错!");
                    }
                    return vo;
                }
        );
    }

    /**
     * 获取门店分析
     *
     * @param param   查询参数
     * @param rowType 报表列表维度
     * @param rowId   列表项id
     * @return 门店分析
     */
    private List<ListVo> getShopAnalysis(ListQueryParam param, Integer rowType, Long rowId) {
        Long groupId = param.getGroupId();
        if (Objects.equals(rowType, ReportListTypeEnum.SHOP.getValue()))
            throw new BusinessException("门店维度下无门店分析!");
        if (Objects.equals(rowType, ReportListTypeEnum.USER.getValue()))
            throw new BusinessException("人员维度下无门店分析!");
        boolean platform = Objects.equals(rowType, ReportListTypeEnum.PLATFORM.getValue());
        boolean invalidReason = Objects.equals(rowType, ReportListTypeEnum.INVALID_REASON.getValue());
        param.setRowType(ReportListTypeEnum.SHOP.getValue());
        Map<Long, LocalCacheService.KV> shopMap = new HashMap<>();// 指定map获取各个维度所有的门店信息
        List<Long> themeIds = getThemeIds(param);
        List<ListVo> list;
        List<ListVo> hasDataList;
        if (platform) {
            list = this.getList(param, ReportListTypeEnum.PLATFORM);// 按平台 二次拆分
            hasDataList = this.generateSecondAnalysisByRowId(list, rowId);
        } else if (invalidReason) {
            param.setValid(false);
            list = this.getList(param, ReportListTypeEnum.INVALID_REASON);// 按无效原因 二次拆分
            hasDataList = this.generateSecondAnalysisByRowId(list, rowId);
        } else {// 管理层维度
            list = this.getList(param, null);
            List<LocalCacheService.KV> userRoleAuthShops = this.getManageShops(rowType, groupId, rowId);// 获取管理层用户管理门店信息
            if (userRoleAuthShops.isEmpty())
                return new ArrayList<>();
            shopMap = userRoleAuthShops
                    .stream()
                    .collect(Collectors.toMap(LocalCacheService.KV::getId, Function.identity()));// 将查到的门店进行分组, 方便后续获取id集合和门店名称
            List<Long> shopIds = new ArrayList<>(shopMap.keySet());// 获取管理门店集合
            hasDataList = list
                    .stream()
                    .filter(item -> shopIds.contains(item.getRowId()))// 过滤出管理的门店的数据
                    .collect(Collectors.toList())
            ;// 获取有数据的门店
        }
        hasDataList.addAll(this.generateNoDataRow(hasDataList, shopMap));
        return this.sorted(hasDataList);
    }

    /**
     * 获取销顾分析
     *
     * @param param   查询参数
     * @param rowType 报表列表维度
     * @param rowId   列表项id
     * @return 销顾分析
     */
    private List<ListVo> getUserAnalysis(ListQueryParam param, Integer rowType, Long rowId) {
        Long groupId = param.getGroupId();
        if (Objects.equals(rowType, ReportListTypeEnum.USER.getValue()))
            throw new BusinessException("人员维度下无销顾分析!");
        boolean shop = Objects.equals(rowType, ReportListTypeEnum.SHOP.getValue());
        boolean manager1 = Objects.equals(rowType, ReportListTypeEnum.MANAGER_1.getValue());
        boolean manager2 = Objects.equals(rowType, ReportListTypeEnum.MANAGER_2.getValue());
        boolean manager3 = Objects.equals(rowType, ReportListTypeEnum.MANAGER_3.getValue());
        boolean manager4 = Objects.equals(rowType, ReportListTypeEnum.MANAGER_4.getValue());
        boolean platform = Objects.equals(rowType, ReportListTypeEnum.PLATFORM.getValue());
        boolean invalidReason = Objects.equals(rowType, ReportListTypeEnum.INVALID_REASON.getValue());
        param.setRowType(ReportListTypeEnum.USER.getValue());
        Map<Long, LocalCacheService.KV> userMap = new HashMap<>();// 指定map获取各个维度所有的销顾信息
        List<ListVo> list;
        List<ListVo> hasDataList;
        if (shop) {
            list = this.getList(param, ReportListTypeEnum.SHOP);// 按门店 二次拆分
            hasDataList = this.generateSecondAnalysisByRowId(list, rowId);
        } else if (platform) {
            list = this.getList(param, ReportListTypeEnum.PLATFORM);// 按平台 二次拆分
            hasDataList = this.generateSecondAnalysisByRowId(list, rowId);
        } else if (invalidReason) {
            param.setValid(false);
            list = this.getList(param, ReportListTypeEnum.INVALID_REASON);// 按无效原因 二次拆分
            hasDataList = this.generateSecondAnalysisByRowId(list, rowId);
        } else {// 管理层维度
            list = this.getList(param, ReportListTypeEnum.SHOP);
            List<LocalCacheService.KV> userRoleAuthShops = null;// 获取管理层用户管理门店信息
            if (manager1) {
                userRoleAuthShops = localCacheService.getManageShops(groupId, RoleMsgEnum.XSZG.getCode(), rowId);
                userMap = localCacheService.getManageConsultants(groupId, RoleMsgEnum.XSZG.getCode(), rowId)
                        .stream()
                        .collect(Collectors.toMap(LocalCacheService.KV::getId, Function.identity()));
            } else if (manager2) {
                userRoleAuthShops = localCacheService.getManageShops(groupId, RoleMsgEnum.XSJL.getCode(), rowId);
                userMap = localCacheService.getManageConsultants(groupId, RoleMsgEnum.XSJL.getCode(), rowId)
                        .stream()
                        .collect(Collectors.toMap(LocalCacheService.KV::getId, Function.identity()));
            } else if (manager3) {
                userRoleAuthShops = localCacheService.getManageShops(groupId, RoleMsgEnum.XSZJL.getCode(), rowId);
                userMap = localCacheService.getManageConsultants(groupId, RoleMsgEnum.XSZJL.getCode(), rowId)
                        .stream()
                        .collect(Collectors.toMap(LocalCacheService.KV::getId, Function.identity()));
            } else if (manager4) {
                userRoleAuthShops = localCacheService.getManageShops(groupId, RoleMsgEnum.XSSJGL.getCode(), rowId);
                userMap = localCacheService.getManageConsultants(groupId, RoleMsgEnum.XSSJGL.getCode(), rowId)
                        .stream()
                        .collect(Collectors.toMap(LocalCacheService.KV::getId, Function.identity()));
            }
            if (Objects.isNull(userRoleAuthShops) || userRoleAuthShops.isEmpty())
                return new ArrayList<>();
            List<Long> manageShopIds = userRoleAuthShops.stream()
                    .map(LocalCacheService.KV::getId)
                    .collect(Collectors.toList());
            hasDataList = list
                    .stream()
                    .filter(item -> {
                        Map<Long, BaseVo> dataDistributionMsg = item.getDataDistributionMsg();
                        return Objects.nonNull(dataDistributionMsg) &&
                                CollectionUtils.containsAny(manageShopIds, dataDistributionMsg.keySet());
                    })// 过滤出该管理层管理门店有数据的销售顾问
                    .map(item -> {
                        ListVo vo = PublicUtil.copy(this.listVo2BaseVo(item.getDataDistributionMsg().entrySet().stream()
                                .filter(item1 -> manageShopIds.contains(item1.getKey()))// 过滤出管理门店的数据
                                .map(Map.Entry::getValue)
                                .collect(Collectors.toList())), ListVo.class);
                        vo.setRowId(item.getRowId());
                        vo.setRowName(item.getRowName());
                        return vo;
                    })
                    .collect(Collectors.toList())
            ;// 获取有数据的销顾
        }
        hasDataList.addAll(this.generateNoDataRow(hasDataList, userMap));
        Set<Long> userIds = hasDataList.stream().map(ListVo::getRowId).collect(Collectors.toSet());
        Map<Long, UserFansCountVo> userFansCountMap = Objects.isNull(param.getThemeId()) ?
                reportCommonBizService.getUserFansCount(userIds) : reportCommonBizService.getThemeFansCount(param.getThemeId());
        hasDataList.forEach(vo -> vo.setUserFans(userFansCountMap.get(vo.getRowId())));
        return this.sorted(hasDataList);
    }

    /**
     * 获取平台分析
     *
     * @param param   查询参数
     * @param rowType 报表列表维度
     * @param rowId   列表项id
     * @return 平台分析
     */
    private List<ListVo> getPlatformAnalysis(ListQueryParam param, Integer rowType, Long rowId) {
        Long groupId = param.getGroupId();
        if (Objects.equals(rowType, ReportListTypeEnum.PLATFORM.getValue()))
            throw new BusinessException("平台维度下无平台分析!");
        if (Objects.equals(rowType, ReportListTypeEnum.INVALID_REASON.getValue()))
            throw new BusinessException("无效原因维度下无平台分析!");
        boolean shop = Objects.equals(rowType, ReportListTypeEnum.SHOP.getValue());
        boolean user = Objects.equals(rowType, ReportListTypeEnum.USER.getValue());
        param.setRowType(ReportListTypeEnum.PLATFORM.getValue());
        Map<Long, LocalCacheService.KV> platformMap = new HashMap<>();// 指定map获取各个维度所有的平台信息
        PlatformEnum.getInUsePlatform().forEach(item -> {
            Long key = Long.valueOf(item.getValue());
            platformMap.put(key, new LocalCacheService.KV(key, item.getName()));
        });
        List<ListVo> list;
        List<ListVo> hasDataList = new ArrayList<>();
        if (shop) {
            list = this.getList(param, ReportListTypeEnum.SHOP);// 按门店 二次拆分
            hasDataList = this.generateSecondAnalysisByRowId(list, rowId);
        } else if (user) {
            list = this.getList(param, ReportListTypeEnum.USER);// 按用户 二次拆分
            hasDataList = this.generateSecondAnalysisByRowId(list, rowId);
        } else {// 管理层维度
            list = this.getList(param, ReportListTypeEnum.SHOP);// 按门店 二次拆分
            List<Long> authShopIds = this.getManageShopIds(rowType, groupId, rowId);// 获取管理门店id列表
            if (authShopIds.isEmpty())
                return new ArrayList<>();
            for (ListVo item : list) {
                Map<Long, BaseVo> dataDistributionMsg = item.getDataDistributionMsg();
                if (Objects.isNull(dataDistributionMsg) || dataDistributionMsg.isEmpty()) continue;
                ArrayList<BaseVo> baseVos = new ArrayList<>();
                for (Map.Entry<Long, BaseVo> entry : dataDistributionMsg.entrySet()) {
                    if (authShopIds.contains(entry.getKey()))
                        baseVos.add(entry.getValue());
                }
                ListVo copy = PublicUtil.copy(this.listVo2BaseVo(baseVos), ListVo.class);
                copy.setRowId(item.getRowId());
                copy.setRowName(item.getRowName());
                hasDataList.add(copy);
            }
        }
        hasDataList.addAll(this.generateNoDataRow(hasDataList, platformMap));
        return this.sorted(hasDataList);
    }

    /**
     * 获取集团管理人员管理门店详情
     *
     * @param rowType   {@link ReportListTypeEnum}
     * @param groupId   集团id
     * @param managerId 管理人员id
     * @return 该管理人员管理门店详情
     */
    private List<LocalCacheService.KV> getManageShops(Integer rowType, Long groupId, Long managerId) {
        List<LocalCacheService.KV> userRoleAuthShops = new ArrayList<>();// 获取管理层用户管理门店信息
        if (Objects.equals(rowType, ReportListTypeEnum.MANAGER_1.getValue())) {// 一级管理
            userRoleAuthShops = localCacheService.getManageShops(groupId, RoleMsgEnum.XSZG.getCode(), managerId);
        } else if (Objects.equals(rowType, ReportListTypeEnum.MANAGER_2.getValue())) {// 二级管理
            userRoleAuthShops = localCacheService.getManageShops(groupId, RoleMsgEnum.XSJL.getCode(), managerId);
        } else if (Objects.equals(rowType, ReportListTypeEnum.MANAGER_3.getValue())) {// 三级管理
            userRoleAuthShops = localCacheService.getManageShops(groupId, RoleMsgEnum.XSZJL.getCode(), managerId);
        } else if (Objects.equals(rowType, ReportListTypeEnum.MANAGER_4.getValue())) {// 四级管理
            userRoleAuthShops = localCacheService.getManageShops(groupId, RoleMsgEnum.XSSJGL.getCode(), managerId);
        }
        return userRoleAuthShops;
    }

    /**
     * 获取集团管理人员管理门店id列表
     *
     * @param rowType   {@link ReportListTypeEnum}
     * @param groupId   集团id
     * @param managerId 管理人员id
     * @return 该管理人员管理门店id列表
     */
    private List<Long> getManageShopIds(Integer rowType, Long groupId, Long managerId) {
        List<LocalCacheService.KV> manageShops = this.getManageShops(rowType, groupId, managerId);
        if (manageShops.isEmpty()) return new ArrayList<>();
        return manageShops.stream()
                .map(LocalCacheService.KV::getId)
                .collect(Collectors.toList());// 获取管理门店列表
    }

    /**
     * 处理占比分析占比数与占比值
     *
     * @param indexType 指标类型
     * @param vo        待处理数据
     * @param single    单个数据汇总详情
     * @param summary   所有数据汇总
     */
    public void handleRate(Integer indexType, RateAnalysisListVo vo, BaseVo single, BaseVo summary) {
        Integer numerator = 0;
        Integer denominator = 0;
        if (Objects.equals(indexType, 1)) {
            numerator = single.getI1();
            denominator = summary.getI1();
        } else if (Objects.equals(indexType, 2)) {
            numerator = single.getI2();
            denominator = summary.getI2();
        } else if (Objects.equals(indexType, 3)) {
            numerator = single.getI3();
            denominator = summary.getI3();
        } else if (Objects.equals(indexType, 4)) {
            numerator = single.getI4();
            denominator = summary.getI4();
        } else if (Objects.equals(indexType, 5)) {
            numerator = single.getI5();
            denominator = summary.getI5();
        } else if (Objects.equals(indexType, 6)) {
            numerator = single.getI6();
            denominator = summary.getI6();
        } else if (Objects.equals(indexType, 7)) {
            numerator = single.getI7();
            denominator = summary.getI7();
        } else if (Objects.equals(indexType, 8)) {
            numerator = single.getI8();
            denominator = summary.getI8();
        } else if (Objects.equals(indexType, 9)) {
            numerator = single.getI9();
            denominator = summary.getI9();
        }
        vo.setIndexValue(numerator);
        if (Objects.equals(numerator, 0) || Objects.equals(denominator, 0))
            vo.setRate(BigDecimal.ZERO);
        else
            vo.setRate(BigDecimal.valueOf(numerator).divide(BigDecimal.valueOf(denominator), 4, RoundingMode.HALF_UP));
    }

    /**
     * 整合相同维度数据(比如张三数据分布在ABC门店, 则将张三数据累加)
     *
     * @param list 所有数据
     * @return 整合结果
     */
    private List<ListVo> integrateAllSameDimensionData(List<ListVo> list) {
        Map<Long, BaseVo> result = new HashMap<>();
        list.forEach(item -> {
            Map<Long, BaseVo> distributionMsg = item.getDataDistributionMsg();
            if (Objects.isNull(distributionMsg) || distributionMsg.isEmpty())
                return;
            distributionMsg.forEach((k, v) -> {
                BaseVo baseVo = result.get(k);
                if (Objects.isNull(baseVo))
                    result.put(k, v);
                else
                    result.put(k, this.listVo2BaseVo(Arrays.asList(v, baseVo)));
            });
        });
        ListVo vo = new ListVo();
        vo.setDataDistributionMsg(result);
        return Collections.singletonList(vo);
    }

    /**
     * 根据无效原因分组
     *
     * @param queryResult 查询数据库结果集
     * @return K:无效类型编码, V:包含该编码的直播集合
     */
    private Map<Long, List<LivePool>> groupByInvalidReason(List<LivePool> queryResult) {
        Map<Long, List<LivePool>> map = new HashMap<>();
        InvalidReasonEnum.getCheckIndexByWorkType(WorksTypeEnum.LIVE)
                .forEach(item -> queryResult.stream()
                        .collect(Collectors.groupingBy(item1 -> item1.getInvalidCollector(item.getValue())))
                        .forEach((k, v) -> {
                            if (Objects.equals(k, item.getValue()))
                                map.put(Long.valueOf(k), v);
                        })
                )
        ;
        return map;
    }

    /**
     * 获取管理层数据
     *
     * @param param 查询参数
     * @return 管理层数据
     */
    private List<ListVo> getManagerList(ListQueryParam param, List<Long> themeIds) {
        Integer rowType = param.getRowType();
        boolean manager1 = Objects.equals(rowType, ReportListTypeEnum.MANAGER_1.getValue());
        boolean manager2 = Objects.equals(rowType, ReportListTypeEnum.MANAGER_2.getValue());
        boolean manager3 = Objects.equals(rowType, ReportListTypeEnum.MANAGER_3.getValue());
        boolean manager4 = Objects.equals(rowType, ReportListTypeEnum.MANAGER_4.getValue());
        String roleCode;
        if (manager1)
            roleCode = RoleMsgEnum.XSZG.getCode();
        else if (manager2)
            roleCode = RoleMsgEnum.XSJL.getCode();
        else if (manager3)
            roleCode = RoleMsgEnum.XSZJL.getCode();
        else if (manager4)
            roleCode = RoleMsgEnum.XSSJGL.getCode();
        else
            roleCode = "";
        Boolean themeDimension = param.getThemeDimension();

        List<ManagerInfo> groupManager = localCacheService.getGroupManager(param.getGroupId());
        List<ListVo> vos = new ArrayList<>();// 返回数据
        groupManager.stream()
                .filter(item -> Objects.equals(item.getRoleCode(), roleCode))
                .findFirst()
                .ifPresent(item -> {// item: 对应级别管理层人员列表
                    param.setRowType(ReportListTypeEnum.SHOP.getValue());// 按'门店'维度获取数据列表
                    List<ListVo> list = this.getList(param, null);
                    List<ManagerInfo.RangeInfo> users = item.getUsers();// 拥有该角色的管理层人
                    users.forEach(item1 -> {// item1: 管理人
                        List<Long> manageShopIds = item1.getShopIds();// 该管理层人员管理门店列表
                        if (themeDimension) {// 如果是按主题维度查看
                            List<Long> themeShopIds = this.getThemeShopIds(param.getThemeId());
                            if (!CollectionUtils.containsAny(manageShopIds, themeShopIds)) return;// 该管理人员不管该主题的数据
                        }
                        ListVo vo = PublicUtil.copy(this.listVo2BaseVo(list.stream()
                                .filter(item2 -> manageShopIds.contains(item2.getRowId()))// item2: 列表类型报表每行数据
                                .collect(Collectors.toList())), ListVo.class
                        );
                        Long managerId = item1.getUserId();// 管理人员id
                        String userName = item1.getUserName();// 管理人员名
                        vo.setRowId(managerId);
                        vo.setRowName(StringUtils.hasText(userName) ? userName :
                                localCacheService.getStaffBaseInfo(managerId).getName());
                        vos.add(vo);
                    });
                });
        return this.sorted(vos);
    }

    /**
     * 获取主题的参与门店
     *
     * @param themeId 主题id
     * @return 主题的参与门店
     */
    public List<Long> getThemeShopIds(Long themeId) {
        return this.getThemeVo(themeId).getShopIds();
    }

    /**
     * 根据主题id获取主题
     *
     * @param themeId 主题id
     * @return id对应主题
     */
    public ActivityThemeVo getThemeVo(Long themeId) {
        return ActivityTheme.toVO(Objects.requireNonNull(activityThemeService.getById(themeId)));
    }

    /**
     * 获取查询条件包装器
     *
     * @param param  查询参数
     * @param ignore 是否忽略(汇总/列表使用)
     * @return 查询条件包装器
     */
    private QueryWrapper<LivePool> getBaseWrapper(PoolQueryParam param, boolean... ignore) {
        Integer rowType = param.getRowType();
        Long rowId = param.getRowId();
        Long outRowId = param.getOutRowId();
        Integer outRowType = param.getOutRowType();
        Date beginTime = param.getBeginTime();
        Date endTime = param.getEndTime();
        Long themeId = param.getThemeId();
        List<Long> shopIds;
        Boolean themeDimension = param.getThemeDimension();// 是否为主题维度
        Boolean valid = param.getValid();
        boolean managerDimension = Objects.equals(rowType, ReportListTypeEnum.MANAGER_1.getValue()) ||
                Objects.equals(rowType, ReportListTypeEnum.MANAGER_2.getValue()) ||
                Objects.equals(rowType, ReportListTypeEnum.MANAGER_3.getValue()) ||
                Objects.equals(rowType, ReportListTypeEnum.MANAGER_4.getValue());
        boolean outManagerDimension = Objects.equals(outRowType, ReportListTypeEnum.MANAGER_1.getValue()) ||
                Objects.equals(outRowType, ReportListTypeEnum.MANAGER_2.getValue()) ||
                Objects.equals(outRowType, ReportListTypeEnum.MANAGER_3.getValue()) ||
                Objects.equals(outRowType, ReportListTypeEnum.MANAGER_4.getValue());
        int length = ignore.length;// 可变参长度
        if (managerDimension)// 管理层维度
            shopIds = this.getManageShopIds(rowType, param.getGroupId(), rowId);
        else
            shopIds = param.getShopIds();
        shopIds.retainAll(param.getShopIds());// 求交集, 防止数据范围越界
        if (shopIds.isEmpty())
            shopIds.add(-1L);// 防止找不到门店, 而忽略门店条件查了所有门店数据
        List<Long> userIds = param.getUserIds();
        QueryWrapper<LivePool> wrapper = new QueryWrapper<>();
        List<Long> userIdsByNameLike = new ArrayList<>();
        if (StringUtils.hasText(param.getUserName()))
            userIdsByNameLike = this.getUserIdsByNameLike(param.getGroupId(), param.getUserName());
        wrapper.lambda()
                .eq(themeDimension, LivePool::getThemeId, themeId)
                .in(!shopIds.isEmpty(), LivePool::getBelongShopId, shopIds)
                .in(!userIds.isEmpty(), LivePool::getBelongUserId, userIds)
                .ge(Objects.nonNull(beginTime) /*&& !themeDimension*/, LivePool::getLiveStartTime, beginTime)
                .le(Objects.nonNull(endTime) /*&& !themeDimension*/, LivePool::getLiveStartTime, endTime)
                .le(Objects.nonNull(valid) && !valid, LivePool::getValidLive, 0)
                .gt(Objects.nonNull(valid) && valid, LivePool::getValidLive, 0)
                .and(wrapper1 -> wrapper1
                        .isNull(LivePool::getStatus)
                        .or()
                        .eq(LivePool::getStatus, 1)
                )// 不查询调整账号导致失效的作品
                .and(Objects.nonNull(rowType) && !managerDimension && Objects.equals(length, 0) || (length > 0 && !ignore[0]), wrapper1 -> wrapper1
                        .eq(Objects.equals(rowType, ReportListTypeEnum.SHOP.getValue()), LivePool::getBelongShopId, rowId)
                        .eq(Objects.equals(rowType, ReportListTypeEnum.USER.getValue()), LivePool::getBelongUserId, rowId)
                        .eq(Objects.equals(rowType, ReportListTypeEnum.PLATFORM.getValue()), LivePool::getPlatform, rowId)
                        .last(Objects.equals(rowType, ReportListTypeEnum.INVALID_REASON.getValue()), "find_in_set(" + rowId + ", invalid_reason)")
                )
                /* 以下是分析列表下跳池 */
                .and(!outManagerDimension && Objects.nonNull(outRowType) && Objects.nonNull(outRowId), wrapper1 -> wrapper1
                        .eq(Objects.equals(outRowType, ReportListTypeEnum.SHOP.getValue()), LivePool::getBelongShopId, outRowId)
                        .eq(Objects.equals(outRowType, ReportListTypeEnum.USER.getValue()), LivePool::getBelongUserId, outRowId)
                        .eq(Objects.equals(outRowType, ReportListTypeEnum.PLATFORM.getValue()), LivePool::getPlatform, outRowId)
                        .last(Objects.equals(outRowType, ReportListTypeEnum.INVALID_REASON.getValue()), "find_in_set(" + outRowId + ", invalid_reason)")
                )
                .and(outManagerDimension, wrapper1 -> wrapper1
                        .in(LivePool::getBelongShopId, this.getManageShopIds(outRowType, param.getGroupId(), outRowId))
                )
                /* 以下是池过滤条件 */
                .like(Objects.nonNull(param.getTitle()), LivePool::getTitle, param.getTitle())
                .like(Objects.nonNull(param.getUserNick()), LivePool::getUserNick, param.getUserNick())
                .eq(Objects.nonNull(param.getPlatformType()), LivePool::getPlatform, param.getPlatformType())
                .eq(Objects.nonNull(param.getPlatform()), LivePool::getPlatform, param.getPlatform())
                .in(!userIdsByNameLike.isEmpty(), LivePool::getBelongUserId, userIdsByNameLike)
                /* 处理排序 */
                .last(SortFieldEnum.getSortSql(param.getOrderAtt(), param.getOrder()))
        ;
        return wrapper;
    }

    /**
     * 根据人员名字查找集团名字包含该字符串的人员id集合
     *
     * @param groupId  集团id
     * @param userName 名字字符串
     * @return 集团名字包含该字符串的人员id集合
     */
    private List<Long> getUserIdsByNameLike(Long groupId, String userName) {
        if (!StringUtils.hasText(userName))
            return new ArrayList<>();
        List<Long> userIds = localCacheService.getGroupStaffs(groupId).stream()
                .filter(item -> item.getName().contains(userName))
                .map(StaffBaseInfo::getId)
                .collect(Collectors.toList());
        if (userIds.isEmpty())
            userIds.add(-1L);
        return userIds;
    }

    /**
     * 数据库直播列表数据转为基础数据
     *
     * @param list 数据库直播列表数据
     * @return 基础数据
     */
    private BaseVo list2BaseVo(List<LivePool> list) {
        List<LivePool> validList = list.stream()
                .filter(item -> item.getValidLive() > 0)
                .collect(Collectors.toList());
        BaseVo vo = new BaseVo();
        vo.setI1(list.size());// 直播数
        vo.setI2(sumInt(validList, LivePool::getUv));// 观看人数
        vo.setI3(sumInt(validList, LivePool::getLikeNum));// 点赞量
        vo.setI4(sumInt(validList, LivePool::getCommentNum));// 评论数
        vo.setI5(sumInt(validList, LivePool::getShareNum));// 分享数
        vo.setI6(sumInt(validList, LivePool::getNewFanNum));// 新增粉丝数
        vo.setI7(sumInt(validList, LivePool::getLiveDuration));// 直播时长
        vo.setI8(validList.size());// 有效直播场数
        vo.setI9(sumInt(validList, LivePool::getUvPeak));// 观看人数巅峰
        BigDecimal bigDecimal = validList.stream()
                .filter(r -> r.getValidLive() > 1).map(LivePool::getScore)
                .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
        vo.setScore(bigDecimal.compareTo(BigDecimal.ZERO) <= 0
                ? BigDecimal.ZERO : bigDecimal.divide(BigDecimal.valueOf(validList.size()), 4, RoundingMode.HALF_UP));
        return vo;
    }

    /**
     * listVos数据累加整合成baseVo对象
     *
     * @param list listVos数据
     * @return baseVo
     */
    private <Q extends BaseVo> BaseVo listVo2BaseVo(List<Q> list) {
        BaseVo vo = new BaseVo();
        vo.setI1(sumInt(list, BaseVo::getI1));
        vo.setI2(sumInt(list, BaseVo::getI2));
        vo.setI3(sumInt(list, BaseVo::getI3));
        vo.setI4(sumInt(list, BaseVo::getI4));
        vo.setI5(sumInt(list, BaseVo::getI5));
        vo.setI6(sumInt(list, BaseVo::getI6));
        vo.setI7(sumInt(list, BaseVo::getI7));
        vo.setI8(sumInt(list, BaseVo::getI8));
        vo.setI9(sumInt(list, BaseVo::getI9));
        BigDecimal totalScore = PublicUtil.isEmpty(list) ? BigDecimal.ZERO
                : list.stream().map(BaseVo::getScore).reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
        vo.setScore(totalScore.compareTo(BigDecimal.ZERO) > 0
                ? totalScore.divide(BigDecimal.valueOf(list.size()), 4, RoundingMode.HALF_UP) : BigDecimal.ZERO);
        return vo;
    }

    /**
     * 结果集排序
     *
     * @param collect 待排序集合
     * @return 已排序结果
     */
    private List<ListVo> sorted(List<ListVo> collect) {
        return collect.stream()
                .sorted(Comparator.comparing(ListVo::getI8, Comparator.reverseOrder())
                        .thenComparing(ListVo::getI1, Comparator.reverseOrder())
                )
                .collect(Collectors.toList());
    }

    /**
     * 构造数据库没有数据的报表行
     *
     * @param hasDataList 已经有数据的列表集合
     * @param allDataMap  应该有数据的K-V
     * @return 数据库没有数据的行
     */
    private List<ListVo> generateNoDataRow(List<ListVo> hasDataList, Map<Long, LocalCacheService.KV> allDataMap) {
        List<Long> hasDataRowIds = hasDataList.stream()
                .map(ListVo::getRowId)
                .collect(Collectors.toList());// 获取有数据的行id集合
        return new ArrayList<>(allDataMap.keySet())
                .stream()
                .filter(item -> !hasDataRowIds.contains(item))// 过滤出没有数据的行id集合
                .map(item -> {
                    ListVo vo = new ListVo();
                    vo.setRowId(item);
                    String name = allDataMap.get(item).getName();
                    vo.setRowName(StringUtils.hasText(name) ? name : DEFAULT_ROW_NAME);
                    return vo;
                })
                .collect(Collectors.toList())
                ;
    }

    /**
     * 获取列表项数据分布情况
     *
     * @param item              列表项数据集合
     * @param analysisDimension 二次分析维度
     * @return 列表项数据分布情况
     */
    private Map<Long, BaseVo> getDataDistribution(List<LivePool> item, ReportListTypeEnum analysisDimension) {
        if (Objects.isNull(analysisDimension))
            return Collections.emptyMap();
        Collector<LivePool, ?, Map<Long, List<LivePool>>> collector = null;
        if (Objects.equals(analysisDimension, ReportListTypeEnum.SHOP))
            collector = Collectors.groupingBy(LivePool::getBelongShopId);
        else if (Objects.equals(analysisDimension, ReportListTypeEnum.USER))
            collector = Collectors.groupingBy(LivePool::getBelongUserId);
        else if (Objects.equals(analysisDimension, ReportListTypeEnum.PLATFORM))
            collector = Collectors.groupingBy(item1 -> Long.valueOf(item1.getPlatform().getValue()));
        Map<Long, BaseVo> dataDistribution = new HashMap<>();
        Map<Long, List<LivePool>> secondGroupByResult;
        if (Objects.equals(analysisDimension, ReportListTypeEnum.INVALID_REASON)) {
            secondGroupByResult = this.groupByInvalidReason(item);
        } else {
            secondGroupByResult = item.stream()
                    .collect(collector);// 将直播记录数据根据不同维度分组
        }
        secondGroupByResult.forEach((k, v) -> dataDistribution.put(k, this.list2BaseVo(v)));
        return dataDistribution;
    }

    /**
     * 根据rowId生成二次分析结果
     *
     * @param list  二次分组的总列表
     * @param rowId 行数据id
     * @return rowId生成的二次分析结果
     */
    private List<ListVo> generateSecondAnalysisByRowId(List<ListVo> list, Long rowId) {
        return list.stream()
                .map(item -> {
                    Map<Long, BaseVo> msg = item.getDataDistributionMsg();
                    BaseVo baseVo;
                    if (Objects.nonNull(msg) && msg.containsKey(rowId) && Objects.nonNull(baseVo = msg.get(rowId))) {
                        ListVo vo = PublicUtil.copy(baseVo, ListVo.class);
                        vo.setRowId(item.getRowId());
                        String rowName = item.getRowName();
                        vo.setRowName(StringUtils.hasText(rowName) ? rowName : DEFAULT_ROW_NAME);
                        return vo;
                    }
                    return null;
                })
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
    }
}
