package cn.fw.dalaran.service.biz.report.live;

import lombok.AllArgsConstructor;
import lombok.Getter;
import org.springframework.util.StringUtils;

import java.util.Objects;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2023/6/9 15:41
 * @Description 直播池排序相关枚举
 */
@Getter
@AllArgsConstructor
public enum SortFieldEnum {

    SCORE("score", "score", "得分率"),
    UV("uv", "uv", "观看人数"),
    FULL_UV("uvPeak", "uv_peak", "峰值观看人数"),
    DURATION("liveDuration", "live_duration", "时长(分钟)"),
    LIVE_START_TIME("liveStartTime", "live_start_time", "直播开始时间"),
    LIVE_END_TIME("liveEndTime", "live_end_time", "直播结束时间"),
    LIKE_NUM("likeNum", "like_num", "点赞数"),
    SHARE_NUM("shareNum", "share_num", "分享数"),
    COMMENT_NUM("commentNum", "comment_num", "评论数"),
    NEW_FAN_NUM("newFanNum", "new_fan_num", "增粉数"),
    RECEIVE_NUM("receiveNum", "receive_num", "收礼数"),
    RECEIVE_AMOUNT("receiveAmount", "receive_amount", "收礼金额(元)"),
    ;

    /**
     * 排序字段(VO)
     */
    private final String vo;
    /**
     * 排序字段(DB)
     */
    private final String field;
    /**
     * 排序字段名称
     */
    private final String fieldName;

    /**
     * 根据前端字符串获取后端表字段
     *
     * @param vo 前端vo字符串
     * @return 后端表字段
     */
    public static String getSortField(String vo) {
        if (!StringUtils.hasText(vo))
            return "";
        for (SortFieldEnum fieldEnum : SortFieldEnum.values()) {
            if (Objects.equals(fieldEnum.getVo(), vo))
                return fieldEnum.getField();
        }
        return "";
    }

    /**
     * 获取排序SQL
     *
     * @param vo          前端vo字符串
     * @param orderMethod 排序方式(1:升序, 2:降序)
     * @return 排序SQL
     */
    public static String getSortSql(String vo, Integer orderMethod) {
        String sortField = getSortField(vo);
        if (StringUtils.hasText(sortField))
            return " order by " + sortField + (Objects.equals(orderMethod, 1) || Objects.isNull(orderMethod) ? " asc" : " desc");
        return " order by score desc";
    }
}
