package cn.fw.dalaran.service.biz.report.score;

import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.dto.report.score.ScoreResultDto;
import cn.fw.dalaran.service.biz.report.CalSupport;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 短视频、直播得分实时汇总实现
 *
 * @author: LYM
 * @date: 2023/12/20 11:20
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ScoreActualCountBizService implements CalSupport {

    /**
     * 发布人主题综合得分(单主题)
     */
    public void calStaffThemeScore(List<ScoreResultDto> staffList, List<LivePool> livePools, List<VideoPool> videoPools) {
        this.calStaffLiveScore(livePools, staffList);
        this.calStaffVideoScore(videoPools, staffList);
    }

    /**
     * 发布人直播得分
     */
    private void calStaffLiveScore(List<LivePool> livePools, List<ScoreResultDto> staffList) {
        Map<Long, List<LivePool>> map = livePools.stream().collect(Collectors.groupingBy(LivePool::getBelongUserId));
        for (ScoreResultDto scoreResultDto : staffList) {
            List<LivePool> poolList = map.getOrDefault(scoreResultDto.getStaffId(), new ArrayList<>());
            if (PublicUtil.isEmpty(poolList)) {
                continue;
            }
            BigDecimal scoreTotal = sumDecimal(poolList, LivePool::getScore);
            scoreResultDto.setLiveScore(divide(scoreTotal, scoreResultDto.getNeedLiveNum(), 4, RoundingMode.HALF_UP));
            //综合得分
            List<LivePool> scorePools = poolList.stream().filter(r -> r.getValidLive() == 11).collect(Collectors.toList());
            if (PublicUtil.isEmpty(scorePools)) {
                continue;
            }
            BigDecimal themeScore = scoreResultDto.getLiveScore().multiply(scoreResultDto.getLiveRate());
            scoreResultDto.setLiveThemeScore(themeScore.compareTo(scoreResultDto.getLiveRate()) >= 0
                    ? scoreResultDto.getLiveRate() : themeScore);
        }
    }

    /**
     * 发布人短视频得分
     */
    private void calStaffVideoScore(List<VideoPool> videoPools, List<ScoreResultDto> staffList) {
        Map<Long, List<VideoPool>> map = videoPools.stream().collect(Collectors.groupingBy(VideoPool::getBelongUserId));
        for (ScoreResultDto scoreResultDto : staffList) {
            List<VideoPool> poolList = map.getOrDefault(scoreResultDto.getStaffId(), new ArrayList<>());
            if (PublicUtil.isEmpty(poolList)) {
                continue;
            }
            BigDecimal scoreTotal = sumDecimal(poolList, VideoPool::getScore);
            scoreResultDto.setVideoScore(divide(scoreTotal, scoreResultDto.getNeedVideoNum(), 4, RoundingMode.HALF_UP));
            //综合得分
            List<VideoPool> scorePools = poolList.stream().filter(r -> r.getValidVideo() == 11).collect(Collectors.toList());
            if (PublicUtil.isEmpty(scorePools)) {
                continue;
            }
            BigDecimal themeScore = scoreResultDto.getVideoScore().multiply(scoreResultDto.getVideoRate());
            scoreResultDto.setVideoThemeScore(themeScore.compareTo(scoreResultDto.getVideoScore()) >= 0
                    ? scoreResultDto.getVideoScore() : themeScore);
        }
    }

    /**
     * 计算门店主题综合得分（单主题）
     */
    public void calShopThemeScore(List<ScoreResultDto> shopList, List<LivePool> livePools,
                                  List<VideoPool> videoPools) {
        this.calShopLiveScore(livePools, shopList);
        this.calShopVideoScore(videoPools, shopList);
    }

    /**
     * 门店直播得分
     */
    private void calShopLiveScore(List<LivePool> livePools, List<ScoreResultDto> shopList) {
        Map<Long, List<LivePool>> map = livePools.stream().collect(Collectors.groupingBy(LivePool::getBelongShopId));
        for (ScoreResultDto scoreResultDto : shopList) {
            List<LivePool> poolList = map.getOrDefault(scoreResultDto.getShopId(), new ArrayList<>());
            BigDecimal liveScoreTotal = sumDecimal(poolList, LivePool::getScore);
            scoreResultDto.setLiveScore(divide(liveScoreTotal, scoreResultDto.getNeedLiveNum(), 4, RoundingMode.HALF_UP));
            //综合得分
            List<LivePool> scorePools = poolList.stream().filter(r -> r.getValidLive() == 11).collect(Collectors.toList());
            if (PublicUtil.isEmpty(scorePools)) {
                continue;
            }
            //得分实际占比
            BigDecimal liveThemeScoreTotal = sumDecimal(scorePools, LivePool::getScore);
            BigDecimal liveThemeScoreAve = divide(liveThemeScoreTotal, scoreResultDto.getNeedLiveNum(), 4, RoundingMode.HALF_UP);
            BigDecimal liveThemeScore = liveThemeScoreAve.multiply(scoreResultDto.getLiveRate());
            scoreResultDto.setLiveThemeScore(liveThemeScore.compareTo(scoreResultDto.getLiveRate()) >= 0
                    ? scoreResultDto.getLiveRate() : liveThemeScore);
        }
        log.info(JSON.toJSONString(shopList));
    }

    /**
     * 门店短视频得分
     */
    private void calShopVideoScore(List<VideoPool> videoPools, List<ScoreResultDto> shopList) {
        Map<Long, List<VideoPool>> map = videoPools.stream().collect(Collectors.groupingBy(VideoPool::getBelongShopId));
        for (ScoreResultDto scoreResultDto : shopList) {
            List<VideoPool> poolList = map.getOrDefault(scoreResultDto.getShopId(), new ArrayList<>());
            BigDecimal videoScoreTotal = sumDecimal(poolList, VideoPool::getScore);
            scoreResultDto.setVideoScore(divide(videoScoreTotal, scoreResultDto.getNeedVideoNum(), 4, RoundingMode.HALF_UP));
            //综合得分
            List<VideoPool> scorePools = poolList.stream().filter(r -> r.getValidVideo() == 11).collect(Collectors.toList());
            if (PublicUtil.isEmpty(scorePools)) {
                continue;
            }
            //得分实际占比
            BigDecimal videoThemeScoreTotal = sumDecimal(scorePools, VideoPool::getScore);
            BigDecimal videoThemeScoreAve = divide(videoThemeScoreTotal, scoreResultDto.getNeedVideoNum(), 4, RoundingMode.HALF_UP);
            BigDecimal videoThemeScore = videoThemeScoreAve.multiply(scoreResultDto.getVideoRate());
            scoreResultDto.setVideoThemeScore(videoThemeScore.compareTo(scoreResultDto.getVideoRate()) >= 0
                    ? scoreResultDto.getVideoRate() : videoThemeScore);
        }
        log.info(JSON.toJSONString(shopList));
    }

    /**
     * 计算管理层主题综合得分
     *
     * @param managerShops 管理门店
     * @param liveList     直播
     * @param videoList    视频
     */
    public List<ScoreResultDto> calManagerThemeScore(List<ScoreResultDto> managerShops, List<LivePool> liveList,
                                                     List<VideoPool> videoList) {
        Map<Long, List<ScoreResultDto>> mangerMap = managerShops.stream().collect(Collectors.groupingBy(ScoreResultDto::getManagerId));
        Map<Long, List<LivePool>> managerLiveMap = liveList.stream().collect(Collectors.groupingBy(LivePool::getFirstManagerId));
        Map<Long, List<VideoPool>> managerVideoMap = videoList.stream().collect(Collectors.groupingBy(VideoPool::getFirstManagerId));
        List<ScoreResultDto> result = new LinkedList<>();
        for (Map.Entry<Long, List<ScoreResultDto>> entry : mangerMap.entrySet()) {
            List<ScoreResultDto> shops = entry.getValue();
            List<LivePool> managerLives = managerLiveMap.getOrDefault(entry.getKey(), new ArrayList<>());
            List<VideoPool> managerVideos = managerVideoMap.getOrDefault(entry.getKey(), new ArrayList<>());
            ScoreResultDto managerScore = this.calThemeScore(shops, managerLives, managerVideos);
            ScoreResultDto scoreResultDto = shops.get(0);
            managerScore.setThemeId(scoreResultDto.getThemeId());
            managerScore.setLiveRate(scoreResultDto.getLiveRate());
            managerScore.setVideoRate(scoreResultDto.getVideoRate());
            managerScore.setManagerId(scoreResultDto.getManagerId());
            managerScore.setManagerName(scoreResultDto.getManagerName());
            result.add(managerScore);
        }
        return result;
    }

    /**
     * 计算主题综合得分
     *
     * @param managerShops 门店范围
     * @param liveList     有效直播数据
     * @param videoList    短视频数据
     */
    public ScoreResultDto calThemeScore(List<ScoreResultDto> managerShops, List<LivePool> liveList,
                                        List<VideoPool> videoList) {
        this.calShopThemeScore(managerShops, liveList, videoList);
        return this.countData(managerShops);
    }

    /**
     * 统计数据
     */
    private ScoreResultDto countData(List<ScoreResultDto> managerShops) {
        ScoreResultDto themeScore = new ScoreResultDto();
        //汇总目标数据
        themeScore.setNeedLiveNum(sumInt(managerShops, ScoreResultDto::getNeedLiveNum));
        themeScore.setLiveQualifiedNum(sumInt(managerShops, ScoreResultDto::getLiveQualifiedNum));
        themeScore.setNeedVideoNum(sumInt(managerShops, ScoreResultDto::getNeedVideoNum));
        themeScore.setVideoQualifiedNum(sumInt(managerShops, ScoreResultDto::getVideoQualifiedNum));
        //计算综合得分
        BigDecimal liveScoreTotal = sumDecimal(managerShops, ScoreResultDto::getLiveScore);
        themeScore.setLiveScore(divide(liveScoreTotal, managerShops.size(), 4, RoundingMode.HALF_UP));

        BigDecimal videoScoreTotal = sumDecimal(managerShops, ScoreResultDto::getVideoScore);
        themeScore.setVideoScore(divide(videoScoreTotal, managerShops.size(), 4, RoundingMode.HALF_UP));

        BigDecimal liveThemeScoreTotal = sumDecimal(managerShops, ScoreResultDto::getLiveThemeScore);
        themeScore.setLiveThemeScore(divide(liveThemeScoreTotal, managerShops.size(), 4, RoundingMode.HALF_UP));

        BigDecimal videoThemeScoreTotal = sumDecimal(managerShops, ScoreResultDto::getVideoThemeScore);
        themeScore.setVideoThemeScore(divide(videoThemeScoreTotal, managerShops.size(), 4, RoundingMode.HALF_UP));
        return themeScore;
    }

    /**
     * 计算月度综合得分(多主题)
     *
     * @param managerShops 门店范围
     * @param liveList     有效直播数据
     * @param videoList    短视频数据
     */
    public ScoreResultDto calMonthScore(List<ScoreResultDto> managerShops, List<LivePool> liveList,
                                        List<VideoPool> videoList) {
        Map<Long, List<ScoreResultDto>> themeShopMap = managerShops.stream().collect(Collectors.groupingBy(ScoreResultDto::getThemeId));
        Map<Long, List<LivePool>> themeLiveMap = liveList.stream().collect(Collectors.groupingBy(LivePool::getThemeId));
        Map<Long, List<VideoPool>> themeVideoMap = videoList.stream().collect(Collectors.groupingBy(VideoPool::getThemeId));
        List<ScoreResultDto> themeScoreList = new LinkedList<>();
        for (Map.Entry<Long, List<ScoreResultDto>> entry : themeShopMap.entrySet()) {
            List<LivePool> themeLives = themeLiveMap.getOrDefault(entry.getKey(), new ArrayList<>());
            List<VideoPool> videoLives = themeVideoMap.getOrDefault(entry.getKey(), new ArrayList<>());
            themeScoreList.add(this.calThemeScore(entry.getValue(), themeLives, videoLives));
        }
        return this.countData(themeScoreList);
    }

    /**
     * 计算门店得分(多主题)
     */
    public List<ScoreResultDto> calShopMonthScoreBatch(List<ScoreResultDto> shopList, List<LivePool> livePools,
                                                       List<VideoPool> videoPools) {

        Map<Long, List<ScoreResultDto>> themeShop = shopList.stream().collect(Collectors.groupingBy(ScoreResultDto::getThemeId));
        Map<Long, List<LivePool>> themeLive = livePools.stream().collect(Collectors.groupingBy(LivePool::getThemeId));
        Map<Long, List<VideoPool>> themeVideo = videoPools.stream().collect(Collectors.groupingBy(VideoPool::getThemeId));
        //处理
        for (Map.Entry<Long, List<ScoreResultDto>> entry : themeShop.entrySet()) {
            Long themeId = entry.getKey();
            List<ScoreResultDto> shops = entry.getValue();
            List<LivePool> lives = themeLive.getOrDefault(themeId, new LinkedList<>());
            List<VideoPool> videos = themeVideo.getOrDefault(themeId, new LinkedList<>());
            this.calShopThemeScore(shops, lives, videos);
        }
        Map<Long, List<ScoreResultDto>> shopMap = shopList.stream().collect(Collectors.groupingBy(ScoreResultDto::getShopId));
        return shopMap.values().stream().map(shops -> {
            ScoreResultDto scoreResultDto = this.countData(shops);
            scoreResultDto.setShopId(shops.get(0).getShopId());
            return scoreResultDto;
        }).collect(Collectors.toList());
    }

    /**
     * 发布人得分(多主题)
     */
    public List<ScoreResultDto> calStaffMonthScoreBatch(List<ScoreResultDto> staffList, List<LivePool> livePools,
                                                        List<VideoPool> videoPools) {

        Map<Long, List<ScoreResultDto>> themeStaff = staffList.stream().collect(Collectors.groupingBy(ScoreResultDto::getThemeId));
        Map<Long, List<LivePool>> themeLive = livePools.stream().collect(Collectors.groupingBy(LivePool::getThemeId));
        Map<Long, List<VideoPool>> themeVideo = videoPools.stream().collect(Collectors.groupingBy(VideoPool::getThemeId));
        //处理
        for (Map.Entry<Long, List<ScoreResultDto>> entry : themeStaff.entrySet()) {
            Long themeId = entry.getKey();
            List<ScoreResultDto> staffs = entry.getValue();
            List<LivePool> lives = themeLive.getOrDefault(themeId, new LinkedList<>());
            List<VideoPool> videos = themeVideo.getOrDefault(themeId, new LinkedList<>());
            this.calStaffThemeScore(staffs, lives, videos);
        }
        Map<Long, List<ScoreResultDto>> staffMap = staffList.stream().collect(Collectors.groupingBy(ScoreResultDto::getStaffId));
        return staffMap.values().stream().map(shops -> {
            ScoreResultDto scoreResultDto = this.countData(shops);
            scoreResultDto.setStaffId(shops.get(0).getStaffId());
            scoreResultDto.setStaffName(shops.get(0).getStaffName());
            return scoreResultDto;
        }).collect(Collectors.toList());
    }

    /**
     * 计算管理层得分(多主题)
     */
    public List<ScoreResultDto> calManagerMonthScoreBatch(List<ScoreResultDto> managerShops, List<LivePool> liveList,
                                                          List<VideoPool> videoList) {
        Map<Long, List<ScoreResultDto>> themeManager = managerShops.stream().collect(Collectors.groupingBy(ScoreResultDto::getThemeId));
        Map<Long, List<LivePool>> themeLive = liveList.stream().collect(Collectors.groupingBy(LivePool::getThemeId));
        Map<Long, List<VideoPool>> themeVideo = videoList.stream().collect(Collectors.groupingBy(VideoPool::getThemeId));
        //处理
        List<ScoreResultDto> manages = new LinkedList<>();
        for (Map.Entry<Long, List<ScoreResultDto>> entry : themeManager.entrySet()) {
            Long themeId = entry.getKey();
            List<ScoreResultDto> managers = entry.getValue();
            List<LivePool> lives = themeLive.getOrDefault(themeId, new LinkedList<>());
            List<VideoPool> videos = themeVideo.getOrDefault(themeId, new LinkedList<>());
            manages.addAll(this.calManagerThemeScore(managers, lives, videos));
        }
        Map<Long, List<ScoreResultDto>> managerMap = manages.stream().collect(Collectors.groupingBy(ScoreResultDto::getManagerId));
        return managerMap.values().stream().map(managers -> {
            ScoreResultDto resultDto = managers.get(0);
            ScoreResultDto scoreResultDto = this.countData(managers);
            scoreResultDto.setManagerId(resultDto.getManagerId());
            scoreResultDto.setManagerName(resultDto.getManagerName());
            scoreResultDto.setLiveRate(resultDto.getLiveRate());
            scoreResultDto.setVideoRate(resultDto.getVideoRate());
            return scoreResultDto;
        }).collect(Collectors.toList());
    }

    /**
     * 计算主题得分
     */
    public List<ScoreResultDto> calThemeScoreBatch(List<ScoreResultDto> scoreCalRange, List<LivePool> liveList,
                                                   List<VideoPool> videoList) {
        Map<Long, List<LivePool>> themeLiveMap = liveList.stream().collect(Collectors.groupingBy(LivePool::getThemeId));
        Map<Long, List<VideoPool>> themeVideoMap = videoList.stream().collect(Collectors.groupingBy(VideoPool::getThemeId));
        Map<Long, List<ScoreResultDto>> themeShopMap = scoreCalRange.stream().collect(Collectors.groupingBy(ScoreResultDto::getThemeId));

        List<ScoreResultDto> result = new LinkedList<>();
        for (Map.Entry<Long, List<ScoreResultDto>> entry : themeShopMap.entrySet()) {
            List<LivePool> themeLives = themeLiveMap.getOrDefault(entry.getKey(), new ArrayList<>());
            List<VideoPool> videoLives = themeVideoMap.getOrDefault(entry.getKey(), new ArrayList<>());
            List<ScoreResultDto> value = entry.getValue();
            ScoreResultDto scoreResultDto = this.calThemeScore(value, themeLives, videoLives);
            scoreResultDto.setThemeId(entry.getKey());
            scoreResultDto.setLiveRate(value.get(0).getLiveRate());
            scoreResultDto.setVideoRate(value.get(0).getVideoRate());
            result.add(scoreResultDto);
        }
        return result;
    }
}

