package cn.fw.dalaran.service.biz.report.score;

import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.db.snapshot.ConsultantSnapshot;
import cn.fw.dalaran.domain.db.snapshot.ManagerSnapshot;
import cn.fw.dalaran.domain.enums.report.ReportListTypeEnum;
import cn.fw.dalaran.domain.param.report.score.ScoreBaseQueryParam;
import cn.fw.dalaran.domain.param.report.score.ScoreTrendParam;
import cn.fw.dalaran.service.data.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 得分公共方法
 *
 * @author: lym
 * @date: 2023/6/14
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ScoreReportCommonBizService {
    private final LivePoolService livePoolService;
    private final VideoPoolService videoPoolService;
    private final ActivityThemeService activityThemeService;
    private final IManagerSnapshotService managerSnapshotService;
    private final IConsultantSnapshotService consultantSnapshotService;

    /**
     * 主题得分报表展示范围
     */
    public void calThemeReportRange(ActivityTheme theme, ScoreBaseQueryParam param,
                                    Integer rowType, String rowId, Integer pointType) {
        param.setThemeId(theme.getId());
        param.setThemeIds(Collections.singletonList(theme.getId()));
        param.setShopIds(theme.getShopIdByReport(param.getShopIds()));
        //二次限定
        if (PublicUtil.isEmpty(pointType) || PublicUtil.isEmpty(param.getShopIds())) {
            return;
        }
        if (ReportListTypeEnum.SHOP.getValue().equals(rowType)) {
            param.setShopIds(Collections.singletonList(Long.valueOf(rowId)));
            return;
        }
        if (ReportListTypeEnum.MANAGER_1.getValue().equals(rowType)) {
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .eq(ManagerSnapshot::getThemeId, theme.getId())
                    .eq(ManagerSnapshot::getFirstManagerId, Long.valueOf(rowId)));
            param.setShopIds(snapshotList.stream().map(ManagerSnapshot::getShopId).collect(Collectors.toList()));
            return;
        }
        if (ReportListTypeEnum.MANAGER_2.getValue().equals(rowType)) {
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .eq(ManagerSnapshot::getThemeId, theme.getId())
                    .eq(ManagerSnapshot::getSecondManagerId, Long.valueOf(rowId)));
            param.setShopIds(snapshotList.stream().map(ManagerSnapshot::getShopId).collect(Collectors.toList()));
            return;
        }
        if (ReportListTypeEnum.MANAGER_3.getValue().equals(rowType)) {
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .eq(ManagerSnapshot::getThemeId, theme.getId())
                    .eq(ManagerSnapshot::getThirdManagerId, Long.valueOf(rowId)));
            param.setShopIds(snapshotList.stream().map(ManagerSnapshot::getShopId).collect(Collectors.toList()));
            return;
        }
        if (ReportListTypeEnum.MANAGER_4.getValue().equals(rowType)) {
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .eq(ManagerSnapshot::getThemeId, theme.getId())
                    .eq(ManagerSnapshot::getFourthManagerId, Long.valueOf(rowId)));
            param.setShopIds(snapshotList.stream().map(ManagerSnapshot::getShopId).collect(Collectors.toList()));
        }
    }

    /**
     * 主题得分报表展示范围
     */
    public void calMonthReportRange(ScoreBaseQueryParam param, Integer rowType, String rowId, Integer pointType) {
        Date start = DateUtil.monthBegin(param.getBeginTime());
        Date end = DateUtil.monthEnd(param.getEndTime());
        List<ActivityTheme> themeList = activityThemeService.list(Wrappers.<ActivityTheme>lambdaQuery()
                .eq(ActivityTheme::getGroupId, param.getGroupId())
                .eq(Objects.nonNull(param.getThemeId()), ActivityTheme::getId, param.getThemeId())
                .ge(ActivityTheme::getStartTime, start)
                .le(ActivityTheme::getStartTime, end));
        Set<Long> shopRange = new HashSet<>();
        for (ActivityTheme theme : themeList) {
            List<Long> shopIdByReport = theme.getShopIdByReport(param.getShopIds());
            if (PublicUtil.isEmpty(shopIdByReport)) {
                continue;
            }
            shopRange.addAll(shopIdByReport);
            param.getThemeIds().add(theme.getId());
            param.getThemeList().add(theme);
        }
        param.setShopIds(new ArrayList<>(shopRange));
        //二次限定
        if (PublicUtil.isEmpty(pointType) || PublicUtil.isEmpty(param.getThemeIds())) {
            return;
        }
        if (ReportListTypeEnum.SHOP.getValue().equals(rowType)) {
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .in(ManagerSnapshot::getThemeId, param.getThemeIds())
                    .eq(ManagerSnapshot::getShopId, Long.valueOf(rowId)));
            param.setThemeIds(snapshotList.stream().map(ManagerSnapshot::getThemeId).collect(Collectors.toList()));
            param.setShopIds(Collections.singletonList(Long.valueOf(rowId)));
            return;
        }
        if (ReportListTypeEnum.USER.getValue().equals(rowType)) {
            List<ConsultantSnapshot> snapshotList = consultantSnapshotService.list(Wrappers.<ConsultantSnapshot>lambdaQuery()
                    .in(ConsultantSnapshot::getThemeId, param.getThemeIds())
                    .in(ConsultantSnapshot::getShopId, param.getShopIds())
                    .eq(ConsultantSnapshot::getStaffId, Long.valueOf(rowId)));
            param.setThemeIds(snapshotList.stream().map(ConsultantSnapshot::getThemeId).collect(Collectors.toList()));
            param.setShopIds(snapshotList.stream().map(ConsultantSnapshot::getShopId).collect(Collectors.toList()));
            return;
        }
        if (ReportListTypeEnum.MANAGER_1.getValue().equals(rowType)) {
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .in(ManagerSnapshot::getThemeId, param.getThemeIds())
                    .eq(ManagerSnapshot::getFirstManagerId, Long.valueOf(rowId)));
            param.setThemeIds(snapshotList.stream().map(ManagerSnapshot::getThemeId).collect(Collectors.toList()));
            param.setShopIds(snapshotList.stream().map(ManagerSnapshot::getShopId).collect(Collectors.toList()));
            return;
        }
        if (ReportListTypeEnum.MANAGER_2.getValue().equals(rowType)) {
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .in(ManagerSnapshot::getThemeId, param.getThemeIds())
                    .eq(ManagerSnapshot::getSecondManagerId, Long.valueOf(rowId)));
            param.setThemeIds(snapshotList.stream().map(ManagerSnapshot::getThemeId).collect(Collectors.toList()));
            param.setShopIds(snapshotList.stream().map(ManagerSnapshot::getShopId).collect(Collectors.toList()));
            return;
        }
        if (ReportListTypeEnum.MANAGER_3.getValue().equals(rowType)) {
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .in(ManagerSnapshot::getThemeId, param.getThemeIds())
                    .eq(ManagerSnapshot::getThirdManagerId, Long.valueOf(rowId)));
            param.setThemeIds(snapshotList.stream().map(ManagerSnapshot::getThemeId).collect(Collectors.toList()));
            param.setShopIds(snapshotList.stream().map(ManagerSnapshot::getShopId).collect(Collectors.toList()));
            return;
        }
        if (ReportListTypeEnum.MANAGER_4.getValue().equals(rowType)) {
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .in(ManagerSnapshot::getThemeId, param.getThemeIds())
                    .eq(ManagerSnapshot::getFourthManagerId, Long.valueOf(rowId)));
            param.setThemeIds(snapshotList.stream().map(ManagerSnapshot::getThemeId).collect(Collectors.toList()));
            param.setShopIds(snapshotList.stream().map(ManagerSnapshot::getShopId).collect(Collectors.toList()));
            return;
        }
        if (ReportListTypeEnum.THEME.getValue().equals(rowType)) {
            param.setThemeIds(Collections.singletonList(Long.valueOf(rowId)));
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .eq(ManagerSnapshot::getThemeId, Long.valueOf(rowId)));
            param.setShopIds(snapshotList.stream().map(ManagerSnapshot::getShopId).collect(Collectors.toList()));
        }
    }

    /**
     * 查询直播趋势数据
     */
    public List<LivePool> getLiveTrendData(ScoreTrendParam param) {
        return livePoolService.list(Wrappers.<LivePool>lambdaQuery()
                .select(LivePool::getId, LivePool::getBelongUserId, LivePool::getBelongShopId, LivePool::getLiveStartTime,
                        LivePool::getScore, LivePool::getThemeId, LivePool::getValidLive)
                .in(LivePool::getBelongShopId, param.getShopIds())
                .in(LivePool::getThemeId, param.getThemeIds())
                .eq(LivePool::getStatus, 1)
                .eq(LivePool::getValidLive, 11));
    }

    /**
     * 查询短视频趋势数据
     */
    public List<VideoPool> getVideoTrendData(ScoreTrendParam param) {
        return videoPoolService.list(Wrappers.<VideoPool>lambdaQuery()
                .select(VideoPool::getId, VideoPool::getBelongUserId, VideoPool::getBelongShopId, VideoPool::getScore,
                        VideoPool::getThemeId, VideoPool::getVideoId, VideoPool::getValidVideo, VideoPool::getCollectDay)
                .in(VideoPool::getBelongShopId, param.getShopIds())
                .in(VideoPool::getThemeId, param.getThemeIds())
                .eq(VideoPool::getStatus, 1)
                .eq(VideoPool::getValidVideo, 11));
    }

    /**
     * 查询直播有效数据
     */
    public List<LivePool> getLiveList(ScoreBaseQueryParam param, Integer rowType, String rowId, Integer pointType) {
        LambdaQueryWrapper<LivePool> wrapper = Wrappers.<LivePool>lambdaQuery()
                .select(LivePool::getId, LivePool::getBelongUserId, LivePool::getBelongShopId,
                        LivePool::getFirstManagerId, LivePool::getLiveStartTime, LivePool::getSecondManagerId,
                        LivePool::getThirdManagerId, LivePool::getFourthManagerId, LivePool::getScore,
                        LivePool::getThemeId, LivePool::getTheme, LivePool::getValidLive)
                .in(LivePool::getBelongShopId, param.getShopIds())
                .in(LivePool::getThemeId, param.getThemeIds())
                .gt(LivePool::getValidLive, 1)
                .eq(LivePool::getStatus, 1);
        if (PublicUtil.isNotEmpty(pointType)) {
            Long toRowId = Long.valueOf(rowId);
            if (ReportListTypeEnum.SHOP.getValue().equals(rowType)) {
                wrapper.eq(LivePool::getBelongShopId, toRowId);
            } else if (ReportListTypeEnum.USER.getValue().equals(rowType)) {
                wrapper.eq(LivePool::getBelongUserId, toRowId);
            } else if (ReportListTypeEnum.MANAGER_1.getValue().equals(rowType)) {
                wrapper.eq(LivePool::getFirstManagerId, toRowId);
            } else if (ReportListTypeEnum.MANAGER_2.getValue().equals(rowType)) {
                wrapper.eq(LivePool::getSecondManagerId, toRowId);
            } else if (ReportListTypeEnum.MANAGER_3.getValue().equals(rowType)) {
                wrapper.eq(LivePool::getThirdManagerId, toRowId);
            } else if (ReportListTypeEnum.MANAGER_4.getValue().equals(rowType)) {
                wrapper.eq(LivePool::getFourthManagerId, toRowId);
            } else if (ReportListTypeEnum.THEME.getValue().equals(rowType)) {
                wrapper.eq(LivePool::getThemeId, toRowId);
            }
        }
        return livePoolService.list(wrapper);
    }

    /**
     * 查询短视频有效数据
     */
    public List<VideoPool> getVideoList(ScoreBaseQueryParam param, Integer rowType, String rowId, Integer pointType) {
        LambdaQueryWrapper<VideoPool> wrapper = Wrappers.<VideoPool>lambdaQuery()
                .select(VideoPool::getId, VideoPool::getBelongUserId, VideoPool::getBelongShopId,
                        VideoPool::getFirstManagerId, VideoPool::getSecondManagerId, VideoPool::getThirdManagerId,
                        VideoPool::getFourthManagerId, VideoPool::getScore, VideoPool::getThemeId,
                        VideoPool::getTheme, VideoPool::getValidVideo)
                .in(VideoPool::getBelongShopId, param.getShopIds())
                .in(VideoPool::getThemeId, param.getThemeIds())
                .eq(VideoPool::getLatestRecord, 1)
                .eq(VideoPool::getStatus, 1)
                .gt(VideoPool::getValidVideo, 1);
        if (PublicUtil.isNotEmpty(pointType)) {
            Long toRowId = Long.valueOf(rowId);
            if (ReportListTypeEnum.SHOP.getValue().equals(rowType)) {
                wrapper.eq(VideoPool::getBelongShopId, toRowId);
            } else if (ReportListTypeEnum.USER.getValue().equals(rowType)) {
                wrapper.eq(VideoPool::getBelongUserId, toRowId);
            } else if (ReportListTypeEnum.MANAGER_1.getValue().equals(rowType)) {
                wrapper.eq(VideoPool::getFirstManagerId, toRowId);
            } else if (ReportListTypeEnum.MANAGER_2.getValue().equals(rowType)) {
                wrapper.eq(VideoPool::getSecondManagerId, toRowId);
            } else if (ReportListTypeEnum.MANAGER_3.getValue().equals(rowType)) {
                wrapper.eq(VideoPool::getThirdManagerId, toRowId);
            } else if (ReportListTypeEnum.MANAGER_4.getValue().equals(rowType)) {
                wrapper.eq(VideoPool::getFourthManagerId, toRowId);
            } else if (ReportListTypeEnum.THEME.getValue().equals(rowType)) {
                wrapper.eq(VideoPool::getThemeId, toRowId);
            }
        }
        return videoPoolService.list(wrapper);
    }
}
