package cn.fw.dalaran.service.biz.report.score;

import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.common.domain.vo.NormTrendItemVO;
import cn.fw.dalaran.common.domain.vo.NormTrendVO;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.db.snapshot.ConsultantSnapshot;
import cn.fw.dalaran.domain.dto.report.score.ScoreResultDto;
import cn.fw.dalaran.domain.enums.report.ReportListTypeEnum;
import cn.fw.dalaran.domain.param.report.score.ScoreBaseQueryParam;
import cn.fw.dalaran.domain.param.report.score.ScoreListParam;
import cn.fw.dalaran.domain.param.report.score.ScoreTrendParam;
import cn.fw.dalaran.domain.vo.config.FansLimitVo;
import cn.fw.dalaran.domain.vo.report.UserCalRuleVo;
import cn.fw.dalaran.domain.vo.report.UserFansCountVo;
import cn.fw.dalaran.domain.vo.report.score.ScoreSummaryVo;
import cn.fw.dalaran.domain.vo.report.score.ThemeScoreListVo;
import cn.fw.dalaran.rpc.oop.OopRpc;
import cn.fw.dalaran.service.biz.report.ReportCommonBizService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.dalaran.service.data.IConsultantSnapshotService;
import cn.fw.dalaran.service.data.LivePoolService;
import cn.fw.dalaran.service.data.VideoPoolService;
import cn.fw.oop.sdk.result.ShopInfoVo;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 主题得分报表
 *
 * @author: lym
 * @date: 2023/6/14
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ScoreThemeBizService {
    private final OopRpc oopRpc;
    private final LivePoolService livePoolService;
    private final VideoPoolService videoPoolService;
    private final ActivityThemeService activityThemeService;
    private final ReportCommonBizService reportCommonBizService;
    private final IConsultantSnapshotService consultantSnapshotService;
    private final ScoreActualCountBizService scoreActualCountBizService;
    private final ScoreReportCommonBizService scoreReportCommonBizService;

    /**
     * 主题得分统计
     *
     * @param param 查询参数
     * @return 主题得分统计
     */
    public ScoreSummaryVo getScoreSummary(ScoreBaseQueryParam param) {
        ScoreSummaryVo scoreSummaryVo = new ScoreSummaryVo();
        ActivityTheme theme = activityThemeService.getById(param.getThemeId());
        scoreReportCommonBizService.calThemeReportRange(theme, param, null, null, null);
        if (PublicUtil.isEmpty(param.getShopIds())) {
            return scoreSummaryVo;
        }
        //有效计分数据
        List<LivePool> liveList = scoreReportCommonBizService.getLiveList(param, null, null, null);
        List<VideoPool> videoList = scoreReportCommonBizService.getVideoList(param, null, null, null);
        List<ScoreResultDto> shopRange = reportCommonBizService.getScoreCalRange(2,
                param.getGroupId(), param.getThemeIds(), param.getShopIds(), null);
        ScoreResultDto themeScore = scoreActualCountBizService.calThemeScore(shopRange, liveList, videoList);
        scoreSummaryVo.setI2(themeScore.getLiveThemeScore());
        scoreSummaryVo.setI3(themeScore.getVideoThemeScore());
        BigDecimal themeScore1 = themeScore.getThemeScore();
        scoreSummaryVo.setI1(themeScore1);
        return scoreSummaryVo;
    }

    /**
     * 获取主题得分趋势
     *
     * @param param 查询参数
     * @return 主题趋势
     */
    public NormTrendVO<BigDecimal> getTrend(ScoreTrendParam param) {
        NormTrendVO<BigDecimal> trendVo = new NormTrendVO<>();
        ActivityTheme theme = activityThemeService.getById(param.getThemeId());
        scoreReportCommonBizService.calThemeReportRange(theme, param, null, null, null);
        if (PublicUtil.isEmpty(param.getShopIds())) {
            return trendVo;
        }
        List<NormTrendItemVO<BigDecimal>> trendList = new LinkedList<>();
        List<ScoreResultDto> scoreCalRange = reportCommonBizService.getScoreCalRange(2,
                param.getGroupId(), param.getThemeIds(), param.getShopIds(), null);
        List<LivePool> liveList = scoreReportCommonBizService.getLiveTrendData(param);
        List<LocalDate> trendDate = reportCommonBizService.getTrendDate(theme.getStartTime(), theme.getEndTime());
        List<VideoPool> videoList = scoreReportCommonBizService.getVideoTrendData(param);
        for (LocalDate date : trendDate) {
            Map<String, VideoPool> filterVideos = videoList.stream()
                    .filter(r -> !r.getCollectDay().isAfter(date))
                    .collect(Collectors.toMap(VideoPool::getVideoId, r -> r, (o, n) -> n));
            List<LivePool> dateLives = liveList.stream()
                    .filter(r -> !DateUtil.date2LocalDate(r.getLiveStartTime()).isAfter(date))
                    .collect(Collectors.toList());
            ScoreResultDto result = scoreActualCountBizService.calThemeScore(scoreCalRange, dateLives, new ArrayList<>(filterVideos.values()));
            NormTrendItemVO<BigDecimal> itemVO = new NormTrendItemVO<>();
            itemVO.setDate(date.toString());
            itemVO.setValue(result.getThemeScore());
            trendList.add(itemVO);
        }
        trendVo.setTeam(trendList);
        return trendVo;
    }

    /**
     * 获取主题得分分析列表
     *
     * @param param 查询参数
     * @return 主题得分分析列表
     */
    public List<ThemeScoreListVo> getScoreAnalysis(ScoreListParam param) {
        ActivityTheme theme = activityThemeService.getById(param.getThemeId());
        scoreReportCommonBizService.calThemeReportRange(theme, param, param.getRowType(), param.getRowId(), param.getPointType());
        //本次分析类型
        Integer rowType = PublicUtil.isNotEmpty(param.getPointType()) ? param.getPointType() : param.getRowType();
        ReportListTypeEnum typeEnum = ReportListTypeEnum.ofValue(rowType);
        if (PublicUtil.isEmpty(param.getShopIds()) || PublicUtil.isEmpty(typeEnum) || PublicUtil.isEmpty(param.getShopIds())) {
            return new ArrayList<>();
        }
        //有效计分数据
        List<LivePool> liveList = scoreReportCommonBizService.getLiveList(param,
                param.getRowType(), param.getRowId(), param.getPointType());
        List<VideoPool> videoList = scoreReportCommonBizService.getVideoList(param,
                param.getRowType(), param.getRowId(), param.getPointType());
        switch (typeEnum) {
            case SHOP:
                return this.analysisByShop(param, liveList, videoList);
            case USER:
                return this.analysisByConsultant(param, liveList, videoList);
            case MANAGER_1:
                return this.analysisByManager(param, liveList, videoList, RoleMsgEnum.XSZG);
            case MANAGER_2:
                liveList.forEach(r -> r.setFirstManagerId(r.getSecondManagerId()));
                videoList.forEach(r -> r.setFirstManagerId(r.getSecondManagerId()));
                return this.analysisByManager(param, liveList, videoList, RoleMsgEnum.XSJL);
            case MANAGER_3:
                liveList.forEach(r -> r.setFirstManagerId(r.getThirdManagerId()));
                videoList.forEach(r -> r.setFirstManagerId(r.getThirdManagerId()));
                return this.analysisByManager(param, liveList, videoList, RoleMsgEnum.XSZJL);
            case MANAGER_4:
                liveList.forEach(r -> r.setFirstManagerId(r.getFourthManagerId()));
                videoList.forEach(r -> r.setFirstManagerId(r.getFourthManagerId()));
                return this.analysisByManager(param, liveList, videoList, RoleMsgEnum.XSSJGL);
            default:
                return new ArrayList<>();
        }
    }

    /**
     * 按门店分析
     */
    private List<ThemeScoreListVo> analysisByShop(ScoreListParam param, List<LivePool> liveList,
                                                  List<VideoPool> videoList) {
        List<ShopInfoVo> shopInfoList = oopRpc.getShopByIds(param.getShopIds());
        Map<Object, String> nameMap = shopInfoList.stream()
                .collect(Collectors.toMap(ShopInfoVo::getId, ShopInfoVo::getShortName));
        //计算门店得分率
        List<ScoreResultDto> shopRange = reportCommonBizService.getScoreCalRange(2,
                param.getGroupId(), param.getThemeIds(), param.getShopIds(), null);
        scoreActualCountBizService.calShopThemeScore(shopRange, liveList, videoList);
        Map<Object, ScoreResultDto> dataMap = shopRange.stream().collect(Collectors.toMap(ScoreResultDto::getShopId, r -> r));
        return this.buildAnalysisResult(dataMap, nameMap);
    }

    /**
     * 按销售顾问分析
     */
    private List<ThemeScoreListVo> analysisByConsultant(ScoreListParam param, List<LivePool> liveList,
                                                        List<VideoPool> videoList) {
        List<ScoreResultDto> staffRange = reportCommonBizService.getScoreCalRange(1,
                param.getGroupId(), param.getThemeIds(), param.getShopIds(), null);
        scoreActualCountBizService.calStaffThemeScore(staffRange, liveList, videoList);
        Map<Object, ScoreResultDto> datamap = staffRange.stream().collect(Collectors.toMap(ScoreResultDto::getStaffId, r -> r));
        Map<Object, String> nameMap = staffRange.stream().collect(Collectors.toMap(ScoreResultDto::getStaffId, ScoreResultDto::getStaffName));
        List<ThemeScoreListVo> themeScoreListVos = this.buildAnalysisResult(datamap, nameMap);
        Map<Long, UserFansCountVo> userFansCountMap =  reportCommonBizService.getThemeFansCount(param.getThemeId());
        themeScoreListVos.forEach(vo -> vo.setUserFans(userFansCountMap.get(Long.parseLong(vo.getId()))));
        return themeScoreListVos;
    }

    /**
     * 按管理层分析
     *
     * @param param     主题
     * @param liveList  K:管理层人员id, V:归属该管理层人员的直播数据(包含2,11)
     * @param videoList K:管理层人员id, V:归属该管理层人员的短视频数据(包含2,11)
     * @param role      所有管理层人员信息
     * @return 管理层综合得分列表报表
     */
    private List<ThemeScoreListVo> analysisByManager(ScoreListParam param, List<LivePool> liveList,
                                                     List<VideoPool> videoList, RoleMsgEnum role) {
        List<ScoreResultDto> manageShopRange = reportCommonBizService.getScoreCalRange(3, param.getGroupId(),
                param.getThemeIds(), param.getShopIds(), role);
        List<ScoreResultDto> managerList = scoreActualCountBizService.calManagerThemeScore(manageShopRange, liveList, videoList);
        Map<Object, String> nameMap = managerList.stream().collect(Collectors.toMap(ScoreResultDto::getManagerId, ScoreResultDto::getManagerName));
        Map<Object, ScoreResultDto> dataMap = managerList.stream().collect(Collectors.toMap(ScoreResultDto::getManagerId, r -> r));
        return this.buildAnalysisResult(dataMap, nameMap);
    }

    /**
     * 组装分析结果
     *
     * @param dataMap 数据集合
     * @param nameMap 名称集合
     * @return 结果
     */
    private List<ThemeScoreListVo> buildAnalysisResult(Map<Object, ScoreResultDto> dataMap, Map<Object, String> nameMap) {
        return nameMap.keySet().stream()
                .map(dataId -> {
                    ThemeScoreListVo listVo = new ThemeScoreListVo();
                    listVo.setId(dataId.toString());
                    listVo.setName(nameMap.get(dataId));
                    //设置数据
                    ScoreResultDto result = dataMap.get(dataId);
                    if (PublicUtil.isNotEmpty(result)) {
                        listVo.setI1(result.getThemeScore());
                        listVo.setI2(result.getLiveThemeScore());
                        listVo.setI3(result.getVideoThemeScore());
                        listVo.setI4(result.getLiveRate());
                        listVo.setI5(result.getLiveScore());
                        listVo.setI6(result.getVideoRate());
                        listVo.setI7(result.getVideoScore());
                    }
                    return listVo;
                }).sorted(Comparator.comparing(ThemeScoreListVo::getI1, Comparator.reverseOrder())
                        .thenComparing(ThemeScoreListVo::getI2, Comparator.reverseOrder())
                        .thenComparing(ThemeScoreListVo::getI3, Comparator.reverseOrder()))
                .collect(Collectors.toList());
    }

    /**
     * 主题得分情况
     */
    public UserCalRuleVo getScoreUserCalRule(Long themeId, Long staffId) {
        ActivityTheme theme = activityThemeService.getById(themeId);
        PublicUtil.assertNotEmpty(theme, "主题不存在");
        //有效主题
        List<VideoPool> videoPoolList = videoPoolService.list(Wrappers.<VideoPool>lambdaQuery()
                .select(VideoPool::getId)
                .eq(VideoPool::getValidVideo, 2)
                .in(VideoPool::getThemeId, themeId)
                .in(VideoPool::getBelongUserId, staffId)
                .eq(VideoPool::getLatestRecord, 1));
        //有效直播
        List<LivePool> livePoolList = livePoolService.list(Wrappers.<LivePool>lambdaQuery()
                .select(LivePool::getId)
                .eq(LivePool::getValidLive, 2)
                .in(LivePool::getThemeId, themeId)
                .in(LivePool::getBelongUserId, staffId));
        UserCalRuleVo userCalRuteVo = new UserCalRuleVo();
        //短视频
        userCalRuteVo.setNeedVideoValidTimes(theme.getVideoValidTimes());
        userCalRuteVo.setCurrentVideoValidTimes(videoPoolList.size());
        userCalRuteVo.setVideoTimesQualified(videoPoolList.size() >= theme.getVideoValidTimes());
        //直播
        userCalRuteVo.setNeedLiveValidTimes(theme.getLiveValidTimes());
        userCalRuteVo.setCurrentLiveValidTimes(livePoolList.size());
        userCalRuteVo.setLiveTimesQualified(livePoolList.size() >= theme.getLiveValidTimes());
        //账号粉丝前置条件判定
        List<ConsultantSnapshot> list = consultantSnapshotService.list(Wrappers.<ConsultantSnapshot>lambdaQuery()
                .eq(ConsultantSnapshot::getThemeId, themeId)
                .eq(ConsultantSnapshot::getStaffId, staffId));
        if (PublicUtil.isNotEmpty(theme.getFansLimit()) && PublicUtil.isNotEmpty(list)) {
            ConsultantSnapshot snapshot = list.get(0);
            Map<Integer, Integer> collect = JSON.parseArray(theme.getFansLimit(), FansLimitVo.class).stream()
                    .collect(Collectors.toMap(FansLimitVo::getPlatformType, FansLimitVo::getMinNum));
            List<UserCalRuleVo.AccountFansLimit> fansLimits = snapshot.getAccountInfoList().stream().map(r -> {
                UserCalRuleVo.AccountFansLimit accountFansLimit = new UserCalRuleVo.AccountFansLimit();
                accountFansLimit.setAccountNo(r.getAccount());
                accountFansLimit.setPlatformType(r.getType().getValue());
                accountFansLimit.setPlatformName(r.getType().getName());
                accountFansLimit.setMinNum(collect.getOrDefault(r.getType().getValue(), 0));
                accountFansLimit.setFansNum(r.getFansNum());
                //accountFansLimit.setFansQualified(r.getFansNum() >= accountFansLimit.getMinNum());
                return accountFansLimit;
            }).collect(Collectors.toList());
            userCalRuteVo.setFansLimit(fansLimits);
        }
        return userCalRuteVo;
    }
}
