package cn.fw.dalaran.service.biz.report.video;

import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.db.snapshot.ConsultantSnapshot;
import cn.fw.dalaran.domain.db.snapshot.ManagerSnapshot;
import cn.fw.dalaran.domain.enums.report.ReportListTypeEnum;
import cn.fw.dalaran.domain.param.report.video.VideoBaseQueryParam;
import cn.fw.dalaran.domain.param.report.video.VideoTrendParam;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.dalaran.service.data.IConsultantSnapshotService;
import cn.fw.dalaran.service.data.IManagerSnapshotService;
import cn.fw.dalaran.service.data.VideoPoolService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 短视频报表公共方法
 *
 * @author: juzi
 * @date: 2023/9/14 14:04
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class VideoReportCommonBizService {
    private final VideoPoolService videoPoolService;
    private final ActivityThemeService activityThemeService;
    private final IManagerSnapshotService managerSnapshotService;
    private final IConsultantSnapshotService consultantSnapshotService;

    /**
     * 主题得分报表展示范围
     */
    public void calMonthReportRange(VideoBaseQueryParam param, Integer rowType, String rowId, Integer pointType) {
        if(param.getThemeDimension()){
            ActivityTheme theme = activityThemeService.getById(param.getThemeId());
            param.setThemeId(theme.getId());
            param.setThemeIds(Collections.singletonList(theme.getId()));
            param.setShopIds(theme.getShopIdByReport(param.getShopIds()));
        }else {
            Date start = DateUtil.monthBegin(param.getBeginTime());
            Date end = DateUtil.monthEnd(param.getEndTime());
            List<ActivityTheme> themeList = activityThemeService.list(Wrappers.<ActivityTheme>lambdaQuery()
                    .eq(ActivityTheme::getGroupId, param.getGroupId())
                    .ge(ActivityTheme::getStartTime, start)
                    .le(ActivityTheme::getStartTime, end));
            Set<Long> shopRange = new HashSet<>();
            for (ActivityTheme theme : themeList) {
                List<Long> shopIdByReport = theme.getShopIdByReport(param.getShopIds());
                if (PublicUtil.isEmpty(shopIdByReport)) {
                    continue;
                }
                shopRange.addAll(shopIdByReport);
                param.getThemeIds().add(theme.getId());
            }
            param.setShopIds(new ArrayList<>(shopRange));
        }
        //二次限定
        if (PublicUtil.isEmpty(pointType) || PublicUtil.isEmpty(param.getThemeIds())) {
            return;
        }
        if (ReportListTypeEnum.SHOP.getValue().equals(rowType)) {
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .in(ManagerSnapshot::getThemeId, param.getThemeIds())
                    .eq(ManagerSnapshot::getShopId, Long.valueOf(rowId)));
            param.setThemeIds(snapshotList.stream().map(ManagerSnapshot::getThemeId).collect(Collectors.toList()));
            param.setShopIds(Collections.singletonList(Long.valueOf(rowId)));
            return;
        }
        if (ReportListTypeEnum.MANAGER_1.getValue().equals(rowType)) {
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .in(ManagerSnapshot::getThemeId, param.getThemeIds())
                    .in(ManagerSnapshot::getShopId, param.getShopIds())
                    .eq(ManagerSnapshot::getFirstManagerId, Long.valueOf(rowId)));
            param.setThemeIds(snapshotList.stream().map(ManagerSnapshot::getThemeId).collect(Collectors.toList()));
            param.setShopIds(snapshotList.stream().map(ManagerSnapshot::getShopId).collect(Collectors.toList()));
            return;
        }
        if (ReportListTypeEnum.MANAGER_2.getValue().equals(rowType)) {
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .in(ManagerSnapshot::getThemeId, param.getThemeIds())
                    .in(ManagerSnapshot::getShopId, param.getShopIds())
                    .eq(ManagerSnapshot::getSecondManagerId, Long.valueOf(rowId)));
            param.setThemeIds(snapshotList.stream().map(ManagerSnapshot::getThemeId).collect(Collectors.toList()));
            param.setShopIds(snapshotList.stream().map(ManagerSnapshot::getShopId).collect(Collectors.toList()));
            return;
        }
        if (ReportListTypeEnum.MANAGER_3.getValue().equals(rowType)) {
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .in(ManagerSnapshot::getThemeId, param.getThemeIds())
                    .in(ManagerSnapshot::getShopId, param.getShopIds())
                    .eq(ManagerSnapshot::getThirdManagerId, Long.valueOf(rowId)));
            param.setThemeIds(snapshotList.stream().map(ManagerSnapshot::getThemeId).collect(Collectors.toList()));
            param.setShopIds(snapshotList.stream().map(ManagerSnapshot::getShopId).collect(Collectors.toList()));
            return;
        }
        if (ReportListTypeEnum.MANAGER_4.getValue().equals(rowType)) {
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .in(ManagerSnapshot::getThemeId, param.getThemeIds())
                    .in(ManagerSnapshot::getShopId, param.getShopIds())
                    .eq(ManagerSnapshot::getFourthManagerId, Long.valueOf(rowId)));
            param.setThemeIds(snapshotList.stream().map(ManagerSnapshot::getThemeId).collect(Collectors.toList()));
            param.setShopIds(snapshotList.stream().map(ManagerSnapshot::getShopId).collect(Collectors.toList()));
            return;
        }
        if (ReportListTypeEnum.THEME.getValue().equals(rowType)) {
            param.setThemeIds(Collections.singletonList(Long.valueOf(rowId)));
            List<ManagerSnapshot> snapshotList = managerSnapshotService.list(Wrappers.<ManagerSnapshot>lambdaQuery()
                    .eq(ManagerSnapshot::getThemeId, Long.valueOf(rowId)));
            param.setThemeIds(Collections.singletonList(Long.valueOf(rowId)));
            param.setShopIds(snapshotList.stream().map(ManagerSnapshot::getShopId).collect(Collectors.toList()));
        }
    }

    /**
     * 查询趋势数据
     */
    public List<VideoPool> getVideoTrendData(VideoTrendParam param) {
        return videoPoolService.list(Wrappers.<VideoPool>lambdaQuery()
                .select(VideoPool::getId, VideoPool::getBelongUserId, VideoPool::getBelongShopId, VideoPool::getFirstManagerId,
                        VideoPool::getSecondManagerId, VideoPool::getThirdManagerId, VideoPool::getFourthManagerId,
                        VideoPool::getUv, VideoPool::getFullUv, VideoPool::getLikeNum, VideoPool::getShareNum,
                        VideoPool::getCommentNum, VideoPool::getCollectDay, VideoPool::getNewFanNum, VideoPool::getValidVideo,
                        VideoPool::getPlatform, VideoPool::getInvalidReason, VideoPool::getScore, VideoPool::getVideoId)
                .in(VideoPool::getThemeId, param.getThemeIds())
                .in(VideoPool::getBelongShopId, param.getShopIds())
                .eq(VideoPool::getStatus, 1));// 不查询调整账号导致失效的作品)
    }

    /**
     * 查询短视频报表分析短视频数据范围
     */
    public List<VideoPool> getVideoDataRange(VideoBaseQueryParam param, Integer rowType, Integer pointType,
                                             String rowId) {
        LambdaQueryWrapper<VideoPool> baseWrapper = Wrappers.<VideoPool>lambdaQuery()
                .select(VideoPool::getId, VideoPool::getBelongUserId, VideoPool::getBelongShopId,
                        VideoPool::getFirstManagerId, VideoPool::getSecondManagerId, VideoPool::getThirdManagerId,
                        VideoPool::getFourthManagerId, VideoPool::getUv, VideoPool::getFullUv, VideoPool::getLikeNum,
                        VideoPool::getShareNum, VideoPool::getCommentNum, VideoPool::getNewFanNum,
                        VideoPool::getValidVideo, VideoPool::getPlatform, VideoPool::getInvalidReason,
                        VideoPool::getScore, VideoPool::getThemeId)
                .in(VideoPool::getThemeId, param.getThemeIds())
                .in(VideoPool::getBelongShopId, param.getShopIds())
                .ge(VideoPool::getPublishTime, param.getBeginTime())
                .le(VideoPool::getPublishTime, param.getEndTime())
                .eq(VideoPool::getLatestRecord, 1)
                .eq(VideoPool::getStatus, 1);// 不查询调整账号导致失效的作品
        //如果是不合格分析入口查询，特殊处理
        if (ReportListTypeEnum.INVALID_REASON.getValue().equals(rowType)
                || ReportListTypeEnum.INVALID_REASON.getValue().equals(pointType)) {
            baseWrapper.le(VideoPool::getValidVideo, 0);
        }
        //设置分析类型
        if (PublicUtil.isNotEmpty(pointType)) {
            if (ReportListTypeEnum.SHOP.getValue().equals(rowType)) {
                baseWrapper.eq(VideoPool::getBelongShopId, Long.valueOf(rowId));
            } else if (ReportListTypeEnum.USER.getValue().equals(rowType)) {
                baseWrapper.eq(VideoPool::getBelongUserId, Long.valueOf(rowId));
            } else if (ReportListTypeEnum.MANAGER_1.getValue().equals(rowType)) {
                baseWrapper.eq(VideoPool::getFirstManagerId, Long.valueOf(rowId));
            } else if (ReportListTypeEnum.MANAGER_2.getValue().equals(rowType)) {
                baseWrapper.eq(VideoPool::getSecondManagerId, Long.valueOf(rowId));
            } else if (ReportListTypeEnum.MANAGER_3.getValue().equals(rowType)) {
                baseWrapper.eq(VideoPool::getThirdManagerId, Long.valueOf(rowId));
            } else if (ReportListTypeEnum.MANAGER_4.getValue().equals(rowType)) {
                baseWrapper.eq(VideoPool::getFourthManagerId, Long.valueOf(rowId));
            } else if (ReportListTypeEnum.PLATFORM.getValue().equals(rowType)) {
                baseWrapper.eq(VideoPool::getPlatform, Long.valueOf(rowId));
            } else if (ReportListTypeEnum.INVALID_REASON.getValue().equals(rowType)) {
                baseWrapper.eq(VideoPool::getInvalidReason, rowId);
            }
        }
        return videoPoolService.list(baseWrapper);
    }

    /**
     * 按无效原因分组视频数据
     */
    public Map<Object, List<VideoPool>> getInvalidDataMap(List<VideoPool> videoList) {
        Map<Object, List<VideoPool>> videoInvalidMap = new HashMap<>();
        videoList.forEach(video -> {
            if (PublicUtil.isNotEmpty(video.getInvalidReason())) {
                List<Integer> videoReasonList = Arrays.stream(video.getInvalidReason().split(","))
                        .mapToInt(Integer::valueOf).boxed().collect(Collectors.toList());
                for (Integer reasonKey : videoReasonList) {
                    List<VideoPool> reasonPools = videoInvalidMap.getOrDefault(reasonKey, new ArrayList<>());
                    reasonPools.add(video);
                    videoInvalidMap.put(reasonKey, reasonPools);
                }
            }
        });
        return videoInvalidMap;
    }

    /**
     * 获取短视频发布人范围
     */
    public List<ConsultantSnapshot> getThemeOrMonthPublisher(Long groupId, List<Long> themeIds, List<Long> shopIds) {
        return consultantSnapshotService.list(Wrappers.<ConsultantSnapshot>lambdaQuery()
                .eq(ConsultantSnapshot::getGroupId, groupId)
                .in(ConsultantSnapshot::getThemeId, themeIds)
                .in(PublicUtil.isNotEmpty(shopIds), ConsultantSnapshot::getShopId, shopIds)
                .groupBy(ConsultantSnapshot::getStaffId));
    }
}

