package cn.fw.dalaran.service.biz.task;

import cn.fw.dalaran.common.domain.enums.RoleMsgEnum;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.dto.AccountDTO;
import cn.fw.dalaran.rpc.erp.ErpRpc;
import cn.fw.dalaran.rpc.erp.UserRoleRpcService;
import cn.fw.dalaran.rpc.oop.OopRpc;
import cn.fw.dalaran.rpc.oop.dto.GroupDTO;
import cn.fw.dalaran.service.biz.account.AccountBizService;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.erp.sdk.api.result.UserRoleDataRange;
import cn.fw.erp.sdk.api.result.UserRoleInfo;
import cn.fw.oop.sdk.result.ShopInfoVo;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 账号人员检查实现
 *
 * @author: lym
 * @date: 2023/6/13
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class AccountStaffCheckBizService {
    private final AccountService accountService;
    private final AccountBizService accountBizService;

    /**
     * 人员销售角色变动调整账号范围
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public void checkStaffRoleRange(Account oldAccount, Map<Long, List<UserRoleDataRange>> staffMap) {
        try {
            //门店账号不处理
            if (oldAccount.getAccountType().equals(1)) {
                return;
            }
            List<UserRoleDataRange> userRoleDataRanges = staffMap.get(oldAccount.getUserId());
            //非销售顾问不处理
            if (PublicUtil.isNotEmpty(userRoleDataRanges)) {
                return;
            }
            UserRoleDataRange dataRange = userRoleDataRanges.get(0);
            //如果销售顾问调整了门店，调整账号
            if (!oldAccount.getShopId().equals(dataRange.getRangeValue())) {
                Account account = PublicUtil.copy(oldAccount, Account.class);
                account.setId(null);
                account.setCreateTime(DateUtil.localDateTime2Date(LocalDateTime.now().plusDays(-1)));
                accountBizService.deleteAccount(oldAccount.getId());
                accountService.save(account);
                //todo 推送账号变动
            }
        } catch (Exception e) {
            log.error("检查销售顾问账号情况失败" + e);
        }
    }


}
