package cn.fw.dalaran.service.biz.task;

import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.domain.enums.report.ReportListTypeEnum;
import cn.fw.dalaran.domain.param.report.score.ScoreListParam;
import cn.fw.dalaran.domain.vo.report.score.ThemeScoreListVo;
import cn.fw.dalaran.rpc.morax.SalaryRpc;
import cn.fw.dalaran.rpc.oop.OopRpc;
import cn.fw.dalaran.rpc.oop.dto.ShopDTO;
import cn.fw.dalaran.service.biz.report.score.ScoreMonthBizService;
import cn.fw.morax.sdk.dto.CustomList;
import cn.fw.morax.sdk.dto.common.IndicatorShopValueReq;
import cn.fw.morax.sdk.dto.common.IndicatorValueDTO;
import cn.fw.morax.sdk.dto.common.IndicatorValueEnum;
import cn.fw.morax.sdk.dto.common.IndicatorValueReq;
import cn.fw.morax.sdk.dto.kpi.MonthlyReportFinishReq;
import cn.fw.oop.sdk.result.ShopInfoVo;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 推送绩效
 *
 * @author: lym
 * @date: 2023/7/29
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class PushSalaryBizService {
    private final OopRpc oopRpc;
    private final SalaryRpc salaryRpc;
    private final ScoreMonthBizService scoreMonthBizService;

    /**
     * @param group
     */
    @Transactional(rollbackFor = Exception.class)
    public void pushGroupSalary(Long groupId, Date triggerDate) {
        List<ShopInfoVo> allShopsByGroup = oopRpc.getAllShopsByGroup(groupId);
        List<Long> shopIds = allShopsByGroup.stream().map(ShopInfoVo::getId).collect(Collectors.toList());
        //数据查询范围
        ScoreListParam scoreListParam = new ScoreListParam();
        scoreListParam.setGroupId(groupId);
        scoreListParam.setShopIds(shopIds);
        scoreListParam.setBeginTime(DateUtil.monthBegin(triggerDate));
        //推送人员绩效指标
        CustomList<IndicatorValueReq> userIndicatorList = new CustomList<>();
        this.getUser(groupId, triggerDate, scoreListParam, userIndicatorList);
        this.getManager1(groupId, triggerDate, scoreListParam, userIndicatorList);
        this.getManager2(groupId, triggerDate, scoreListParam, userIndicatorList);
        if(PublicUtil.isNotEmpty(userIndicatorList)){
            salaryRpc.pushIndicatorReport(userIndicatorList);
        }
        //推送门店绩效指标
        CustomList<IndicatorShopValueReq> shopIndicatorList = new CustomList<>();
        this.getShop(groupId, triggerDate, scoreListParam, shopIndicatorList);
        if(PublicUtil.isNotEmpty(shopIndicatorList)){
            salaryRpc.indicatorShopReport(shopIndicatorList);
        }
        MonthlyReportFinishReq finishReq = new MonthlyReportFinishReq();
        finishReq.setIndicatorCodes(Collections.singletonList("6574EA0766114615"));
        finishReq.setGroupId(groupId);
        finishReq.setDataDate(triggerDate);
        salaryRpc.finishIndicatorStageReport(finishReq);
    }



    /**
     * 销售顾问绩效
     */
    private void getUser(Long groupId, Date date, ScoreListParam scoreListParam, CustomList<IndicatorValueReq> userIndicatorList) {
        scoreListParam.setRowType(ReportListTypeEnum.USER.getValue());
        List<ThemeScoreListVo> scoreAnalysis = scoreMonthBizService.getScoreAnalysis(scoreListParam);
        for (ThemeScoreListVo analysis : scoreAnalysis) {
            userIndicatorList.add(this.buildParam(groupId, analysis, date));
        }
    }

    /**
     * 一级管理人员
     */
    private void getManager1(Long groupId, Date date, ScoreListParam scoreListParam, CustomList<IndicatorValueReq> userIndicatorList) {
        scoreListParam.setRowType(ReportListTypeEnum.MANAGER_1.getValue());
        List<ThemeScoreListVo> scoreAnalysis = scoreMonthBizService.getScoreAnalysis(scoreListParam);
        for (ThemeScoreListVo analysis : scoreAnalysis) {
            userIndicatorList.add(this.buildParam(groupId, analysis, date));
        }
    }

    /**
     * 二级管理人员
     */

    private void getManager2(Long groupId, Date date, ScoreListParam scoreListParam, CustomList<IndicatorValueReq> userIndicatorList) {
        scoreListParam.setRowType(ReportListTypeEnum.MANAGER_2.getValue());
        List<ThemeScoreListVo> scoreAnalysis = scoreMonthBizService.getScoreAnalysis(scoreListParam);
        for (ThemeScoreListVo analysis : scoreAnalysis) {
            userIndicatorList.add(this.buildParam(groupId, analysis, date));
        }
    }

    private IndicatorValueReq buildParam(Long groupId, ThemeScoreListVo analysis, Date date) {
        //构建推送参数
        IndicatorValueReq indicatorValueReq = new IndicatorValueReq();
        indicatorValueReq.setGroupId(groupId);
        indicatorValueReq.setUserId(Long.valueOf(analysis.getId()));
        indicatorValueReq.setDataDate(date);
        indicatorValueReq.setIndicatorCode("6574EA0766114615");
        indicatorValueReq.setValueType(IndicatorValueEnum.RATIO.getValue());
        //指标值
        IndicatorValueDTO indicatorValueDTO = new IndicatorValueDTO();
        indicatorValueDTO.setKey(indicatorValueReq.getIndicatorCode());
        indicatorValueDTO.setValue(analysis.getI1());
        indicatorValueReq.setIndicatorValue(Collections.singletonList(indicatorValueDTO));
        return indicatorValueReq;
    }


    private void getShop(Long groupId, Date triggerDate, ScoreListParam scoreListParam, CustomList<IndicatorShopValueReq> shopIndicatorList) {
        scoreListParam.setRowType(ReportListTypeEnum.SHOP.getValue());
        List<ThemeScoreListVo> scoreAnalysis = scoreMonthBizService.getScoreAnalysis(scoreListParam);
        for (ThemeScoreListVo analysis : scoreAnalysis) {
            shopIndicatorList.add(this.buildShopParam(groupId, analysis, triggerDate));
        }
    }

    private IndicatorShopValueReq buildShopParam(Long groupId, ThemeScoreListVo analysis, Date date) {
        //构建推送参数
        IndicatorShopValueReq shopValueReq = new IndicatorShopValueReq();
        shopValueReq.setGroupId(groupId);
        shopValueReq.setShopId(Long.valueOf(analysis.getId()));
        shopValueReq.setDataDate(date);
        shopValueReq.setIndicatorCode("6574EA0766114615");
        shopValueReq.setValueType(IndicatorValueEnum.RATIO.getValue());
        //指标值
        IndicatorValueDTO indicatorValueDTO = new IndicatorValueDTO();
        indicatorValueDTO.setKey(shopValueReq.getIndicatorCode());
        indicatorValueDTO.setValue(analysis.getI1());
        shopValueReq.setIndicatorValue(Collections.singletonList(indicatorValueDTO));
        return shopValueReq;
    }

}
