package cn.fw.dalaran.service.biz.task;

import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.service.biz.report.calculate.CalThemeScoreBizService;
import cn.fw.dalaran.service.biz.report.calculate.ThemeSnapshotTaskBizService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.List;

/**
 * 报表定时任务处理
 *
 * @author: lym
 * @date: 2023/5/12
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ReportTaskBizService {
    private final ActivityThemeService activityThemeService;
    private final CalThemeScoreBizService calThemeScoreBizService;
    private final ThemeSnapshotTaskBizService themeSnapshotTaskBizService;

    /**
     * 定时生成活动快照
     */
    @Transactional(rollbackFor = Exception.class)
    public void createThemeSnapshot() {
        List<ActivityTheme> themeList = activityThemeService.list(Wrappers.<ActivityTheme>lambdaQuery()
                .le(ActivityTheme::getStartTime, LocalDateTime.now())
                .ge(ActivityTheme::getEndTime, LocalDateTime.now()));
        themeList.forEach(themeSnapshotTaskBizService::createThemeSnapshot);
    }

    /**
     * 计算主题单场得分
     */
    @Transactional(rollbackFor = Exception.class)
    public void handleThemeScoreCal() {
        List<ActivityTheme> themeList = activityThemeService.list(Wrappers.<ActivityTheme>lambdaQuery()
                .ge(ActivityTheme::getEndTime, LocalDateTime.of(LocalDate.now().minusDays(3), LocalTime.MIN)));
        for (ActivityTheme theme : themeList) {
            calThemeScoreBizService.calVideoOrLiveScore(theme);
        }
    }

    /**
     * 补全当天未上报数据
     */
    @Transactional(rollbackFor = Exception.class)
    public void syncPreDayVideo() {
//        List<ActivityTheme> themeList = activityThemeService.list(Wrappers.<ActivityTheme>lambdaQuery()
//                .ge(ActivityTheme::getEndTime, LocalDateTime.of(LocalDate.now().minusDays(3), LocalTime.MIN)));
//        List<ActivityTheme> themeList = this.getThemeList(time);
//        themeList.forEach(themeSnapshotTaskBizService::syncPreDayVideo);
    }
}
