package cn.fw.dalaran.service.data;

import cn.fw.dalaran.domain.db.LivePool;
import cn.fw.dalaran.domain.dto.report.score.StaffScoreSumDto;
import com.baomidou.mybatisplus.extension.service.IService;

import java.time.LocalDate;
import java.util.List;
import java.util.Map;

/**
 * @author kurisu
 * @date 2021-11-17 14:54
 * @description 直播池service
 */
public interface LivePoolService extends IService<LivePool> {
    /**
     * 根据账号删除对应时间的数据
     *
     * @param accountId
     * @param date
     */
    void removeAccountDataByDate(Long accountId, LocalDate date);

    /**
     * 获取主题活动员工有效直播数
     *
     * @param userId  用户id
     * @param themeId 活动主题id
     * @return 有效直播数
     */
    Integer getValidLiveCnt(Long userId, Long themeId);

    /**
     * 设置最佳直播
     *
     * @param liveId 直播数据id
     * @return
     */
    boolean markBestLive(Long liveId);

    /**
     * 查询综合得分总览数据
     *
     * @param themeIds
     * @param shopIds
     * @return
     */
    List<StaffScoreSumDto> getScoreSummary(List<Long> themeIds, List<Long> shopIds);

    /**
     * 查询综合得分趋势
     *
     * @param themeIds
     * @param shopIds
     * @param dateRange
     * @return
     */
    Map<LocalDate, List<StaffScoreSumDto>> getScoreSummaryTrend(List<Long> themeIds, List<Long> shopIds, List<LocalDate> dateRange);
}
