package cn.fw.dalaran.service.data;

import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.dto.report.VideoTrendDto;
import cn.fw.dalaran.domain.dto.report.score.StaffScoreSumDto;
import cn.fw.dalaran.domain.param.report.video.VideoBaseQueryParam;
import cn.fw.dalaran.domain.param.report.video.VideoTrendParam;
import cn.fw.dalaran.domain.vo.report.video.VideoSummaryVo;
import com.baomidou.mybatisplus.extension.service.IService;
import org.springframework.scheduling.annotation.Async;

import java.time.LocalDate;
import java.util.List;
import java.util.Map;

/**
 * @author kurisu
 * @date 2021-11-17 14:54
 * @description 短视频池service
 */
public interface VideoPoolService extends IService<VideoPool> {

    /**
     * 根据账号删除对应时间的数据
     *
     * @param accountId
     * @param date
     */
    void removeAccountDataByDate(Long accountId, LocalDate date);

    /**
     * 获取主题活动员工有效视频数
     *
     * @param userId  用户id
     * @param themeId 活动主题id
     * @return 有效视频数
     */
    Integer getValidVideoCnt(Long userId, Long themeId);

    /**
     * 找到最新的video信息
     *
     * @param videoId 视频id
     * @return 该视频id对应的最新的视频
     */
    VideoPool getLatestVideo(String videoId);

    /**
     * 获取最近2个月视频归属情况
     *
     * @return 最近2个月视频归属情况   K: 视频id, V: 视频详情
     */
    Map<String, VideoPool> getRecent2MonthVideo();

    /**
     * 获取最近2个月被标记无效视频id列表
     *
     * @return 最近2个月被标记无效视频id列表
     */
    List<String> getRecent2MonthInvalidStatusVideoIds();

    /**
     * 根据视频id标记该视频为老数据
     *
     * @param videoId 视频id
     */
    void markVideoIsOld(String videoId);

    /**
     * 查询综合得分总览数据
     *
     * @param themeIds
     * @param shopIds
     * @return
     */
    List<StaffScoreSumDto> getScoreSummary(List<Long> themeIds, List<Long> shopIds);

    /**
     * 查询综合得分趋势数据
     *
     * @param themeIds
     * @param shopIds
     * @return
     */
    Map<LocalDate, List<StaffScoreSumDto>> getScoreSummaryTrend(List<Long> themeIds, List<Long> shopIds);



}
