package cn.fw.dalaran.service.data.impl;

import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.dao.AccountDao;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import cn.fw.dalaran.service.data.AccountService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author kurisu
 * @date 2021-11-17 14:54
 * @description AccountService
 */
@Service
@RequiredArgsConstructor
public class AccountServiceImpl extends ServiceImpl<AccountDao, Account> implements AccountService {

    private final AccountDao accountDao;

    /**
     * 修改账户
     *
     * @param param 待修改字段值封装
     */
    @Override
    public boolean alertAccount(Account param) {
        return accountDao.alertAccount(param);
    }

    /**
     * 根据用户id获取账户
     *
     * @param userId 用户id
     * @return 对应账户
     */
    @Override
    public List<Account> getByUserId(Long userId) {
        return accountDao.getByUserId(userId);
    }

    /**
     * 根据账户号获取账户列表(忽略逻辑删除)
     *
     * @param accountNo 账户号
     * @return 对应账户列表
     */
    @Override
    public List<Account> getByAccountNoIgnoreLogicDel(String accountNo) {
        return accountDao.getByAccountNoIgnoreLogicDel(accountNo);
    }

    /**
     * 根据id获取账户(忽略逻辑删除)
     *
     * @param accountId 账户id
     * @return 对应账户
     */
    @Override
    public Account getByIdIgnoreLogicDel(Long accountId) {
        return accountDao.getByIdIgnoreLogicDel(accountId);
    }

    /**
     * 根据ids获取账户列表(忽略逻辑删除)
     *
     * @param accountIds 账户ids
     * @return 对应账户列表
     */
    @Override
    public List<Account> getByIdsIgnoreLogicDel(List<Long> accountIds) {
        return accountDao.getByIdsIgnoreLogicDel(accountIds);
    }

    /**
     * 获取账户列表(忽略逻辑删除)
     *
     * @return 对应账户列表
     */
    @Override
    public List<Account> getAllIgnoreLogicDel() {
        return accountDao.getAllIgnoreLogicDel();
    }

    @Override
    public List<Account> getByAccountNo(Long groupId, String account, PlatformEnum type) {
        return this.list(Wrappers.<Account>lambdaQuery()
                .eq(Account::getGroupId, groupId)
                .eq(Account::getAccount, account)
                .eq(Account::getType, type));
    }

}
