package cn.fw.dalaran.service.data.impl;

import cn.fw.dalaran.common.ConfigProperties;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.ImageUtils;
import cn.fw.dalaran.dao.config.ActivityThemeDao;
import cn.fw.dalaran.domain.db.config.ActivityTheme;
import cn.fw.dalaran.domain.vo.ActivityThemeVo;
import cn.fw.dalaran.rpc.erp.ErpTaskRpc;
import cn.fw.dalaran.service.Common;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.dalaran.service.data.ConfigGroupService;
import cn.fw.dalaran.service.data.ThemeFileService;
import cn.fw.dalaran.service.data.ValidConfigNewService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2022/1/8 17:30
 * @Description
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ActivityThemeServiceImpl extends ServiceImpl<ActivityThemeDao, ActivityTheme> implements ActivityThemeService {

    private final ConfigProperties configProperties;
    private final ActivityThemeDao activityThemeDao;
    private final ConfigGroupService configGroupService;
    private final ThemeFileService themeFileService;
    private final ValidConfigNewService validConfigNewService;
    private final ErpTaskRpc erpTaskRpc;
    private final Common common;


    /**
     * 验证门店是否重合
     *
     * @param shopIds1
     * @param shopIds2
     * @return
     */
    public boolean validShopOverlap(List<Long> shopIds1, List<Long> shopIds2) {
        if (Objects.equals(shopIds1.get(0), -1L) || Objects.equals(shopIds2.get(0), -1L)) {
            return true;
        } else {
            shopIds1.retainAll(shopIds2);
            return !shopIds1.isEmpty();
        }
    }

    /**
     * 验证主题是否结束数据抓取
     *
     * @param themeEndTime 主题结束时间
     * @param delayDays    延期天数
     * @return 主题是否结束数据抓取
     */
    @Override
    public boolean themeHaveStopCatchData(Date themeEndTime, int delayDays) {
        long timeSub = System.currentTimeMillis() - themeEndTime.getTime();
        return timeSub > (1 + delayDays) * 24 * 3600 * 1000L + 1000;
    }

    /**
     * 查询集团话题列表
     *
     * @param groupId       集团id
     * @param configGroupId 所属配置组id
     * @return
     */
    @Override
    public List<ActivityThemeVo> list(Long groupId, Long configGroupId) {
        return this.lambdaQuery()
                .eq(ActivityTheme::getGroupId, groupId)
                .eq(ActivityTheme::getConfigGroupId, configGroupId)
                .orderByDesc(ActivityTheme::getId)
                .list()
                .stream()
                .map(ActivityTheme::toVO)
                .collect(Collectors.toList());
    }

    /**
     * 活动主题数据封面图处理
     */
    public static void main(String[] args) {
        int themeId = 666;
        String tempDir = System.getProperty("java.io.tmpdir");// 获取当前环境临时文件夹路径
        Arrays.asList("440813334994949",
                "018cab7b941745498ebff4a0821f92e4",
                "0f9369e24ee84401b2817305b9bfc0ec",
                "60bb3211983244f9bfc01c44745d64c0",
                "a78039c905034762b55e4abc2e1ded37",
                "be8125bf2be6474c8973b5c09498ca66")
                .forEach(item -> {
                    try {
                        File downloadFile = ImageUtils.convertFileByUrl("https://gate.feewee.cn/file/show?fid=" + item, item);
                        ImageUtils.modifyResolution(downloadFile.getPath(),
                                tempDir + "activityTheme" + File.separator + themeId /*+ "#" + activityTheme.getTheme()*/ + File.separator + "settingCover",
                                "fw_theme_cover_" + item, 512, 512);
                        boolean result = downloadFile.delete();
                    } catch (Exception e) {
                        log.error("下载转换封面图失败", e);
                    }
                })
        ;
    }

    /**
     * 获取已经逻辑删除的主题列表
     *
     * @return 已经逻辑删除的主题列表
     */
    @Override
    public List<ActivityTheme> getHasDelActivityTheme() {
        return activityThemeDao.getHasDelActivityTheme();
    }

    /**
     * 获取还在进行中的活动主题列表
     *
     * @return 还在进行中的活动主题列表
     */
    @Override
    public List<ActivityTheme> getGoingActivityTheme() {
        return this.lambdaQuery()
                .ge(ActivityTheme::getEndTime, new Date())// 因为抓数据延迟了1天
                .list();
//        return this.lambdaQuery()
//                .ge(ActivityTheme::getEndTime, DateUtil.minusDays(DateUtil.dayEnd(new Date()), 1))// 因为抓数据延迟了1天
//                .list();
    }

}
