package cn.fw.dalaran.service.data.impl;

import cn.fw.dalaran.dao.ValidConfigNewDao;
import cn.fw.dalaran.domain.db.config.ValidConfigNew;
import cn.fw.dalaran.domain.enums.config.ValidConfigTypeEnum;
import cn.fw.dalaran.domain.enums.config.ValidLiveEnum;
import cn.fw.dalaran.domain.enums.config.ValidVideoEnum;
import cn.fw.dalaran.domain.enums.config.WorksTypeEnum;
import cn.fw.dalaran.domain.vo.config.ValidConfigNewVo;
import cn.fw.dalaran.service.data.ValidConfigNewService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;


/**
 * @author wmy3969
 * @version 1.0
 * @date 2022-02-15 12:37:25
 * @Description
 */
@Service("validConfigNewService")
@RequiredArgsConstructor
public class ValidConfigNewServiceImpl extends ServiceImpl<ValidConfigNewDao, ValidConfigNew> implements ValidConfigNewService {

    private final ApplicationEventPublisher publisher;


    @Override
    public int insertBatchSomeColumn(List<ValidConfigNew> list) {
        return baseMapper.insertBatchSomeColumn(list);
    }

    @Override
    public int updateBatch(List<ValidConfigNew> list) {
        return baseMapper.updateBatch(list);
    }

    /**
     * 查询集团直播/短视频配置
     *
     * @param configGroupId 所属配置组id
     * @param type          类型id
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ValidConfigNewVo queryList(Long configGroupId, Integer type) {
        List<ValidConfigNew> list;
        list = this.lambdaQuery()
                .eq(ValidConfigNew::getConfigGroupId, configGroupId)
                .eq(ValidConfigNew::getType, type)
                .list();// 获取数据库中的配置列表
        if (Objects.equals(ValidConfigTypeEnum.VIDEO.getValue(), type)) {
            int videoConfigLength = ValidVideoEnum.values().length;// 获取枚举中规定的视频指标数
            if (list.size() != videoConfigLength) {
                list = this.initVideoConfigList(list, configGroupId);
            }
        } else if (Objects.equals(ValidConfigTypeEnum.LIVE.getValue(), type)) {
            int liveConfigLength = ValidLiveEnum.values().length;// 获取枚举中规定的直播指标数
            if (list.size() != liveConfigLength) {
                list = this.initLiveConfigList(list, configGroupId);
            }
        }
//        else if (Objects.equals(3, type)) {
//            if (list.size() != 1) {
//                list.add(ValidConfigNew.createConfig(-1, configGroupId, type, -1, true));
//                this.saveBatch(list);
//            }
//        }
        return ValidConfigNew.toVO(list);
    }

    /**
     * 根据枚举初始化视频最低标准配置
     *
     * @param list          老配置列表
     * @param configGroupId 所属配置组id
     * @return 新配置列表
     */
    @Transactional(rollbackFor = Exception.class)
    public List<ValidConfigNew> initVideoConfigList(List<ValidConfigNew> list, Long configGroupId) {
        if (!list.isEmpty())
            this.removeByIds(list.stream()
                    .map(ValidConfigNew::getId)
                    .collect(Collectors.toList()));// 删除原来配置
        List<ValidConfigNew> newConfigList = Arrays.stream(ValidVideoEnum.values())
                .map(item -> {
                    Integer minValue = 0;
                    boolean valid = false;
                    Optional<ValidConfigNew> first = list.stream()
                            .filter(item1 -> Objects.equals(item.getValue(), item1.getIndexType()))
                            .findFirst();
                    if (first.isPresent()) {
                        ValidConfigNew oldConfig = first.get();
                        minValue = oldConfig.getMinValue();
                        valid = oldConfig.getValid();
                    }
                    return ValidConfigNew.createConfig(item.getValue(), configGroupId, WorksTypeEnum.VIDEO.getValue(),
                            minValue, valid);
                })
                .collect(Collectors.toList());
        this.saveBatch(newConfigList);
        return newConfigList;
    }

    /**
     * 根据枚举初始化直播最低标准配置
     *
     * @param list          老配置列表
     * @param configGroupId 所属配置组id
     * @return 新配置列表
     */
    @Transactional(rollbackFor = Exception.class)
    public List<ValidConfigNew> initLiveConfigList(List<ValidConfigNew> list, Long configGroupId) {
        if (!list.isEmpty())
            this.removeByIds(list.stream()
                    .map(ValidConfigNew::getId)
                    .collect(Collectors.toList()));// 删除原来配置
        List<ValidConfigNew> newConfigList = Arrays.stream(ValidLiveEnum.values())
                .map(item -> {
                    Integer minValue = 0;
                    boolean valid = false;
                    if (Objects.equals(item.getValue(), -1))// 付费推广判定倍率(默认为1倍)
                        minValue = 1;
                    Optional<ValidConfigNew> first = list.stream()
                            .filter(item1 -> Objects.equals(item.getValue(), item1.getIndexType()))
                            .findFirst();
                    if (first.isPresent()) {
                        ValidConfigNew oldConfig = first.get();
                        minValue = oldConfig.getMinValue();
                        valid = oldConfig.getValid();
                    }
                    return ValidConfigNew.createConfig(item.getValue(), configGroupId, WorksTypeEnum.LIVE.getValue(),
                            minValue, valid);
                })
                .collect(Collectors.toList());
        this.saveBatch(newConfigList);
        return newConfigList;
    }
}
