package example;

import cn.fw.dalaran.common.utils.ImageUtils;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

public class ImageListener implements ActionListener {
    private Graphics graphics = null;

    int[][] imagePixArr1;
    int[][] imagePixArr2;

    // set
    public void setGraphics(Graphics graphics) {
        this.graphics = graphics;
    }

    /**
     * 无参构造方法
     *
     * @throws Exception
     */
    public ImageListener() throws Exception {
    }

    /**
     * 初始化图片监听程序(构造方法)
     *
     * @param image1Url 图片1地址(URL网址)
     * @param image2Url 图片2地址(URL网址)
     * @param outputDir 图片修改分辨率后文件存储地址
     * @param width     修改分辨率后图片宽度
     * @param height    修改分辨率后图片高度
     * @throws Exception
     */
    public ImageListener(String image1Url, String image2Url, String outputDir,
                         Integer width, Integer height) throws Exception {
        File imageFile1 = ImageUtils.convertFileByUrl(image1Url);
        File imageFile2 = ImageUtils.convertFileByUrl(image2Url);
        imagePixArr1 = ImageUtils.readImagePixel(new File(ImageUtils.modifyResolution(imageFile1.getPath(),
                outputDir, width, height)));
        imagePixArr2 = ImageUtils.readImagePixel(new File(ImageUtils.modifyResolution(imageFile2.getPath(),
                outputDir, width, height)));
    }

    /**
     * 初始化图片监听程序(构造方法)
     *
     * @param image1Url  图片1地址(URL网址)
     * @param image2File 图片2文件(本地文件)
     * @param outputDir  图片修改分辨率后文件存储地址
     * @param width      修改分辨率后图片宽度
     * @param height     修改分辨率后图片高度
     * @throws Exception
     */
    public ImageListener(String image1Url, File image2File, String outputDir,
                         Integer width, Integer height) throws Exception {
        File imageFile1 = ImageUtils.convertFileByUrl(image1Url);
        imagePixArr1 = ImageUtils.readImagePixel(new File(ImageUtils.modifyResolution(imageFile1.getPath(),
                outputDir, width, height)));
        imagePixArr2 = ImageUtils.readImagePixel(new File(ImageUtils.modifyResolution(image2File.getPath(),
                outputDir, width, height)));
    }

    /**
     * 初始化图片监听程序(构造方法)
     *
     * @param image1File 图片1文件(本地文件)
     * @param image2File 图片2文件(本地文件)
     * @param outputDir  图片修改分辨率后文件存储地址
     * @param width      修改分辨率后图片宽度
     * @param height     修改分辨率后图片高度
     * @throws Exception
     */
    public ImageListener(File image1File, File image2File, String outputDir,
                         Integer width, Integer height) throws Exception {
        imagePixArr1 = ImageUtils.readImagePixel(new File(ImageUtils.modifyResolution(image1File.getPath(),
                outputDir, width, height)));
        imagePixArr2 = ImageUtils.readImagePixel(new File(ImageUtils.modifyResolution(image2File.getPath(),
                outputDir, width, height)));
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        // 先获取按钮上的字符串 用来判断调用什么代码
        String buttonStr = e.getActionCommand();
        if (buttonStr.equals("加载图片")) {
            ImageUtils.drawImage(graphics, imagePixArr1, 30, 100, null);
            ImageUtils.drawImage(graphics, imagePixArr2, 774, 100, null);
        } else if (buttonStr.equals("开始匹配")) {
            ImageUtils.calcSimilarity(ImageUtils.getFingerprint(imagePixArr1), ImageUtils.getFingerprint(imagePixArr2));
        }
    }
}
